@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2013 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,25 +19,33 @@
 **/
 
 
-require_once dirname(__FILE__).'/include/func.inc.php';
-require_once dirname(__FILE__).'/include/classes/class.chttp_request.php';
+header('Access-Control-Allow-Origin: *');
+header('Access-Control-Allow-Headers: Content-Type');
+header('Access-Control-Allow-Methods: POST');
+header('Access-Control-Max-Age: 1000');
 
-$allowed_content = array(
+if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
+	return;
+}
+
+require_once dirname(__FILE__).'/include/func.inc.php';
+require_once dirname(__FILE__).'/include/classes/core/CHttpRequest.php';
+
+$allowed_content = [
 	'application/json-rpc' => 'json-rpc',
 	'application/json' => 'json-rpc',
 	'application/jsonrequest' => 'json-rpc',
-);
-$http_request = new CHTTP_request();
+];
+$http_request = new CHttpRequest();
 $content_type = $http_request->header('Content-Type');
 $content_type = explode(';', $content_type);
 $content_type = $content_type[0];
 
 if (!isset($allowed_content[$content_type])) {
 	header('HTTP/1.0 412 Precondition Failed');
-	exit();
+	return;
 }
 
-
 require_once dirname(__FILE__).'/include/classes/core/Z.php';
 
 header('Content-Type: application/json');
@@ -46,22 +54,27 @@ $data = $http_request->body();
 try {
 	Z::getInstance()->run(ZBase::EXEC_MODE_API);
 
-	$jsonRpc = new CJSONrpc($data);
+	$apiClient = API::getWrapper()->getClient();
+
+	// unset wrappers so that calls between methods would be made directly to the services
+	API::setWrapper();
+
+	$jsonRpc = new CJsonRpc($apiClient, $data);
 	echo $jsonRpc->execute();
 }
 catch (Exception $e) {
 	// decode input json request to get request's id
 	$jsonData = CJs::decodeJson($data);
 
-	$response = array(
+	$response = [
 		'jsonrpc' => '2.0',
-		'error' => array(
+		'error' => [
 			'code' => 1,
-			'message' => $e->getMessage()
-		),
+			'message' => $e->getMessage(),
+			'data' => ''
+		],
 		'id' => (isset($jsonData['id']) ? $jsonData['id'] : null)
-	);
+	];
 
 	echo CJs::encodeJson($response);
 }
-
