@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -77,7 +77,9 @@ class CControllerMediatypeList extends CController {
 			'uncheck' => $this->hasInput('uncheck'),
 			'sort' => $sortField,
 			'sortorder' => $sortOrder,
-			'filter' => $filter
+			'filter' => $filter,
+			'profileIdx' => 'web.media_types.filter',
+			'active_tab' => CProfile::get('web.media_types.filter.active', 1)
 		];
 
 		// get media types
