@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ class CControllerReportStatus extends CController {
 	protected function doAction() {
 		// no data is passed to the view
 		$response = new CControllerResponseData([]);
-		$response->setTitle(_('Status of Zabbix'));
+		$response->setTitle(_('System information'));
 		$this->setResponse($response);
 	}
 }
