@@ -23,39 +23,43 @@ $this->addJsFile('gtlc.js');
 $this->addJsFile('flickerfreescreen.js');
 $this->addJsFile('class.svg.canvas.js');
 $this->addJsFile('class.svg.map.js');
+$this->addJsFile('layout.mode.js');
 
 (new CWidget())
 	->setTitle(_('Maps'))
-	->setControls(
+	->setWebLayoutMode(CView::getLayoutMode())
+	->setControls(new CList([
 		(new CForm('get'))
 			->cleanItems()
 			->addVar('action', 'map.view')
 			->addVar('sysmapid', $data['map']['sysmapid'])
-			->addVar('fullscreen', $data['fullscreen'])
-			->addItem(
-				(new CList())
-					->addItem([
-						new CLabel(_('Minimum severity'), 'severity_min'),
-						(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-						$data['pageFilter']->getSeveritiesMinCB()
-					])
-					->addItem($data['map']['editable']
-						? (new CButton('edit', _('Edit map')))
-							->onClick('redirect("sysmap.php?sysmapid='.$data['map']['sysmapid'].'")')
-						: null
-					)
-					->addItem(get_icon('favourite', [
-						'fav' => 'web.favorite.sysmapids',
-						'elname' => 'sysmapid',
-						'elid' => $data['map']['sysmapid']
-					]))
-					->addItem(get_icon('fullscreen', ['fullscreen' => $data['fullscreen']]))
+			->setAttribute('aria-label', _('Main filter'))
+			->addItem((new CList())
+				->addItem([
+					new CLabel(_('Minimum severity'), 'severity_min'),
+					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
+					$data['pageFilter']->getSeveritiesMinCB()
+				])
+			),
+		(new CTag('nav', true, (new CList())
+			->addItem($data['map']['editable']
+				? new CRedirectButton(_('Edit map'), (new CUrl('sysmap.php'))
+					->setArgument('sysmapid', $data['map']['sysmapid'])
+					->getUrl()
+				)
+				: null
 			)
-	)
-	->addItem(
-		get_header_sysmap_table($data['map']['sysmapid'], $data['map']['name'], $data['fullscreen'],
-			$data['severity_min']
-		)
+			->addItem(get_icon('favourite', [
+				'fav' => 'web.favorite.sysmapids',
+				'elname' => 'sysmapid',
+				'elid' => $data['map']['sysmapid']
+			]))
+			->addItem(get_icon('fullscreen'))
+		))
+			->setAttribute('aria-label', _('Content controls'))
+	]))
+	->setBreadcrumbs(
+		get_header_sysmap_table($data['map']['sysmapid'], $data['map']['name'], $data['severity_min'])
 	)
 	->addItem(
 		(new CDiv())
@@ -72,8 +76,7 @@ $this->addJsFile('class.svg.map.js');
 						'resourceid' => $data['map']['sysmapid'],
 						'width' => null,
 						'height' => null,
-						'severity_min' => $data['severity_min'],
-						'fullscreen' => $data['fullscreen']
+						'severity_min' => $data['severity_min']
 					]
 				])->get()
 			)
