@@ -20,15 +20,12 @@
 
 
 $back_url = (new CUrl('zabbix.php'))->setArgument('action', 'dashboard.view');
+
 $table = getTriggersOverview($data['hosts'], $data['triggers'], $back_url->getUrl(), $data['style']);
 
 $output = [
 	'header' => $data['name'],
-	'body' => $table->toString(),
-	'footer' => (new CList())
-		->addItem(_s('Updated: %s', zbx_date2str(TIME_FORMAT_SECONDS)))
-		->addClass(ZBX_STYLE_DASHBRD_WIDGET_FOOT)
-		->toString()
+	'body' => $table->toString()
 ];
 
 if (($messages = getMessages()) !== null) {
