@@ -28,25 +28,24 @@ require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'type' =>           [T_ZBX_INT, O_OPT, null,	IN([GRAPH_TYPE_NORMAL, GRAPH_TYPE_STACKED]), null],
-	'itemids' =>		[T_ZBX_INT, O_MAND, P_SYS,	DB_ID,		null],
-	'period' =>			[T_ZBX_INT, O_OPT, P_NZERO,	BETWEEN(ZBX_MIN_PERIOD, ZBX_MAX_PERIOD), null],
-	'stime' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
-	'isNow' =>			[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null],
-	'profileIdx' =>		[T_ZBX_STR, O_OPT, null,	null,		null],
-	'profileIdx2' =>	[T_ZBX_STR, O_OPT, null,	null,		null],
-	'updateProfile' =>	[T_ZBX_STR, O_OPT, null,	null,		null],
-	'width' =>			[T_ZBX_INT, O_OPT, null,	BETWEEN(CLineGraphDraw::GRAPH_WIDTH_MIN, 65535),	null],
-	'height' =>			[T_ZBX_INT, O_OPT, null,	BETWEEN(CLineGraphDraw::GRAPH_HEIGHT_MIN, 65535),	null],
-	'outer' =>			[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null],
-	'batch' =>			[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null],
-	'onlyHeight' =>		[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null],
-	'legend' =>			[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null],
-	'widget_view' =>	[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null]
+	'type' =>           [T_ZBX_INT,			O_OPT, null,	IN([GRAPH_TYPE_NORMAL, GRAPH_TYPE_STACKED]), null],
+	'itemids' =>		[T_ZBX_INT,			O_MAND, P_SYS,	DB_ID,		null],
+	'from' =>			[T_ZBX_RANGE_TIME,	O_OPT, P_SYS,	null,		null],
+	'to' =>				[T_ZBX_RANGE_TIME,	O_OPT, P_SYS,	null,		null],
+	'profileIdx' =>		[T_ZBX_STR,			O_OPT, null,	null,		null],
+	'profileIdx2' =>	[T_ZBX_STR,			O_OPT, null,	null,		null],
+	'width' =>			[T_ZBX_INT,			O_OPT, null,	BETWEEN(CLineGraphDraw::GRAPH_WIDTH_MIN, 65535),	null],
+	'height' =>			[T_ZBX_INT,			O_OPT, null,	BETWEEN(CLineGraphDraw::GRAPH_HEIGHT_MIN, 65535),	null],
+	'outer' =>			[T_ZBX_INT,			O_OPT, null,	IN('0,1'),	null],
+	'batch' =>			[T_ZBX_INT,			O_OPT, null,	IN('0,1'),	null],
+	'onlyHeight' =>		[T_ZBX_INT,			O_OPT, null,	IN('0,1'),	null],
+	'legend' =>			[T_ZBX_INT,			O_OPT, null,	IN('0,1'),	null],
+	'widget_view' =>	[T_ZBX_INT,			O_OPT, null,	IN('0,1'),	null]
 ];
 if (!check_fields($fields)) {
 	exit();
 }
+validateTimeSelectorPeriod(getRequest('from'), getRequest('to'));
 
 $itemIds = getRequest('itemids');
 
@@ -83,18 +82,16 @@ CArrayHelper::sort($items, ['name', 'hostname', 'itemid']);
 /*
  * Display
  */
-$timeline = calculateTime([
-	'profileIdx' => getRequest('profileIdx', 'web.screens'),
+$timeline = getTimeSelectorPeriod([
+	'profileIdx' => getRequest('profileIdx'),
 	'profileIdx2' => getRequest('profileIdx2'),
-	'updateProfile' => (getRequest('updateProfile', '0') === '1'),
-	'period' => getRequest('period'),
-	'stime' => getRequest('stime'),
-	'isNow' => getRequest('isNow')
+	'from' => getRequest('from'),
+	'to' => getRequest('to')
 ]);
 
 $graph = new CLineGraphDraw(getRequest('type'));
-$graph->setPeriod($timeline['period']);
-$graph->setSTime($timeline['stime']);
+$graph->setPeriod($timeline['to_ts'] - $timeline['from_ts']);
+$graph->setSTime($timeline['from_ts']);
 $graph->showLegend(getRequest('legend', 1));
 
 // change how the graph will be displayed if more than one item is selected
@@ -144,7 +141,7 @@ if ($min_dimentions['height'] > $graph->getHeight()) {
 
 if (getRequest('onlyHeight', '0') === '1') {
 	$graph->drawDimensions();
-	$height = $graph->getHeight();
+	$height = $graph->getHeight() + 1;
 
 	if (getRequest('widget_view') === '1') {
 		$height = $height - CLineGraphDraw::DEFAULT_TOP_BOTTOM_PADDING;
