@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,12 +20,14 @@
 
 
 require_once dirname(__FILE__).'/include/config.inc.php';
+require_once dirname(__FILE__).'/include/hostgroups.inc.php';
 require_once dirname(__FILE__).'/include/hosts.inc.php';
 require_once dirname(__FILE__).'/include/graphs.inc.php';
 require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = isset($_REQUEST['parent_discoveryid']) ? _('Configuration of graph prototypes') : _('Configuration of graphs');
 $page['file'] = 'graphs.php';
+$page['scripts'] = ['colorpicker.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -95,7 +97,7 @@ $_REQUEST['show_legend'] = getRequest('show_legend', 0);
  * Permissions
  */
 $groupId = getRequest('groupid');
-if ($groupId && !API::HostGroup()->isWritable([$groupId])) {
+if ($groupId && !isWritableHostGroups([$groupId])) {
 	access_deny();
 }
 
@@ -138,17 +140,8 @@ elseif (hasRequest('graphid')) {
 		access_deny();
 	}
 }
-elseif ($hostId) {
-	// check whether host is editable by user
-	$host = (bool) API::Host()->get([
-		'output' => [],
-		'hostids' => $hostId,
-		'templated_hosts' => true,
-		'editable' => true
-	]);
-	if (!$host) {
-		access_deny();
-	}
+elseif ($hostId && !isWritableHostTemplates([$hostId])) {
+	access_deny();
 }
 
 /*
@@ -398,9 +391,6 @@ $pageFilter = new CPageFilter([
 ]);
 
 if (empty($_REQUEST['parent_discoveryid'])) {
-	if ($pageFilter->groupid > 0) {
-		$groupId = $pageFilter->groupids;
-	}
 	if ($pageFilter->hostid > 0) {
 		$hostId = $pageFilter->hostid;
 	}
@@ -408,7 +398,7 @@ if (empty($_REQUEST['parent_discoveryid'])) {
 
 if (hasRequest('action') && getRequest('action') == 'graph.masscopyto' && hasRequest('group_graphid')) {
 	// render view
-	$data = getCopyElementsFormData('group_graphid');
+	$data = getCopyElementsFormData('group_graphid', _('Graphs'));
 	$data['action'] = 'graph.masscopyto';
 	$graphView = new CView('configuration.copy.elements', $data);
 	$graphView->render();
@@ -422,6 +412,7 @@ elseif (isset($_REQUEST['form'])) {
 		'parent_discoveryid' => getRequest('parent_discoveryid'),
 		'group_gid' => getRequest('group_gid', []),
 		'hostid' => $hostId,
+		'groupid' => $groupId,
 		'normal_only' => getRequest('normal_only')
 	];
 
@@ -613,12 +604,26 @@ elseif (isset($_REQUEST['form'])) {
 
 		$i = $next;
 	}
-	asort_by_key($data['items'], 'sortorder');
+	CArrayHelper::sort($data['items'], ['sortorder']);
 	$data['items'] = array_values($data['items']);
 
 	// is template
 	$data['is_template'] = ($data['hostid'] == 0) ? false : isTemplate($data['hostid']);
 
+	// Read groupid for selected host or template if groupid filter is set to 'All' (is equal 0).
+
+	if ($data['hostid'] && !$data['groupid']) {
+		$db_hostgroup = API::HostGroup()->get([
+			'output' => ['groupid'],
+			'hostids' => $data['hostid'],
+			'templateids' => $data['hostid']
+		]);
+
+		if ($db_hostgroup) {
+			$data['groupid'] = $db_hostgroup[0]['groupid'];
+		}
+	}
+
 	// render view
 	$graphView = new CView('configuration.graph.edit', $data);
 	$graphView->render();
