@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,8 +21,6 @@
 
 /**
  * Class containing methods for operations with API.
- *
- * @package API
  */
 class CAPIInfo extends CApiService {
 
@@ -31,7 +29,12 @@ class CAPIInfo extends CApiService {
 	 *
 	 * @return string
 	 */
-	public function version() {
+	public function version(array $request) {
+		$api_input_rules = ['type' => API_OBJECT, 'fields' =>[]];
+		if (!CApiInputValidator::validate($api_input_rules, $request, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
 		return ZABBIX_API_VERSION;
 	}
 }
