@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -31,6 +31,13 @@ class CWidget {
 	 */
 	protected $body = [];
 
+	/**
+	 * Layout mode (ZBX_LAYOUT_NORMAL|ZBX_LAYOUT_FULLSCREEN|ZBX_LAYOUT_KIOSKMODE).
+	 *
+	 * @var integer
+	 */
+	protected $web_layout_mode = ZBX_LAYOUT_NORMAL;
+
 	public function setTitle($title) {
 		$this->title = $title;
 
@@ -44,6 +51,27 @@ class CWidget {
 		return $this;
 	}
 
+	/**
+	 * Set layout mode.
+	 *
+	 * @param integer $web_layout_mode
+	 *
+	 * @return CWidget
+	 */
+	public function setWebLayoutMode($web_layout_mode) {
+		$this->web_layout_mode = $web_layout_mode;
+
+		return $this;
+	}
+
+	public function setBreadcrumbs($breadcrumbs = null) {
+		if ($breadcrumbs !== null && in_array($this->web_layout_mode, [ZBX_LAYOUT_NORMAL, ZBX_LAYOUT_FULLSCREEN])) {
+			$this->body[] = $breadcrumbs;
+		}
+
+		return $this;
+	}
+
 	public function addItem($items = null) {
 		if (!is_null($items)) {
 			$this->body[] = $items;
@@ -52,16 +80,6 @@ class CWidget {
 		return $this;
 	}
 
-	public function get() {
-		$widget = [];
-
-		if ($this->title !== null || $this->controls !== null) {
-			$widget[] = $this->createTopHeader();
-		}
-
-		return [$widget, $this->body];
-	}
-
 	public function show() {
 		echo $this->toString();
 
@@ -69,14 +87,35 @@ class CWidget {
 	}
 
 	public function toString() {
-		$tab = $this->get();
+		$widget = [];
 
+		if ($this->web_layout_mode === ZBX_LAYOUT_KIOSKMODE) {
+			$this
+				->addItem(get_icon('fullscreen')
+				->setAttribute('aria-label', _('Content controls')));
+		} elseif ($this->title !== null || $this->controls !== null) {
+			$widget[] = $this->createTopHeader();
+		}
+		$tab = [$widget, $this->body];
 		return unpack_object($tab);
 	}
 
 	private function createTopHeader() {
-		$body = [new CTag('h1', true, $this->title), $this->controls];
+		$divs = [];
 
-		return (new CDiv($body))->addClass(ZBX_STYLE_HEADER_TITLE);
+		if ($this->title !== null) {
+			$divs[] = (new CDiv((new CTag('h1', true, $this->title))->setId(ZBX_STYLE_PAGE_TITLE)))
+				->addClass(ZBX_STYLE_CELL);
+		}
+
+		if ($this->controls !== null) {
+			$divs[] = (new CDiv($this->controls))
+				->addClass(ZBX_STYLE_CELL)
+				->addClass(ZBX_STYLE_NOWRAP);
+		}
+
+		return (new CDiv($divs))
+			->addClass(ZBX_STYLE_HEADER_TITLE)
+			->addClass(ZBX_STYLE_TABLE);
 	}
 }
