@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -48,8 +48,10 @@ class CLdap {
 			$this->cnf = zbx_array_merge($this->cnf, $arg);
 		}
 
-		if (!function_exists('ldap_connect')) {
-			error('LDAP lib error. Cannot find needed functions.');
+		$ldap_status = (new CFrontendSetup())->checkPhpLdapModule();
+
+		if ($ldap_status['result'] != CFrontendSetup::CHECK_OK) {
+			error($ldap_status['error']);
 			return false;
 		}
 	}
