@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -52,12 +52,12 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'source' => 'name',
 			'method' => 'resolveGraph'
 		],
-		'screenElementURL' => [
+		'widgetURL' => [
 			'types' => ['host', 'hostId', 'interfaceWithoutPort', 'user'],
 			'source' => 'url',
 			'method' => 'resolveTexts'
 		],
-		'screenElementURLUser' => [
+		'widgetURLUser' => [
 			'types' => ['user'],
 			'source' => 'url',
 			'method' => 'resolveTexts'
@@ -316,7 +316,6 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 	 * @param int    $triggers[$triggerid]['ns']			(optional)
 	 * @param array  $options
 	 * @param bool   $options['references_only']			resolve only $1-$9 macros
-	 * @param bool   $options['events']						resolve {ITEM.VALUE} macro using 'clock' and 'ns' fields
 	 *
 	 * @return array
 	 */
@@ -409,7 +408,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			// Get macro value.
 			$macro_values = $this->getHostMacros($macros['host'], $macro_values);
 			$macro_values = $this->getIpMacros($macros['interface'], $macro_values);
-			$macro_values = $this->getItemMacros($macros['item'], $triggers, $macro_values, $options['events']);
+			$macro_values = $this->getItemMacros($macros['item'], $macro_values);
 
 			if ($usermacros) {
 				// Get hosts for triggers.
@@ -536,7 +535,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		// Get macro value.
 		$macro_values = $this->getHostMacros($macros['host'], $macro_values);
 		$macro_values = $this->getIpMacros($macros['interface'], $macro_values);
-		$macro_values = $this->getItemMacros($macros['item'], $triggers, $macro_values, false);
+		$macro_values = $this->getItemMacros($macros['item'], $macro_values);
 
 		if ($usermacros) {
 			// Get hosts for triggers.
@@ -591,7 +590,8 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 	public function resolveTriggerUrls(array $triggers) {
 		$macros = [
 			'host' => [],
-			'interface' => []
+			'interface' => [],
+			'item' => []
 		];
 		$usermacros = [];
 		$macro_values = [];
@@ -602,7 +602,11 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			],
 			'macros_n' => [
 				'host' => ['{HOST.ID}', '{HOST.HOST}', '{HOST.NAME}'],
-				'interface' => ['{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}']
+				'interface' => ['{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
+				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
+			],
+			'macro_funcs_n' => [
+				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
 			],
 			'usermacros' => true
 		];
@@ -633,6 +637,26 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 				}
 			}
 
+			foreach ($matched_macros['macros_n']['item'] as $token => $data) {
+				$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+				if (array_key_exists($data['f_num'], $functionids)) {
+					$macros['item'][$functionids[$data['f_num']]][$data['macro']][] = ['token' => $token];
+				}
+			}
+
+			foreach ($matched_macros['macro_funcs_n']['item'] as $token => $data) {
+				$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+				if (array_key_exists($data['f_num'], $functionids)) {
+					$macros['item'][$functionids[$data['f_num']]][$data['macro']][] = [
+						'token' => $token,
+						'function' => $data['function'],
+						'parameters' => $data['parameters']
+					];
+				}
+			}
+
 			if ($matched_macros['usermacros']) {
 				$usermacros[$triggerid] = ['hostids' => [], 'macros' => $matched_macros['usermacros']];
 			}
@@ -641,6 +665,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		// Get macro value.
 		$macro_values = $this->getHostMacros($macros['host'], $macro_values);
 		$macro_values = $this->getIpMacros($macros['interface'], $macro_values);
+		$macro_values = $this->getItemMacros($macros['item'], $macro_values);
 
 		if ($usermacros) {
 			// Get hosts for triggers.
@@ -735,7 +760,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 
 			// Selecting functions.
 			$result = DBselect(
-				'SELECT f.functionid,f.itemid,f.function,f.parameter'.
+				'SELECT f.functionid,f.itemid,f.name,f.parameter'.
 				' FROM functions f'.
 				' WHERE '.dbConditionInt('f.functionid', $functionids)
 			);
@@ -747,6 +772,8 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 
 			while ($row = DBfetch($result)) {
 				$itemids[$row['itemid']] = true;
+				$row['function'] = $row['name'];
+				unset($row['name']);
 
 				$functions['{'.$row['functionid'].'}'] = $row;
 				unset($functions['{'.$row['functionid'].'}']['functionid']);
@@ -1519,6 +1546,65 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		return $items;
 	}
 
+	/**
+	 * Resolve item delay macros, item history and item trend macros.
+	 *
+	 * @param array  $data
+	 * @param string $data[n]['hostid']
+	 * @param string $data[n][<sources>]  see options['source']
+	 * @param array  $options
+	 * @param array  $options['sources']  an array of the field names
+	 *
+	 * @return array
+	 */
+	public function resolveTimeUnitMacros(array $data, array $options) {
+		$usermacros = [];
+		$macro_values = [];
+
+		$types = [
+			'usermacros' => true
+		];
+
+		// Find macros.
+		foreach ($data as $key => $value) {
+			$texts = [];
+			foreach ($options['sources'] as $source) {
+				$texts[] = $value[$source];
+			}
+
+			$matched_macros = $this->extractMacros($texts, $types);
+
+			if ($matched_macros['usermacros']) {
+				$usermacros[$key] = [
+					'hostids' => array_key_exists('hostid', $value) ? [$value['hostid']] : [],
+					'macros' => $matched_macros['usermacros']
+				];
+			}
+		}
+
+		foreach ($this->getUserMacros($usermacros) as $key => $usermacros_data) {
+			$macro_values[$key] = array_key_exists($key, $macro_values)
+				? array_merge($macro_values[$key], $usermacros_data['macros'])
+				: $usermacros_data['macros'];
+		}
+
+		$types = $this->transformToPositionTypes($types);
+
+		// Replace macros to value.
+		foreach (array_keys($macro_values) as $key) {
+			foreach ($options['sources'] as $source) {
+				$matched_macros = $this->getMacroPositions($data[$key][$source], $types);
+
+				foreach (array_reverse($matched_macros, true) as $pos => $macro) {
+					$data[$key][$source] =
+						substr_replace($data[$key][$source], $macro_values[$key][$macro], $pos, strlen($macro));
+				}
+			}
+		}
+
+		return $data;
+	}
+
 	/**
 	 * Resolve function parameter macros to "parameter_expanded" field.
 	 *
@@ -1577,74 +1663,80 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 	 * @return string
 	 */
 	public function resolveMapLabelMacros($label, $replaceHosts = null) {
-		$functionsPattern = '(last|max|min|avg)\(([0-9]+['.ZBX_TIME_SUFFIXES.']?)?\)';
+		$pattern = '/(?P<macros>{'.
+				'('.ZBX_PREG_HOST_FORMAT.(($replaceHosts !== null)
+						? '|({('.self::PATTERN_HOST_INTERNAL.')'.self::PATTERN_MACRO_PARAM.'})' : '').'):'.
+				ZBX_PREG_ITEM_KEY_FORMAT.'\.'.
+				'(last|max|min|avg)\('.
+				'([0-9]+['.ZBX_TIME_SUFFIXES.']?)?'.
+				'\)}{1})/Uux';
 
-		// Find functional macro pattern.
-		$pattern = ($replaceHosts === null)
-			? '/{'.ZBX_PREG_HOST_FORMAT.':.+\.'.$functionsPattern.'}/Uu'
-			: '/{('.ZBX_PREG_HOST_FORMAT.'|{HOSTNAME[0-9]?}|{HOST\.HOST[0-9]?}):.+\.'.$functionsPattern.'}/Uu';
+		if (preg_match_all($pattern, $label, $matches) != false && array_key_exists('macros', $matches)) {
+			// For each functional macro.
+			foreach ($matches['macros'] as $expr) {
+				$macro = $expr;
 
-		preg_match_all($pattern, $label, $matches);
+				if ($replaceHosts !== null) {
+					// Search for macros with all possible indices.
+					foreach ($replaceHosts as $i => $host) {
+						$macroTmp = $macro;
 
-		// For each functional macro.
-		foreach ($matches[0] as $expr) {
-			$macro = $expr;
+						// Replace only macro in first position.
+						$macro = preg_replace('/{({HOSTNAME'.$i.'}|{HOST\.HOST'.$i.'}):(.*)}/U',
+								'{'.$host['host'].':$2}', $macro
+						);
 
-			if ($replaceHosts !== null) {
-				// Search for macros with all possible indices.
-				foreach ($replaceHosts as $i => $host) {
-					$macroTmp = $macro;
-
-					// Replace only macro in first position.
-					$macro = preg_replace('/{({HOSTNAME'.$i.'}|{HOST\.HOST'.$i.'}):(.*)}/U', '{'.$host['host'].':$2}', $macro);
-
-					// Only one simple macro possible inside functional macro.
-					if ($macro !== $macroTmp) {
-						break;
+						// Only one simple macro possible inside functional macro.
+						if ($macro !== $macroTmp) {
+							break;
+						}
 					}
 				}
-			}
 
-			// Try to create valid expression.
-			$expressionData = new CTriggerExpression();
+				// Try to create valid expression.
+				$expressionData = new CTriggerExpression();
 
-			if (!$expressionData->parse($macro) || !isset($expressionData->expressions[0])) {
-				continue;
-			}
+				if (!$expressionData->parse($macro) || !isset($expressionData->expressions[0])) {
+					continue;
+				}
 
-			// Look in DB for corresponding item.
-			$itemHost = $expressionData->expressions[0]['host'];
-			$key = $expressionData->expressions[0]['item'];
-			$function = $expressionData->expressions[0]['functionName'];
+				// Look in DB for corresponding item.
+				$itemHost = $expressionData->expressions[0]['host'];
+				$key = $expressionData->expressions[0]['item'];
+				$function = $expressionData->expressions[0]['functionName'];
 
-			$item = API::Item()->get([
-				'output' => ['itemid', 'value_type', 'units', 'valuemapid', 'lastvalue', 'lastclock'],
-				'webitems' => true,
-				'filter' => [
-					'host' => $itemHost,
-					'key_' => $key
-				]
-			]);
+				$item = API::Item()->get([
+					'output' => ['itemid', 'value_type', 'units', 'valuemapid', 'lastvalue', 'lastclock'],
+					'webitems' => true,
+					'filter' => [
+						'host' => $itemHost,
+						'key_' => $key
+					]
+				]);
 
-			$item = reset($item);
+				$item = reset($item);
 
-			// If no corresponding item found with functional macro key and host.
-			if (!$item) {
-				$label = str_replace($expr, UNRESOLVED_MACRO_STRING, $label);
+				// If no corresponding item found with functional macro key and host.
+				if (!$item) {
+					$label = str_replace($expr, UNRESOLVED_MACRO_STRING, $label);
 
-				continue;
-			}
+					continue;
+				}
 
-			// Do function type (last, min, max, avg) related actions.
-			if ($function === 'last') {
-				$value = $item['lastclock'] ? formatHistoryValue($item['lastvalue'], $item) : UNRESOLVED_MACRO_STRING;
-			}
-			else {
-				$value = getItemFunctionalValue($item, $function, $expressionData->expressions[0]['functionParamList'][0]);
-			}
+				// Do function type (last, min, max, avg) related actions.
+				if ($function === 'last') {
+					$value = $item['lastclock']
+							? formatHistoryValue($item['lastvalue'], $item) : UNRESOLVED_MACRO_STRING;
+				}
+				else {
+					$value = getItemFunctionalValue($item, $function,
+							$expressionData->expressions[0]['functionParamList'][0]
+					);
+				}
 
-			if (isset($value)) {
-				$label = str_replace($expr, $value, $label);
+				if (isset($value)) {
+					$label = str_replace($expr, $value, $label);
+				}
 			}
 		}
 
@@ -1657,7 +1749,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 	 * @param array  $selement
 	 * @param string $selement['label']						label to expand
 	 * @param int    $selement['elementtype']				element type
-	 * @param int    $selement['elementid']					element id
+	 * @param array    $selement['elements']				elements
 	 * @param string $selement['elementExpressionTrigger']	if type is trigger, then trigger expression
 	 *
 	 * @return string
@@ -1689,7 +1781,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 					'SELECT hi.ip,hi.dns,hi.useip,h.host,h.name,h.description,hi.type AS interfacetype'.
 					' FROM interface hi,hosts h'.
 					' WHERE hi.hostid=h.hostid'.
-						' AND hi.main=1 AND hi.hostid='.zbx_dbstr($selement['elementid'])
+						' AND hi.main=1 AND hi.hostid='.zbx_dbstr($selement['elements'][0]['hostid'])
 				);
 
 				// Process interface priorities.
@@ -1712,7 +1804,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 					' WHERE h.hostid=hi.hostid'.
 						' AND hi.hostid=i.hostid'.
 						' AND i.itemid=f.itemid'.
-						' AND hi.main=1 AND f.triggerid='.zbx_dbstr($selement['elementid']).
+						' AND hi.main=1 AND f.triggerid='.zbx_dbstr($selement['elements'][0]['triggerid']).
 					' ORDER BY f.functionid'
 				);
 
@@ -1732,7 +1824,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 				}
 
 				// Get all function ids from expression and link host data against position in expression.
-				preg_match_all('/\{([0-9]+)\}/', $selement['elementExpressionTrigger'], $matches);
+				preg_match_all('/\{([0-9]+)\}/', $selement['elements'][0]['elementExpressionTrigger'], $matches);
 
 				$hostsByNr = [];
 
