@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -63,8 +63,12 @@ class CMacrosResolverGeneral {
 		// Replace functionids with string 'function' to make values search easier.
 		$expression = preg_replace('/\{[0-9]+\}/', 'function', $expression);
 
+		// Replace whitespace with emptyness to make value search easier.
+		$expression = str_replace(" \r\n\t", '', $expression);
+
 		// Search for numeric values in expression.
-		preg_match_all('/'.ZBX_PREG_NUMBER.'/', $expression, $values);
+		preg_match_all('/((?<![\)\.0-9]|[\.0-9]['.ZBX_BYTE_SUFFIXES.ZBX_TIME_SUFFIXES.']|function)\-?'.
+				'([.][0-9]+|[0-9]+[.]?[0-9]*)['.ZBX_BYTE_SUFFIXES.ZBX_TIME_SUFFIXES.']?)/', $expression, $values);
 
 		$macro_values = [];
 
@@ -131,7 +135,7 @@ class CMacrosResolverGeneral {
 	 *
 	 * @return array
 	 */
-	protected function getMacroPositions($text, array $types) {
+	public function getMacroPositions($text, array $types) {
 		$macros = [];
 		$extract_usermacros = array_key_exists('usermacros', $types);
 		$extract_macros = array_key_exists('macros', $types);
@@ -164,6 +168,7 @@ class CMacrosResolverGeneral {
 
 		if ($extract_lldmacros) {
 			$lld_macro_parser = new CLLDMacroParser();
+			$lld_macro_function_parser = new CLLDMacroFunctionParser();
 		}
 
 		if ($extract_functionids) {
@@ -199,6 +204,10 @@ class CMacrosResolverGeneral {
 				$macros[$pos] = $lld_macro_parser->getMatch();
 				$pos += $lld_macro_parser->getLength() - 1;
 			}
+			elseif ($extract_lldmacros && $lld_macro_function_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
+				$macros[$pos] = $lld_macro_function_parser->getMatch();
+				$pos += $lld_macro_function_parser->getLength() - 1;
+			}
 			elseif ($extract_functionids && $functionid_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
 				$macros[$pos] = $functionid_parser->getMatch();
 				$pos += $functionid_parser->getLength() - 1;
@@ -280,6 +289,7 @@ class CMacrosResolverGeneral {
 			$macros['lldmacros'] = [];
 
 			$lld_macro_parser = new CLLDMacroParser();
+			$lld_macro_function_parser = new CLLDMacroFunctionParser();
 		}
 
 		if ($extract_functionids) {
@@ -356,10 +366,17 @@ class CMacrosResolverGeneral {
 					continue;
 				}
 
-				if ($extract_lldmacros && $lld_macro_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
-					$macros['lldmacros'][$lld_macro_parser->getMatch()] = null;
-					$pos += $lld_macro_parser->getLength() - 1;
-					continue;
+				if ($extract_lldmacros) {
+					if ($lld_macro_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
+						$macros['lldmacros'][$lld_macro_parser->getMatch()] = null;
+						$pos += $lld_macro_parser->getLength() - 1;
+						continue;
+					}
+					elseif ($lld_macro_function_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
+						$macros['lldmacros'][$lld_macro_function_parser->getMatch()] = null;
+						$pos += $lld_macro_function_parser->getLength() - 1;
+						continue;
+					}
 				}
 
 				if ($extract_functionids && $functionid_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
@@ -669,13 +686,11 @@ class CMacrosResolverGeneral {
 	 * @param array $macros
 	 * @param array $macros[<functionid>]
 	 * @param array $macros[<functionid>][<macro>]  an array of the tokens
-	 * @param array $triggers
 	 * @param array $macro_values
-	 * @param bool  $events			resolve {ITEM.VALUE} macro using 'clock' and 'ns' fields
 	 *
 	 * @return array
 	 */
-	protected function getItemMacros(array $macros, array $triggers, array $macro_values, $events) {
+	protected function getItemMacros(array $macros, array $macro_values) {
 		if (!$macros) {
 			return $macro_values;
 		}
@@ -691,60 +706,48 @@ class CMacrosResolverGeneral {
 		// False passed to DBfetch to get data without null converted to 0, which is done by default.
 		foreach ($functions as $function) {
 			foreach ($macros[$function['functionid']] as $macro => $tokens) {
-				switch ($macro) {
-					case 'ITEM.VALUE':
-						if ($events) {
-							$trigger = $triggers[$function['triggerid']];
-							$value = item_get_history($function, $trigger['clock'], $trigger['ns']);
-							break;
+				if ($macro === 'ITEM.VALUE' || $macro === 'ITEM.LASTVALUE') {
+					$history = Manager::History()->getLastValues([$function], 1, ZBX_HISTORY_PERIOD);
+
+					if (array_key_exists($function['itemid'], $history)) {
+						$value = $history[$function['itemid']][0]['value'];
+
+						foreach ($tokens as $token) {
+							if (array_key_exists('function', $token)) {
+								if ($token['function'] !== 'regsub' && $token['function'] !== 'iregsub') {
+									continue;
+								}
+
+								if (count($token['parameters']) != 2) {
+									continue;
+								}
+
+								$ci = ($token['function'] === 'iregsub') ? 'i' : '';
+
+								set_error_handler(function ($errno, $errstr) {});
+								$rc = preg_match('/'.$token['parameters'][0].'/'.$ci, $value, $matches);
+								restore_error_handler();
+
+								if ($rc === false) {
+									continue;
+								}
+
+								$macro_value = $token['parameters'][1];
+								$matched_macros = $this->getMacroPositions($macro_value, ['replacements' => true]);
+
+								foreach (array_reverse($matched_macros, true) as $pos => $macro) {
+									$macro_value = substr_replace($macro_value,
+										array_key_exists($macro[1], $matches) ? $matches[$macro[1]] : '',
+										$pos, strlen($macro)
+									);
+								}
+							}
+							else {
+								$macro_value = formatHistoryValue($value, $function);
+							}
+
+							$macro_values[$function['triggerid']][$token['token']] = $macro_value;
 						}
-						// break; is not missing here
-
-					case 'ITEM.LASTVALUE':
-						$history = Manager::History()->getLast([$function], 1, ZBX_HISTORY_PERIOD);
-
-						$value = array_key_exists($function['itemid'], $history)
-							? $history[$function['itemid']][0]['value']
-							: null;
-						break;
-				}
-
-				if ($value !== null) {
-					foreach ($tokens as $token) {
-						if (array_key_exists('function', $token)) {
-							if ($token['function'] !== 'regsub' && $token['function'] !== 'iregsub') {
-								continue;
-							}
-
-							if (count($token['parameters']) != 2) {
-								continue;
-							}
-
-							$ci = ($token['function'] === 'iregsub') ? 'i' : '';
-
-							set_error_handler(function ($errno, $errstr) {});
-							$rc = preg_match('/'.$token['parameters'][0].'/'.$ci, $value, $matches);
-							restore_error_handler();
-
-							if ($rc === false) {
-								continue;
-							}
-
-							$macro_value = $token['parameters'][1];
-							$matched_macros = $this->getMacroPositions($macro_value, ['replacements' => true]);
-
-							foreach (array_reverse($matched_macros, true) as $pos => $macro) {
-								$macro_value = substr_replace($macro_value,
-									array_key_exists($macro[1], $matches) ? $matches[$macro[1]] : '',
-									$pos, strlen($macro)
-								);
-							}
-						}
-						else {
-							$macro_value = formatHistoryValue($value, $function);
-						}
-
-						$macro_values[$function['triggerid']][$token['token']] = $macro_value;
 					}
 				}
 			}
@@ -826,13 +829,16 @@ class CMacrosResolverGeneral {
 	 * Get macros with values.
 	 *
 	 * @param array $data
-	 * @param array $data[<id>]			any identificator
-	 * @param array $data['hostids']	the list of host ids; [<hostid1>, ...]
-	 * @param array $data['macros']		the list of user macros to resolve, ['<usermacro1>' => null, ...]
+	 * @param array $data[n]['hostids']  the list of host ids; [<hostid1>, ...]
+	 * @param array $data[n]['macros']   the list of user macros to resolve, ['<usermacro1>' => null, ...]
 	 *
 	 * @return array
 	 */
 	protected function getUserMacros(array $data) {
+		if (!$data) {
+			return $data;
+		}
+
 		// User macros.
 		$hostids = [];
 		foreach ($data as $element) {
@@ -841,9 +847,7 @@ class CMacrosResolverGeneral {
 			}
 		}
 
-		if (!$hostids) {
-			return $data;
-		}
+		$user_macro_parser = new CUserMacroParser();
 
 		/*
 		 * @var array $host_templates
@@ -856,69 +860,70 @@ class CMacrosResolverGeneral {
 		 * @var array  $host_macros[<hostid>]
 		 * @var array  $host_macros[<hostid>][<macro>]				macro base without curly braces
 		 * @var string $host_macros[<hostid>][<macro>]['value']		base macro value (without context); can be null
-		 * @var array  $host_macros[<hostid>][<macro>]['contexts']	context values; ['<context1>' => '<value1>', ...]
+		 * @var array  $host_macros[<hostid>][<macro>]['contexts']	context values; ['<context>' => '<value>', ...]
 		 */
 		$host_macros = [];
 
-		$user_macro_parser = new CUserMacroParser();
+		if ($hostids) {
+			do {
+				$hostids = array_keys($hostids);
 
-		do {
-			$db_host_macros = DBselect(
-				'SELECT hm.hostid,hm.macro,hm.value'.
-				' FROM hostmacro hm'.
-				' WHERE '.dbConditionInt('hm.hostid', array_keys($hostids))
-			);
-			while ($db_host_macro = DBfetch($db_host_macros)) {
-				if ($user_macro_parser->parse($db_host_macro['macro']) != CParser::PARSE_SUCCESS) {
-					continue;
+				$db_host_macros = DBselect(
+					'SELECT hm.hostid,hm.macro,hm.value'.
+					' FROM hostmacro hm'.
+					' WHERE '.dbConditionInt('hm.hostid', $hostids)
+				);
+				while ($db_host_macro = DBfetch($db_host_macros)) {
+					if ($user_macro_parser->parse($db_host_macro['macro']) != CParser::PARSE_SUCCESS) {
+						continue;
+					}
+
+					$macro = $user_macro_parser->getMacro();
+					$context = $user_macro_parser->getContext();
+
+					if (!array_key_exists($db_host_macro['hostid'], $host_macros)) {
+						$host_macros[$db_host_macro['hostid']] = [];
+					}
+
+					if (!array_key_exists($macro, $host_macros[$db_host_macro['hostid']])) {
+						$host_macros[$db_host_macro['hostid']][$macro] = ['value' => null, 'contexts' => []];
+					}
+
+					if ($context === null) {
+						$host_macros[$db_host_macro['hostid']][$macro]['value'] = $db_host_macro['value'];
+					}
+					else {
+						$host_macros[$db_host_macro['hostid']][$macro]['contexts'][$context] = $db_host_macro['value'];
+					}
 				}
 
-				$macro = $user_macro_parser->getMacro();
-				$context = $user_macro_parser->getContext();
-				$host_macros[$db_host_macro['hostid']][$macro] = ['value' => null, 'contexts' => []];
-
-				if ($context === null) {
-					$host_macros[$db_host_macro['hostid']][$macro]['value'] = $db_host_macro['value'];
+				foreach ($hostids as $hostid) {
+					$host_templates[$hostid] = [];
 				}
-				else {
-					$host_macros[$db_host_macro['hostid']][$macro]['contexts'][$context] = $db_host_macro['value'];
-				}
-			}
 
-			$templateids = [];
-			$db_host_templates = DBselect(
-				'SELECT ht.hostid,ht.templateid'.
-				' FROM hosts_templates ht'.
-				' WHERE '.dbConditionInt('ht.hostid', array_keys($hostids))
-			);
-			while ($db_host_template = DBfetch($db_host_templates)) {
-				$host_templates[$db_host_template['hostid']][] = $db_host_template['templateid'];
-				$templateids[$db_host_template['templateid']] = true;
-
-				if (!array_key_exists($db_host_template['hostid'], $host_macros)) {
-					$host_macros[$db_host_template['hostid']] = [];
+				$templateids = [];
+				$db_host_templates = DBselect(
+					'SELECT ht.hostid,ht.templateid'.
+					' FROM hosts_templates ht'.
+					' WHERE '.dbConditionInt('ht.hostid', $hostids)
+				);
+				while ($db_host_template = DBfetch($db_host_templates)) {
+					$host_templates[$db_host_template['hostid']][] = $db_host_template['templateid'];
+					$templateids[$db_host_template['templateid']] = true;
 				}
-			}
 
-			foreach ($hostids as $key => $value) {
-				if (!array_key_exists($key, $host_templates)) {
-					$host_templates[$key] = [];
+				// only unprocessed templates will be populated
+				$hostids = [];
+				foreach (array_keys($templateids) as $templateid) {
+					if (!array_key_exists($templateid, $host_templates)) {
+						$hostids[$templateid] = true;
+					}
 				}
-			}
-
-			// only unprocessed templates will be populated
-			$hostids = [];
-			foreach ($templateids as $key => $value) {
-				if (!array_key_exists($key, $host_templates)) {
-					$hostids[$key] = true;
-				}
-			}
-		} while ($hostids);
+			} while ($hostids);
+		}
 
 		$all_macros_resolved = true;
 
-		$user_macro_parser = new CUserMacroParser();
-
 		foreach ($data as &$element) {
 			$hostids = [];
 			foreach ($element['hostids'] as $hostid) {
