@@ -709,7 +709,7 @@ class CPageFilter {
 	/**
 	 * Initialize minimum trigger severities.
 	 *
-	 * @param string $severityMin			minimum trigger severity
+	 * @param string $severityMin			minimum severity
 	 * @param array  $options				array of options
 	 * @param int    $options['default']	default severity
 	 * @param string $options['mapId']		ID of a map
@@ -751,30 +751,43 @@ class CPageFilter {
 	/**
 	 * Get hosts combobox with selected item.
 	 *
+	 * @param array  $options
+	 * @param array  $options['action']		onChange action.
+	 *
 	 * @return CComboBox
 	 */
-	public function getHostsCB() {
+	public function getHostsCB(array $options = []) {
 		$items = $classes = [];
 		foreach ($this->hosts as $id => $host) {
 			$items[$id] = $host['name'];
 			$classes[$id] = ($host['status'] == HOST_STATUS_NOT_MONITORED) ? ZBX_STYLE_RED : null;
 		}
-		$options = ['objectName' => 'hosts', 'classes' => $classes];
+		$options += ['objectName' => 'hosts', 'classes' => $classes];
 
-		return $this->_getCB('hostid', $this->hostid, $items, $options);
+		return $this->_getCB('hostid', $this->hostid, $items, $options)
+					->addStyle('max-width: '.ZBX_TEXTAREA_FILTER_STANDARD_WIDTH.'px;');
 	}
 
 	/**
 	 * Get host groups combobox with selected item.
 	 *
+	 * @param array  $options
+	 * @param array  $options['action']		onChange action.
+	 *
 	 * @return CComboBox
 	 */
-	public function getGroupsCB() {
+	public function getGroupsCB(array $options = []) {
 		$items = [];
 		foreach ($this->groups as $id => $group) {
 			$items[$id] = $group['name'];
 		}
-		return $this->_getCB('groupid', $this->groupid, $items, ['objectName' => 'groups']);
+
+		$options += [
+			'objectName' => 'groups'
+		];
+
+		return $this->_getCB('groupid', $this->groupid, $items, $options)
+					->addStyle('max-width: '.ZBX_TEXTAREA_FILTER_STANDARD_WIDTH.'px;');
 	}
 
 	/**
@@ -832,11 +845,16 @@ class CPageFilter {
 	 * @param array  $options
 	 * @param string $options['objectName']
 	 * @param array  $options['classes']	array of class names for the combobox options with item IDs as keys
+	 * @param array  $options['action']		onChange action. Default value is 'submit()'.
 	 *
 	 * @return CComboBox
 	 */
 	private function _getCB($name, $selectedId, $items, array $options = []) {
-		$comboBox = new CComboBox($name, $selectedId, 'javascript: submit();');
+		if (!array_key_exists('action', $options)) {
+			$options['action'] = 'javascript: submit();';
+		}
+
+		$comboBox = (new CComboBox($name, $selectedId, $options['action']))->removeId();
 
 		natcasesort($items);
 
