@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -46,13 +46,11 @@ class CTriggerExpression {
 	 * An array of trigger functions like {Zabbix server:agent.ping.last(0)}
 	 * The array isn't unique. Same functions can repeat.
 	 *
-	 * @see CFunctionMacroParserResult::$expression     for a detailed array structure
-	 *
 	 * @deprecated  use result tokens instead
 	 *
 	 * @var array
 	 */
-	public $expressions = array();
+	public $expressions = [];
 
 	/**
 	 * An options array
@@ -62,7 +60,7 @@ class CTriggerExpression {
 	 *
 	 * @var array
 	 */
-	public $options = array('lldmacros' => true);
+	public $options = ['lldmacros' => true];
 
 	/**
 	 * Source string.
@@ -102,37 +100,51 @@ class CTriggerExpression {
 	/**
 	 * Parser for the {TRIGGER.VALUE} macro.
 	 *
-	 * @var CSetParser
+	 * @var CMacroParser
 	 */
-	protected $macroParser;
+	protected $macro_parser;
 
 	/**
 	 * Parser for function macros.
 	 *
 	 * @var CFunctionMacroParser
 	 */
-	protected $functionMacroParser;
+	protected $function_macro_parser;
 
 	/**
-	 * Parser for user macros.
+	 * Parser for trigger functions.
 	 *
-	 * @var CMacroParser
+	 * @var CFunctionParser
 	 */
-	protected $userMacroParser;
+	protected $function_parser;
 
 	/**
 	 * Parser for LLD macros.
 	 *
-	 * @var CMacroParser
+	 * @var CLLDMacroParser
 	 */
-	protected $lldMacroParser;
+	protected $lld_macro_parser;
+
+	/**
+	 * Parser for LLD macros with functions.
+	 *
+	 * @var CLLDMacroFunctionParser
+	 */
+	protected $lld_macro_function_parser;
+
+	/**
+	 * Parser for user macros.
+	 *
+	 * @var CUserMacroParser
+	 */
+	protected $user_macro_parser;
 
 	/**
 	 * Chars that should be treated as spaces.
 	 *
 	 * @var array
 	 */
-	protected $spaceChars = array(' ' => true, "\r" => true, "\n" => true, "\t" => true);
+	protected $spaceChars = [' ' => true, "\r" => true, "\n" => true, "\t" => true];
 
 	/**
 	 * Object containing the results of parsing.
@@ -145,18 +157,20 @@ class CTriggerExpression {
 	 * @param array $options
 	 * @param bool $options['lldmacros']
 	 */
-	public function __construct($options = array()) {
+	public function __construct($options = []) {
 		if (isset($options['lldmacros'])) {
 			$this->options['lldmacros'] = $options['lldmacros'];
 		}
 
-		$this->binaryOperatorParser = new CSetParser(array('<', '>', '<=', '>=', '+', '-', '/', '*', '=', '<>'));
-		$this->logicalOperatorParser = new CSetParser(array('and', 'or'));
-		$this->notOperatorParser = new CSetParser(array('not'));
-		$this->macroParser = new CSetParser(array('{TRIGGER.VALUE}'));
-		$this->functionMacroParser = new CFunctionMacroParser();
-		$this->userMacroParser = new CMacroParser('$');
-		$this->lldMacroParser = new CMacroParser('#');
+		$this->binaryOperatorParser = new CSetParser(['<', '>', '<=', '>=', '+', '-', '/', '*', '=', '<>']);
+		$this->logicalOperatorParser = new CSetParser(['and', 'or']);
+		$this->notOperatorParser = new CSetParser(['not']);
+		$this->macro_parser = new CMacroParser(['{TRIGGER.VALUE}']);
+		$this->function_macro_parser = new CFunctionMacroParser();
+		$this->function_parser = new CFunctionParser();
+		$this->lld_macro_parser = new CLLDMacroParser();
+		$this->lld_macro_function_parser = new CLLDMacroFunctionParser;
+		$this->user_macro_parser = new CUserMacroParser();
 	}
 
 	/**
@@ -191,7 +205,7 @@ class CTriggerExpression {
 		$this->result = new CTriggerExpressionParserResult();
 		$this->isValid = true;
 		$this->error = '';
-		$this->expressions = array();
+		$this->expressions = [];
 
 		$this->pos = 0;
 		$this->expression = $expression;
@@ -337,16 +351,6 @@ class CTriggerExpression {
 								$state = self::STATE_AFTER_LOGICAL_OPERATOR;
 								break;
 							}
-
-							if (!$afterSpace) {
-								break 3;
-							}
-
-							if ($this->parseUsing($this->notOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
-
-								$state = self::STATE_AFTER_NOT_OPERATOR;
-							}
 							else {
 								break 3;
 							}
@@ -377,12 +381,7 @@ class CTriggerExpression {
 								break 3;
 							}
 
-							if ($this->parseUsing($this->notOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
-
-								$state = self::STATE_AFTER_NOT_OPERATOR;
-							}
-							elseif ($this->parseUsing($this->logicalOperatorParser,
+							if ($this->parseUsing($this->logicalOperatorParser,
 									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
 
 								$state = self::STATE_AFTER_LOGICAL_OPERATOR;
@@ -480,7 +479,7 @@ class CTriggerExpression {
 	 */
 	public function getHosts() {
 		if (!$this->isValid) {
-			return array();
+			return [];
 		}
 
 		return array_unique(zbx_objectValues($this->expressions, 'host'));
@@ -493,49 +492,47 @@ class CTriggerExpression {
 	 * @param CParser   $parser
 	 * @param int       $tokenType
 	 *
-	 * @return CParserResult|bool		CParserResult object if a match has been found, false otherwise
+	 * @return bool
 	 */
 	protected function parseUsing(CParser $parser, $tokenType) {
-		$j = $this->pos;
-
-		$result = $parser->parse($this->expression, $j);
-
-		if (!$result) {
+		if ($parser->parse($this->expression, $this->pos) == CParser::PARSE_FAIL) {
 			return false;
 		}
 
-		$this->pos += $result->length - 1;
+		$this->result->addToken($tokenType, $parser->getMatch(), $this->pos, $parser->getLength());
+		$this->pos += $parser->getLength() - 1;
 
-		$this->result->addToken($tokenType, $result->match, $result->pos, $result->length);
-
-		return $result;
+		return true;
 	}
 
 	/**
 	 * Parses a constant in the trigger expression and moves a current position ($this->pos) on a last symbol of the
-	 * constant
+	 * constant.
 	 *
 	 * The constant can be:
 	 *  - trigger function like {host:item[].func()}
 	 *  - floating point number; can be with suffix [KMGTsmhdw]
 	 *  - macro like {TRIGGER.VALUE}
 	 *  - user macro like {$MACRO}
+	 *  - LLD macro like {#LLD}
+	 *  - LLD macro with function like {{#LLD}.func())}
 	 *
-	 * @return bool returns true if parsed successfully, false otherwise
+	 * @return bool  Returns true if parsed successfully, false otherwise.
 	 */
 	private function parseConstant() {
 		if ($this->parseFunctionMacro() || $this->parseNumber()
-				|| $this->parseUsing($this->macroParser, CTriggerExpressionParserResult::TOKEN_TYPE_MACRO)
-				|| $this->parseUsing($this->userMacroParser, CTriggerExpressionParserResult::TOKEN_TYPE_USER_MACRO)) {
-
+				|| $this->parseUsing($this->user_macro_parser, CTriggerExpressionParserResult::TOKEN_TYPE_USER_MACRO)
+				|| $this->parseUsing($this->macro_parser, CTriggerExpressionParserResult::TOKEN_TYPE_MACRO)) {
 			return true;
 		}
 
-		// LLD macro support for trigger prototypes
-		if (($this->options['lldmacros']
-			&& $this->parseUsing($this->lldMacroParser, CTriggerExpressionParserResult::TOKEN_TYPE_LLD_MACRO))) {
-
-			return true;
+		// LLD macro support for trigger prototypes.
+		if ($this->options['lldmacros']) {
+			if ($this->parseUsing($this->lld_macro_parser, CTriggerExpressionParserResult::TOKEN_TYPE_LLD_MACRO)
+					|| $this->parseUsing($this->lld_macro_function_parser,
+							CTriggerExpressionParserResult::TOKEN_TYPE_LLD_MACRO)) {
+				return true;
+			}
 		}
 
 		return false;
@@ -550,27 +547,43 @@ class CTriggerExpression {
 	private function parseFunctionMacro() {
 		$startPos = $this->pos;
 
-		$parser = new CFunctionMacroParser();
-		$result = $parser->parse($this->expression, $this->pos);
-
-		if (!$result) {
+		if ($this->function_macro_parser->parse($this->expression, $this->pos) == CParser::PARSE_FAIL) {
 			return false;
 		}
 
-		$this->pos += $result->length - 1;
+		if ($this->function_parser->parse($this->function_macro_parser->getFunction()) == CParser::PARSE_FAIL) {
+			return false;
+		}
 
-		$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_FUNCTION_MACRO, $result->match,
-			$startPos, $result->length,
-			array(
-				'host' => $result->expression['host'],
-				'item' => $result->expression['item'],
-				'function' => $result->expression['function'],
-				'functionName' => $result->expression['functionName'],
-				'functionParams' => $result->expression['functionParamList']
-			)
+		$this->pos += $this->function_macro_parser->getLength() - 1;
+
+		$function_param_list = [];
+
+		for ($n = 0; $n < $this->function_parser->getParamsNum(); $n++) {
+			$function_param_list[] = $this->function_parser->getParam($n);
+		}
+
+		$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_FUNCTION_MACRO,
+			$this->function_macro_parser->getMatch(), $startPos, $this->function_macro_parser->getLength(),
+			[
+				'host' => $this->function_macro_parser->getHost(),
+				'item' => $this->function_macro_parser->getItem(),
+				'function' => $this->function_macro_parser->getFunction(),
+				'functionName' => $this->function_parser->getFunction(),
+				'functionParams' => $function_param_list
+			]
 		);
 
-		$this->expressions[] = $result->expression;
+		$this->expressions[] = [
+			'expression' => $this->function_macro_parser->getMatch(),
+			'pos' => $startPos,
+			'host' => $this->function_macro_parser->getHost(),
+			'item' => $this->function_macro_parser->getItem(),
+			'function' => $this->function_macro_parser->getFunction(),
+			'functionName' => $this->function_parser->getFunction(),
+			'functionParam' => $this->function_parser->getParameters(),
+			'functionParamList' => $function_param_list
+		];
 
 		return true;
 	}
@@ -579,6 +592,8 @@ class CTriggerExpression {
 	 * Parses a number constant in the trigger expression and
 	 * moves a current position ($this->pos) on a last symbol of the number
 	 *
+	 * comments: !!! Don't forget sync code with C !!!
+	 *
 	 * @return bool returns true if parsed successfully, false otherwise
 	 */
 	private function parseNumber() {
@@ -610,7 +625,6 @@ class CTriggerExpression {
 		$suffix = null;
 		if (isset($this->expression[$j])
 				&& strpos(ZBX_BYTE_SUFFIXES.ZBX_TIME_SUFFIXES, $this->expression[$j]) !== false) {
-
 			$suffix = $this->expression[$j];
 			$j++;
 		}
@@ -622,9 +636,7 @@ class CTriggerExpression {
 			substr($this->expression, $this->pos, $numberLength),
 			$this->pos,
 			$numberLength,
-			array(
-				'suffix' => $suffix
-			)
+			['suffix' => $suffix]
 		);
 
 		$this->pos = $j - 1;
