@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ class CScreenEvents extends CScreenBase {
 		]);
 
 		$triggers = API::Trigger()->get([
-			'output' => ['triggerid', 'description', 'expression', 'priority'],
+			'output' => [],
 			'selectHosts' => ['hostid', 'name'],
 			'skipDependent' => true,
 			'monitored' => true,
@@ -44,23 +44,17 @@ class CScreenEvents extends CScreenBase {
 			'preservekeys' => true
 		]);
 
-		$options = [
-			'output' => ['eventid', 'r_eventid', 'objectid', 'clock'],
+		$events = API::Event()->get([
+			'output' => ['eventid', 'r_eventid', 'objectid', 'clock', 'ns', 'name', 'acknowledged', 'severity'],
+			'select_acknowledges' => ['action'],
 			'source' => EVENT_SOURCE_TRIGGERS,
 			'object' => EVENT_OBJECT_TRIGGER,
 			'value' => TRIGGER_VALUE_TRUE,
-			'objectids' => zbx_objectValues($triggers, 'triggerid'),
+			'objectids' => array_keys($triggers),
 			'sortfield' => ['clock', 'eventid'],
 			'sortorder' => ZBX_SORT_DOWN,
 			'limit' => $this->screenitem['elements']
-		];
-
-		if ($config['event_ack_enable']) {
-			$options['output'][] = 'acknowledged';
-			$options['select_acknowledges'] = ['userid', 'clock', 'message', 'action'];
-		}
-
-		$events = API::Event()->get($options);
+		]);
 
 		$sort_clock = [];
 		$sort_event = [];
@@ -70,13 +64,9 @@ class CScreenEvents extends CScreenBase {
 				continue;
 			}
 
-			$events[$key]['trigger'] = $triggers[$event['objectid']];
-			$events[$key]['host'] = reset($events[$key]['trigger']['hosts']);
+			$events[$key]['host'] = reset($triggers[$event['objectid']]['hosts']);
 			$sort_clock[$key] = $event['clock'];
 			$sort_event[$key] = $event['eventid'];
-
-			$merged_event = array_merge($event, $triggers[$event['objectid']]);
-			$events[$key]['trigger']['description'] = CMacrosResolverHelper::resolveEventDescription($merged_event);
 		}
 		array_multisort($sort_clock, SORT_DESC, $sort_event, SORT_DESC, $events);
 
@@ -110,18 +100,15 @@ class CScreenEvents extends CScreenBase {
 		}
 
 		foreach ($events as $event) {
-			$trigger = $event['trigger'];
 			$host = $event['host'];
 
 			if ($event['r_eventid'] == 0) {
 				$in_closing = false;
 
-				if ($config['event_ack_enable']) {
-					foreach ($event['acknowledges'] as $acknowledge) {
-						if ($acknowledge['action'] == ZBX_ACKNOWLEDGE_ACTION_CLOSE_PROBLEM) {
-							$in_closing = true;
-							break;
-						}
+				foreach ($event['acknowledges'] as $acknowledge) {
+					if (($acknowledge['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE) {
+						$in_closing = true;
+						break;
 					}
 				}
 
@@ -138,20 +125,18 @@ class CScreenEvents extends CScreenBase {
 			$statusSpan = new CSpan($value_str);
 
 			// Add colors span depending on configuration and trigger parameters.
-			addTriggerValueStyle($statusSpan, $value, $event['clock'],
-				$config['event_ack_enable'] && $event['acknowledged']
-			);
+			addTriggerValueStyle($statusSpan, $value, $event['clock'], $event['acknowledged'] == EVENT_ACKNOWLEDGED);
 
 			$table->addRow([
 				zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event['clock']),
 				($event['r_eventid'] == 0) ? '' : zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event['r_clock']),
 				$host['name'],
 				new CLink(
-					$trigger['description'],
+					$event['name'],
 					'tr_events.php?triggerid='.$event['objectid'].'&eventid='.$event['eventid']
 				),
 				$statusSpan,
-				getSeverityCell($trigger['priority'], $config)
+				getSeverityCell($event['severity'], $config)
 			]);
 		}
 
