@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -42,26 +42,26 @@ class CScreenGraph extends CScreenBase {
 
 		if ($this->screenitem['dynamic'] == SCREEN_DYNAMIC_ITEM && $this->hostid) {
 			// get host
-			$hosts = API::Host()->get(array(
+			$hosts = API::Host()->get([
 				'hostids' => $this->hostid,
-				'output' => array('hostid', 'name')
-			));
+				'output' => ['hostid', 'name']
+			]);
 			$host = reset($hosts);
 
 			// get graph
-			$graph = API::Graph()->get(array(
+			$graph = API::Graph()->get([
 				'graphids' => $resourceId,
 				'output' => API_OUTPUT_EXTEND,
-				'selectHosts' => array('hostid'),
+				'selectHosts' => ['hostid'],
 				'selectGraphItems' => API_OUTPUT_EXTEND
-			));
+			]);
 			$graph = reset($graph);
 
 			// if items from one host we change them, or set calculated if not exist on that host
 			if (count($graph['hosts']) == 1) {
 				if ($graph['ymax_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE && $graph['ymax_itemid']) {
 					$newDynamic = getSameGraphItemsForHost(
-						array(array('itemid' => $graph['ymax_itemid'])),
+						[['itemid' => $graph['ymax_itemid']]],
 						$this->hostid,
 						false
 					);
@@ -77,7 +77,7 @@ class CScreenGraph extends CScreenBase {
 
 				if ($graph['ymin_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE && $graph['ymin_itemid']) {
 					$newDynamic = getSameGraphItemsForHost(
-						array(array('itemid' => $graph['ymin_itemid'])),
+						[['itemid' => $graph['ymin_itemid']]],
 						$this->hostid,
 						false
 					);
@@ -119,34 +119,30 @@ class CScreenGraph extends CScreenBase {
 		}
 
 		// get time control
-		$timeControlData = array(
+		$timeControlData = [
 			'id' => $this->getDataId(),
 			'containerid' => $containerId,
 			'objDims' => $graphDims,
 			'loadSBox' => 0,
-			'loadImage' => 1,
-			'periodFixed' => CProfile::get('web.screens.timelinefixed', 1),
-			'sliderMaximumTimePeriod' => ZBX_MAX_PERIOD
-		);
+			'loadImage' => 1
+		];
 
 		$isDefault = false;
 		if ($graphDims['graphtype'] == GRAPH_TYPE_PIE || $graphDims['graphtype'] == GRAPH_TYPE_EXPLODED) {
-			if ($this->screenitem['dynamic'] == SCREEN_SIMPLE_ITEM || !$this->screenitem['url']) {
+			if ($this->screenitem['dynamic'] == SCREEN_SIMPLE_ITEM || $this->screenitem['url'] === '') {
 				$this->screenitem['url'] = 'chart6.php?graphid='.$resourceId.'&screenid='.$this->screenitem['screenid'];
 				$isDefault = true;
 			}
 
-			$this->timeline['starttime'] = date(TIMESTAMP_FORMAT, get_min_itemclock_by_graphid($resourceId));
-
-			$timeControlData['src'] = $this->screenitem['url'].'&width='.$this->screenitem['width']
-				.'&height='.$this->screenitem['height'].'&legend='.$legend
-				.'&graph3d='.$graph3d.$this->getProfileUrlParams();
+			$timeControlData['src'] = $this->screenitem['url'].'&width='.$this->screenitem['width'].
+				'&height='.$this->screenitem['height'].'&legend='.$legend.
+				'&graph3d='.$graph3d.$this->getProfileUrlParams();
 			$timeControlData['src'] .= ($this->mode == SCREEN_MODE_EDIT)
-				? '&period=3600&stime='.date(TIMESTAMP_FORMAT, time())
-				: '&period='.$this->timeline['period'].'&stime='.$this->timeline['stimeNow'];
+				? '&from='.ZBX_PERIOD_DEFAULT_FROM.'&to='.ZBX_PERIOD_DEFAULT_TO
+				: '&from='.$this->timeline['from'].'&to='.$this->timeline['to'];
 		}
 		else {
-			if ($this->screenitem['dynamic'] == SCREEN_SIMPLE_ITEM || !$this->screenitem['url']) {
+			if ($this->screenitem['dynamic'] == SCREEN_SIMPLE_ITEM || $this->screenitem['url'] === '') {
 				$this->screenitem['url'] = 'chart2.php?graphid='.$resourceId.'&screenid='.$this->screenitem['screenid'];
 				$isDefault = true;
 			}
@@ -157,27 +153,27 @@ class CScreenGraph extends CScreenBase {
 				}
 			}
 
-			$timeControlData['src'] = $this->screenitem['url'].'&width='.$this->screenitem['width']
-				.'&height='.$this->screenitem['height'].'&legend='.$legend.$this->getProfileUrlParams();
+			$timeControlData['src'] = $this->screenitem['url'].'&width='.$this->screenitem['width'].
+				'&height='.$this->screenitem['height'].'&legend='.$legend.$this->getProfileUrlParams();
 			$timeControlData['src'] .= ($this->mode == SCREEN_MODE_EDIT)
-				? '&period=3600&stime='.date(TIMESTAMP_FORMAT, time())
-				: '&period='.$this->timeline['period'].'&stime='.$this->timeline['stimeNow'];
+				? '&from='.ZBX_PERIOD_DEFAULT_FROM.'&to='.ZBX_PERIOD_DEFAULT_TO
+				: '&from='.$this->timeline['from'].'&to='.$this->timeline['to'];
 		}
 
 		// output
 		if ($this->mode == SCREEN_MODE_JS) {
-			return 'timeControl.addObject("'.$this->getDataId().'", '.CJs::encodeJson($this->timeline).', '
-				.CJs::encodeJson($timeControlData).')';
+			return 'timeControl.addObject("'.$this->getDataId().'", '.CJs::encodeJson($this->timeline).', '.
+				CJs::encodeJson($timeControlData).')';
 		}
 		else {
 			if ($this->mode == SCREEN_MODE_SLIDESHOW) {
-				insert_js('timeControl.addObject("'.$this->getDataId().'", '.CJs::encodeJson($this->timeline).', '
-					.CJs::encodeJson($timeControlData).');'
+				insert_js('timeControl.addObject("'.$this->getDataId().'", '.CJs::encodeJson($this->timeline).', '.
+					CJs::encodeJson($timeControlData).');'
 				);
 			}
 			else {
-				zbx_add_post_js('timeControl.addObject("'.$this->getDataId().'", '.CJs::encodeJson($this->timeline).', '
-					.CJs::encodeJson($timeControlData).');'
+				zbx_add_post_js('timeControl.addObject("'.$this->getDataId().'", '.CJs::encodeJson($this->timeline).
+					', '.CJs::encodeJson($timeControlData).');'
 				);
 			}
 
@@ -185,10 +181,12 @@ class CScreenGraph extends CScreenBase {
 				$item = new CDiv();
 			}
 			elseif ($this->mode == SCREEN_MODE_PREVIEW) {
-				$item = new CLink(null, 'charts.php?graphid='.$resourceId.'&period='.$this->timeline['period'].
-						'&stime='.$this->timeline['stimeNow']);
+				$item = new CLink(null, 'charts.php?graphid='.$resourceId.'&from='.$this->timeline['from'].
+					'&to='.$this->timeline['to']
+				);
 			}
-			$item->setAttribute('id', $containerId);
+
+			$item->setId($containerId);
 
 			return $this->getOutput($item);
 		}
