@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -39,9 +39,7 @@ class CScreenSimpleGraph extends CScreenBase {
 			'id' => $this->getDataId(),
 			'containerid' => $containerid,
 			'objDims' => $graphDims,
-			'loadImage' => 1,
-			'periodFixed' => CProfile::get('web.screens.timelinefixed', 1),
-			'sliderMaximumTimePeriod' => ZBX_MAX_PERIOD
+			'loadImage' => 1
 		];
 
 		// host feature
@@ -51,8 +49,9 @@ class CScreenSimpleGraph extends CScreenBase {
 		}
 
 		if ($this->mode == SCREEN_MODE_PREVIEW && !empty($resourceid)) {
-			$this->action = 'history.php?action='.HISTORY_GRAPH.'&itemids[]='.$resourceid.'&period='.$this->timeline['period'].
-					'&stime='.$this->timeline['stimeNow'].$this->getProfileUrlParams();
+			$this->action = 'history.php?action='.HISTORY_GRAPH.'&itemids[]='.$resourceid.
+				'&from='.$this->timeline['from'].'&to='.$this->timeline['to'].
+				$this->getProfileUrlParams();
 		}
 
 		if ($resourceid && $this->mode != SCREEN_MODE_EDIT) {
@@ -66,8 +65,8 @@ class CScreenSimpleGraph extends CScreenBase {
 			: 'chart3.php?';
 
 		$timeControlData['src'] .= ($this->mode == SCREEN_MODE_EDIT)
-			? '&period=3600&stime='.date(TIMESTAMP_FORMAT, time())
-			: '&period='.$this->timeline['period'].'&stime='.$this->timeline['stimeNow'];
+			? '&from='.ZBX_PERIOD_DEFAULT_FROM.'&to='.ZBX_PERIOD_DEFAULT_TO
+			: '&from='.$this->timeline['from'].'&to='.$this->timeline['to'];
 
 		$timeControlData['src'] .= $this->getProfileUrlParams();
 
@@ -87,8 +86,9 @@ class CScreenSimpleGraph extends CScreenBase {
 				$item = new CDiv();
 			}
 			elseif ($this->mode == SCREEN_MODE_PREVIEW) {
-				$item = new CLink(null, 'history.php?action='.HISTORY_GRAPH.'&itemids[]='.$resourceid.'&period='.$this->timeline['period'].
-						'&stime='.$this->timeline['stimeNow']);
+				$item = new CLink(null, 'history.php?action='.HISTORY_GRAPH.'&itemids[]='.$resourceid.
+					'&from='.$this->timeline['from'].'&to='.$this->timeline['to']
+				);
 			}
 			$item->setId($containerid);
 
