@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -162,32 +162,30 @@ function get_icon($type, $params = []) {
 			return $icon;
 
 		case 'fullscreen':
-			$url = new CUrl();
+			switch (CView::getLayoutMode()) {
+				case ZBX_LAYOUT_KIOSKMODE:
+					$icon = (new CButton(null, '&nbsp;'))
+						->setTitle(_('Normal view'))
+						->setAttribute('data-layout-mode', ZBX_LAYOUT_NORMAL)
+						->addClass(ZBX_LAYOUT_MODE)
+						->addClass(ZBX_STYLE_BTN_DASHBRD_NORMAL)
+						->addClass(ZBX_STYLE_BTN_MIN);
+					break;
 
-			if ($params['fullscreen'] == 0) {
-				$url->setArgument('fullscreen', '1');
+				case ZBX_LAYOUT_FULLSCREEN:
+					$icon = (new CButton(null, '&nbsp;'))
+						->setTitle(_('Kiosk mode'))
+						->setAttribute('data-layout-mode', ZBX_LAYOUT_KIOSKMODE)
+						->addClass(ZBX_LAYOUT_MODE)
+						->addClass(ZBX_STYLE_BTN_KIOSK);
+					break;
 
-				$icon = (new CRedirectButton(SPACE, $url->getUrl()))
-					->setTitle(_('Fullscreen'))
-					->addClass(ZBX_STYLE_BTN_MAX);
-			}
-			else {
-				$url->setArgument('fullscreen', '0');
-
-				$icon = (new CRedirectButton(SPACE, $url->getUrl()))
-					->setTitle(_('Normal view'))
-					->addClass(ZBX_STYLE_BTN_MIN);
-			}
-
-			return $icon;
-
-		case 'dashconf':
-			$icon = (new CRedirectButton(SPACE, 'dashconf.php'))
-				->addClass(ZBX_STYLE_BTN_CONF)
-				->setTitle(_('Configure'));
-
-			if ($params['enabled']) {
-				$icon = [$icon, (new CDiv())->addClass(ZBX_STYLE_ACTIVE_INDIC)];
+				default:
+					$icon = (new CButton(null, '&nbsp;'))
+						->setTitle(_('Fullscreen'))
+						->setAttribute('data-layout-mode', ZBX_LAYOUT_FULLSCREEN)
+						->addClass(ZBX_LAYOUT_MODE)
+						->addClass(ZBX_STYLE_BTN_MAX);
 			}
 
 			return $icon;
@@ -195,20 +193,12 @@ function get_icon($type, $params = []) {
 		case 'screenconf':
 			return (new CRedirectButton(SPACE, null))
 				->addClass(ZBX_STYLE_BTN_CONF)
-				->setTitle(_('Refresh time'));
+				->setTitle(_('Refresh interval'));
 
 		case 'overviewhelp':
 			return (new CRedirectButton(SPACE, null))
 				->addClass(ZBX_STYLE_BTN_INFO);
-
-		case 'reset':
-			return (new CRedirectButton(SPACE, null))
-				->addClass(ZBX_STYLE_BTN_RESET)
-				->setTitle(_('Reset'))
-				->onClick('timeControl.objectReset();');
 	}
-
-	return null;
 }
 
 /**
@@ -290,7 +280,13 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 	/*
 	 * list and host (template) name
 	 */
-	$list = (new CList())->addClass(ZBX_STYLE_OBJECT_GROUP);
+	$list = (new CList())
+		->addClass(ZBX_STYLE_OBJECT_GROUP)
+		->addClass(ZBX_STYLE_FILTER_BREADCRUMB);
+
+	$breadcrumbs = (new CListItem(null))
+		->setAttribute('role', 'navigation')
+		->setAttribute('aria-label', _x('Hierarchy', 'screen reader'));
 
 	if ($is_template) {
 		$template = new CSpan(
@@ -301,7 +297,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 			$template->addClass(ZBX_STYLE_SELECTED);
 		}
 
-		$list->addItem([
+		$breadcrumbs->addItem([
 			new CSpan(
 				new CLink(_('All templates'), 'templates.php?templateid='.$db_host['templateid'].url_param('groupid'))
 			),
@@ -310,6 +306,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		]);
 
 		$db_host['hostid'] = $db_host['templateid'];
+		$list->addItem($breadcrumbs);
 	}
 	else {
 		$proxy_name = '';
@@ -348,19 +345,24 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 			$host->addClass(ZBX_STYLE_SELECTED);
 		}
 
-		$list->addItem([
+		$breadcrumbs->addItem([
 			new CSpan(new CLink(_('All hosts'), 'hosts.php?hostid='.$db_host['hostid'].url_param('groupid'))),
 			'/',
 			$host
 		]);
+		$list->addItem($breadcrumbs);
 		$list->addItem($status);
 		$list->addItem(getHostAvailabilityTable($db_host));
 
 		if ($db_host['flags'] == ZBX_FLAG_DISCOVERY_CREATED && $db_host['hostDiscovery']['ts_delete'] != 0) {
-			$list->addItem(getHostLifetimeIndicator(time(), $db_host['hostDiscovery']['ts_delete']));
+			$info_icons = [getHostLifetimeIndicator(time(), $db_host['hostDiscovery']['ts_delete'])];
+			$list->addItem(makeInformationList($info_icons));
 		}
 	}
 
+	$content_menu = (new CList())
+		->setAttribute('role', 'navigation')
+		->setAttribute('aria-label', _('Content menu'));
 	/*
 	 * the count of rows
 	 */
@@ -373,7 +375,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		if ($current_element == 'applications') {
 			$applications->addClass(ZBX_STYLE_SELECTED);
 		}
-		$list->addItem($applications);
+		$content_menu->addItem($applications);
 
 		// items
 		$items = new CSpan([
@@ -383,7 +385,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		if ($current_element == 'items') {
 			$items->addClass(ZBX_STYLE_SELECTED);
 		}
-		$list->addItem($items);
+		$content_menu->addItem($items);
 
 		// triggers
 		$triggers = new CSpan([
@@ -393,7 +395,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		if ($current_element == 'triggers') {
 			$triggers->addClass(ZBX_STYLE_SELECTED);
 		}
-		$list->addItem($triggers);
+		$content_menu->addItem($triggers);
 
 		// graphs
 		$graphs = new CSpan([
@@ -403,7 +405,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		if ($current_element == 'graphs') {
 			$graphs->addClass(ZBX_STYLE_SELECTED);
 		}
-		$list->addItem($graphs);
+		$content_menu->addItem($graphs);
 
 		// screens
 		if ($is_template) {
@@ -414,7 +416,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 			if ($current_element == 'screens') {
 				$screens->addClass(ZBX_STYLE_SELECTED);
 			}
-			$list->addItem($screens);
+			$content_menu->addItem($screens);
 		}
 
 		// discovery rules
@@ -425,7 +427,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		if ($current_element == 'discoveries') {
 			$lld_rules->addClass(ZBX_STYLE_SELECTED);
 		}
-		$list->addItem($lld_rules);
+		$content_menu->addItem($lld_rules);
 
 		// web scenarios
 		$http_tests = new CSpan([
@@ -435,7 +437,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		if ($current_element == 'web') {
 			$http_tests->addClass(ZBX_STYLE_SELECTED);
 		}
-		$list->addItem($http_tests);
+		$content_menu->addItem($http_tests);
 	}
 	else {
 		$discovery_rule = (new CSpan())->addItem(
@@ -465,7 +467,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		if ($current_element == 'items') {
 			$item_prototypes->addClass(ZBX_STYLE_SELECTED);
 		}
-		$list->addItem($item_prototypes);
+		$content_menu->addItem($item_prototypes);
 
 		// trigger prototypes
 		$trigger_prototypes = new CSpan([
@@ -477,7 +479,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		if ($current_element == 'triggers') {
 			$trigger_prototypes->addClass(ZBX_STYLE_SELECTED);
 		}
-		$list->addItem($trigger_prototypes);
+		$content_menu->addItem($trigger_prototypes);
 
 		// graph prototypes
 		$graph_prototypes = new CSpan([
@@ -487,7 +489,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		if ($current_element == 'graphs') {
 			$graph_prototypes->addClass(ZBX_STYLE_SELECTED);
 		}
-		$list->addItem($graph_prototypes);
+		$content_menu->addItem($graph_prototypes);
 
 		// host prototypes
 		if ($db_host['flags'] == ZBX_FLAG_DISCOVERY_NORMAL) {
@@ -498,32 +500,40 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 			if ($current_element == 'hosts') {
 				$host_prototypes->addClass(ZBX_STYLE_SELECTED);
 			}
-			$list->addItem($host_prototypes);
+			$content_menu->addItem($host_prototypes);
 		}
 	}
 
+	$list->addItem($content_menu);
+
 	return $list;
 }
 
 /**
- * Create CDiv with sysmap information
+ * Create breadcrumbs header object with sysmap parents information.
  *
- * @param int    $sysmapid
- * @param string $name
+ * @param int    $sysmapid      Used as value for sysmaid in map link generation.
+ * @param string $name          Used as label for map link generation.
+ * @param int    $severity_min  Used as value for severity_min in map link generation.
  *
  * @return object
  */
-function get_header_sysmap_table($sysmapid, $name, $fullscreen, $severity_min) {
+function get_header_sysmap_table($sysmapid, $name, $severity_min) {
 	$list = (new CList())
+		->setAttribute('role', 'navigation')
+		->setAttribute('aria-label', _x('Hierarchy', 'screen reader'))
 		->addClass(ZBX_STYLE_OBJECT_GROUP)
+		->addClass(ZBX_STYLE_FILTER_BREADCRUMB)
 		->addItem([
-			(new CSpan())->addItem(new CLink(_('All maps'), 'sysmaps.php')),
+			(new CSpan())->addItem(new CLink(_('All maps'), new CUrl('sysmaps.php'))),
 			'/',
 			(new CSpan())
 				->addClass(ZBX_STYLE_SELECTED)
 				->addItem(
-					new CLink($name, 'zabbix.php?action=map.view&sysmapid='.$sysmapid.'&fullscreen='.$fullscreen.
-						'&severity_min='.$severity_min
+					new CLink($name, (new CUrl('zabbix.php'))
+						->setArgument('action', 'map.view')
+						->setArgument('sysmapid', $sysmapid)
+						->setArgument('severity_min', $severity_min)
 					)
 				)
 		]);
@@ -531,18 +541,24 @@ function get_header_sysmap_table($sysmapid, $name, $fullscreen, $severity_min) {
 	// get map parent maps
 	$parent_sysmaps = get_parent_sysmaps($sysmapid);
 	if ($parent_sysmaps) {
-		$hor_list = new CHorList();
+		$parent_maps = (new CList())
+			->setAttribute('role', 'navigation')
+			->setAttribute('aria-label', _('Upper level maps'))
+			->addClass(ZBX_STYLE_OBJECT_GROUP);
 
 		foreach ($parent_sysmaps as $parent_sysmap) {
-			$hor_list->addItem(
-				new CLink($parent_sysmap['name'], 'zabbix.php?action=map.view'.
-					'&sysmapid='.$parent_sysmap['sysmapid'].'&fullscreen='.$fullscreen.'&severity_min='.$severity_min
+			$parent_maps->addItem(
+				new CLink($parent_sysmap['name'], (new CUrl('zabbix.php'))
+					->setArgument('action', 'map.view')
+					->setArgument('sysmapid', $parent_sysmap['sysmapid'])
+					->setArgument('severity_min', $severity_min)
 				)
 			);
 		}
 
-		$list->addItem(new CSpan(_('Upper level maps').':'));
-		$list->addItem($hor_list);
+		$list->addItem(_('Upper level maps').':');
+
+		return new CHorList([$list, $parent_maps]);
 	}
 
 	return $list;
@@ -568,7 +584,7 @@ function makeFormFooter(CButtonInterface $main_button = null, array $other_butto
 	}
 
 	return (new CList())
-		->addClass('table-forms')
+		->addClass(ZBX_STYLE_TABLE_FORMS)
 		->addItem([
 			(new CDiv())->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
 			(new CDiv($other_buttons))
@@ -593,10 +609,14 @@ function getHostAvailabilityTable($host) {
 				$ai = (new CSpan($type))->addClass(ZBX_STYLE_STATUS_GREEN);
 				break;
 			case HOST_AVAILABLE_FALSE:
-				$ai = (new CSpan($type))
-					->addClass(ZBX_STYLE_STATUS_RED)
-					->addClass(ZBX_STYLE_CURSOR_POINTER)
-					->setHint($host[$prefix.'error'], ZBX_STYLE_RED);
+				$ai = (new CSpan($type))->addClass(ZBX_STYLE_STATUS_RED);
+
+				if ($host[$prefix.'error'] !== '') {
+					$ai
+						->addClass(ZBX_STYLE_CURSOR_POINTER)
+						->setHint($host[$prefix.'error'], ZBX_STYLE_RED);
+				}
+
 				break;
 			case HOST_AVAILABLE_UNKNOWN:
 				$ai = (new CSpan($type))->addClass(ZBX_STYLE_STATUS_GREY);
@@ -626,7 +646,7 @@ function getHostGroupLifetimeIndicator($current_time, $ts_delete) {
 	else {
 		$warning = _s(
 			'The host group is not discovered anymore and will be deleted in %1$s (on %2$s at %3$s).',
-			zbx_date2age($ts_delete),
+			zbx_date2age($current_time, $ts_delete),
 			zbx_date2str(DATE_FORMAT, $ts_delete),
 			zbx_date2str(TIME_FORMAT, $ts_delete)
 		);
@@ -653,7 +673,34 @@ function getHostLifetimeIndicator($current_time, $ts_delete) {
 	else {
 		$warning = _s(
 			'The host is not discovered anymore and will be deleted in %1$s (on %2$s at %3$s).',
-			zbx_date2age($ts_delete),
+			zbx_date2age($current_time, $ts_delete),
+			zbx_date2str(DATE_FORMAT, $ts_delete),
+			zbx_date2str(TIME_FORMAT, $ts_delete)
+		);
+	}
+
+	return makeWarningIcon($warning);
+}
+
+/**
+ * Returns the discovered application lifetime indicator.
+ *
+ * @param string $current_time	current Unix timestamp
+ * @param array  $ts_delete		deletion timestamp of the application
+ *
+ * @return CDiv
+ */
+function getApplicationLifetimeIndicator($current_time, $ts_delete) {
+	// Check if the element should've been deleted in the past.
+	if ($current_time > $ts_delete) {
+		$warning = _(
+			'The application is not discovered anymore and will be deleted the next time discovery rule is processed.'
+		);
+	}
+	else {
+		$warning = _s(
+			'The application is not discovered anymore and will be deleted in %1$s (on %2$s at %3$s).',
+			zbx_date2age($current_time, $ts_delete),
 			zbx_date2str(DATE_FORMAT, $ts_delete),
 			zbx_date2str(TIME_FORMAT, $ts_delete)
 		);
@@ -680,7 +727,7 @@ function getItemLifetimeIndicator($current_time, $ts_delete) {
 	else {
 		$warning = _s(
 			'The item is not discovered anymore and will be deleted in %1$s (on %2$s at %3$s).',
-			zbx_date2age($ts_delete),
+			zbx_date2age($current_time, $ts_delete),
 			zbx_date2str(DATE_FORMAT, $ts_delete),
 			zbx_date2str(TIME_FORMAT, $ts_delete)
 		);
@@ -689,104 +736,15 @@ function getItemLifetimeIndicator($current_time, $ts_delete) {
 	return makeWarningIcon($warning);
 }
 
-/**
- * Create array with all inputs required for date selection and calendar.
- *
- * @param string      $name
- * @param int|array   $date unix timestamp/date array(Y,m,d,H,i)
- * @param string|null $relatedCalendar name of the calendar which must be closed when this calendar opens
- *
- * @return array
- */
-function createDateSelector($name, $date, $relatedCalendar = null) {
-	$onClick = 'var pos = getPosition(this); pos.top += 10; pos.left += 16; CLNDR["'.$name.
-		'_calendar"].clndr.clndrshow(pos.top, pos.left);';
-	if ($relatedCalendar) {
-		$onClick .= ' CLNDR["'.$relatedCalendar.'_calendar"].clndr.clndrhide();';
-	}
-
-	if (is_array($date)) {
-		$y = $date['y'];
-		$m = $date['m'];
-		$d = $date['d'];
-		$h = $date['h'];
-		$i = $date['i'];
-	}
-	else {
-		$y = date('Y', $date);
-		$m = date('m', $date);
-		$d = date('d', $date);
-		$h = date('H', $date);
-		$i = date('i', $date);
-	}
-
-	$fields = [
-		(new CNumericBox($name.'_year', $y, 4))
-			->setWidth(ZBX_TEXTAREA_4DIGITS_WIDTH)
-			->setAttribute('placeholder', _('yyyy')),
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		'-',
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		(new CTextBox($name.'_month', $m, false, 2))
-			->setWidth(ZBX_TEXTAREA_2DIGITS_WIDTH)
-			->addStyle('text-align: right;')
-			->setAttribute('placeholder', _('mm'))
-			->onChange('validateDatePartBox(this, 1, 12, 2);'),
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		'-',
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		(new CTextBox($name.'_day', $d, false, 2))
-			->setWidth(ZBX_TEXTAREA_2DIGITS_WIDTH)
-			->addStyle('text-align: right;')
-			->setAttribute('placeholder', _('dd'))
-			->onChange('validateDatePartBox(this, 1, 31, 2);'),
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		(new CTextBox($name.'_hour', $h, false, 2))
-			->setWidth(ZBX_TEXTAREA_2DIGITS_WIDTH)
-			->addStyle('text-align: right;')
-			->setAttribute('placeholder', _('hh'))
-			->onChange('validateDatePartBox(this, 0, 23, 2);'),
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		':',
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		(new CTextBox($name.'_minute', $i, false, 2))
-			->setWidth(ZBX_TEXTAREA_2DIGITS_WIDTH)
-			->addStyle('text-align: right;')
-			->setAttribute('placeholder', _('mm'))
-			->onChange('validateDatePartBox(this, 0, 59, 2);'),
-		(new CButton())
-			->addClass(ZBX_STYLE_ICON_CAL)
-			->onClick($onClick)
-	];
-
-	zbx_add_post_js('create_calendar(null,'.
-		'["'.$name.'_day","'.$name.'_month","'.$name.'_year","'.$name.'_hour","'.$name.'_minute"],'.
-		'"'.$name.'_calendar",'.
-		'"'.$name.'");'
-	);
-
-	return $fields;
-}
-
 /**
  * Renders a page footer.
  *
- * @param bool $with_logo
  * @param bool $with_version
  *
  * @return CDiv
  */
-function makePageFooter($with_version = true)
-{
-	return (new CDiv([
-		$with_version ? 'Zabbix '.ZABBIX_VERSION.'. ' : null,
-		'&copy; '.ZABBIX_COPYRIGHT_FROM.'&ndash;'.ZABBIX_COPYRIGHT_TO.', ',
-		(new CLink('Zabbix SIA', 'http://www.zabbix.com/'))
-			->addClass(ZBX_STYLE_GREY)
-			->addClass(ZBX_STYLE_LINK_ALT)
-			->setAttribute('target', '_blank')
-	]))->addClass(ZBX_STYLE_FOOTER);
+function makePageFooter($with_version = true) {
+	return (new CTag('footer', true, CBrandHelper::getFooterContent($with_version)))->setAttribute('role', 'contentinfo');
 }
 
 /**
@@ -796,8 +754,7 @@ function makePageFooter($with_version = true)
  *
  * @return CComboBox
  */
-function makeAdministrationGeneralMenu($selected)
-{
+function makeAdministrationGeneralMenu($selected) {
 	return new CComboBox('configDropDown', $selected, 'redirect(this.options[this.selectedIndex].value);', [
 		'adm.gui.php' => _('GUI'),
 		'adm.housekeeper.php' => _('Housekeeping'),
@@ -814,47 +771,157 @@ function makeAdministrationGeneralMenu($selected)
 }
 
 /**
- * Renders an error icon like [x] with error message
+ * Renders an icon list.
+ *
+ * @param array $info_icons  The list of information icons.
+ *
+ * @return CDiv|string
+ */
+function makeInformationList($info_icons) {
+	return $info_icons ? (new CDiv($info_icons))->addClass(ZBX_STYLE_REL_CONTAINER) : '';
+}
+
+/**
+ * Renders an information icon like green [i] with message.
+ *
+ * @param string $message
+ *
+ * @return CSpan
+ */
+function makeInformationIcon($message) {
+	return (new CSpan())
+		->addClass(ZBX_STYLE_ICON_INFO)
+		->addClass(ZBX_STYLE_STATUS_GREEN)
+		->setHint($message);
+}
+
+/**
+ * Renders an icon for host in maintenance.
+ *
+ * @param int    $type         Type of the maintenance.
+ * @param string $name         Name of the maintenance.
+ * @param string $description  Description of the maintenance.
+ *
+ * @return CSpan
+ */
+function makeMaintenanceIcon($type, $name, $description) {
+	$hint = $name.' ['.($type
+		? _('Maintenance without data collection')
+		: _('Maintenance with data collection')).']';
+
+	if ($description !== '') {
+		$hint .= "\n".$description;
+	}
+
+	return (new CSpan())
+		->addClass(ZBX_STYLE_ICON_MAINT)
+		->addClass(ZBX_STYLE_CURSOR_POINTER)
+		->setHint($hint);
+}
+
+/**
+ * Renders an icon for suppressed problem.
+ *
+ * @param array  $icon_data
+ * @param string $icon_data[]['suppress_until']    Time until the problem is suppressed.
+ * @param string $icon_data[]['maintenance_name']  Name of the maintenance.
+ *
+ * @return CSpan
+ */
+function makeSuppressedProblemIcon(array $icon_data) {
+	$suppress_until = max(zbx_objectValues($icon_data, 'suppress_until'));
+
+	CArrayHelper::sort($icon_data, ['maintenance_name']);
+	$maintenance_names = implode(', ', zbx_objectValues($icon_data, 'maintenance_name'));
+
+	return (new CSpan())
+		->addClass(ZBX_STYLE_ICON_INVISIBLE)
+		->addClass(ZBX_STYLE_CURSOR_POINTER)
+		->setHint(
+			_s('Suppressed till: %1$s', ($suppress_until < strtotime('tomorrow'))
+				? zbx_date2str(TIME_FORMAT, $suppress_until)
+				: zbx_date2str(DATE_TIME_FORMAT, $suppress_until)
+			).
+			"\n".
+			_s('Maintenance: %1$s', $maintenance_names)
+		);
+}
+
+/**
+ * Renders an action icon.
+ *
+ * @param array  $icon_data
+ * @param string $icon_data[icon]  Icon style.
+ * @param array  $icon_data[hint]  Hintbox content (optional).
+ * @param int    $icon_data[num]   Number displayed over the icon (optional).
+ *
+ * @return CSpan
+ */
+function makeActionIcon(array $icon_data) {
+	$icon = (new CSpan())->addClass($icon_data['icon']);
+
+	if (array_key_exists('num', $icon_data)) {
+		if ($icon_data['num'] > 99) {
+			$icon_data['num'] = '99+';
+		}
+		$icon->setAttribute('data-count', $icon_data['num']);
+	}
+
+	if (array_key_exists('hint', $icon_data)) {
+		$icon->setHint($icon_data['hint'], '', true, 'max-width: '.ZBX_ACTIONS_POPUP_MAX_WIDTH.'px;');
+	}
+	elseif (array_key_exists('title', $icon_data)) {
+		$icon->setTitle($icon_data['title']);
+	}
+
+	if (array_key_exists('aria-label', $icon_data)) {
+		$icon
+			->addItem($icon_data['aria-label'])
+			->addClass(ZBX_STYLE_INLINE_SR_ONLY);
+	}
+
+	return $icon;
+}
+
+/**
+ * Renders an error icon like red [i] with error message
  *
  * @param string $error
  *
  * @return CSpan
  */
-function makeErrorIcon($error)
-{
-	return (new CSpan(bold('&times;')))
+function makeErrorIcon($error) {
+	return (new CSpan())
+		->addClass(ZBX_STYLE_ICON_INFO)
 		->addClass(ZBX_STYLE_STATUS_RED)
-		->addClass(ZBX_STYLE_CURSOR_POINTER)
 		->setHint($error, ZBX_STYLE_RED);
 }
 
 /**
- * Renders an unknown icon like [?] with error message
+ * Renders an unknown icon like grey [i] with error message
  *
  * @param string $error
  *
  * @return CSpan
  */
-function makeUnknownIcon($error)
-{
-	return (new CSpan(bold('?')))
-		->addClass(ZBX_STYLE_STATUS_GREY)
-		->addClass(ZBX_STYLE_CURSOR_POINTER)
+function makeUnknownIcon($error) {
+	return (new CSpan())
+		->addClass(ZBX_STYLE_ICON_INFO)
+		->addClass(ZBX_STYLE_STATUS_DARK_GREY)
 		->setHint($error, ZBX_STYLE_RED);
 }
 
 /**
- * Renders a warning icon like [!] with error message
+ * Renders a warning icon like yellow [i] with error message
  *
  * @param string $error
  *
  * @return CSpan
  */
-function makeWarningIcon($error)
-{
-	return (new CSpan(bold('!')))
+function makeWarningIcon($error) {
+	return (new CSpan())
+		->addClass(ZBX_STYLE_ICON_INFO)
 		->addClass(ZBX_STYLE_STATUS_YELLOW)
-		->addClass(ZBX_STYLE_CURSOR_POINTER)
 		->setHint($error);
 }
 
@@ -863,8 +930,7 @@ function makeWarningIcon($error)
  *
  * @return CButton
  */
-function makeDebugButton()
-{
+function makeDebugButton() {
 	return (new CDiv(
 		(new CLink(_('Debug'), '#debug'))
 			->onClick("javascript: if (!isset('state', this)) { this.state = 'none'; }".
@@ -878,7 +944,7 @@ function makeDebugButton()
 }
 
 /**
- * Returns css for trigger severity backgrounds
+ * Returns css for trigger severity backgrounds.
  *
  * @param array $config
  * @param array $config[severity_color_0]
@@ -890,8 +956,9 @@ function makeDebugButton()
  *
  * @return string
  */
-function getTriggerSeverityCss($config)
-{
+function getTriggerSeverityCss($config) {
+	$css = '';
+
 	$severities = [
 		ZBX_STYLE_NA_BG => $config['severity_color_0'],
 		ZBX_STYLE_INFO_BG => $config['severity_color_1'],
@@ -901,10 +968,40 @@ function getTriggerSeverityCss($config)
 		ZBX_STYLE_DISASTER_BG => $config['severity_color_5']
 	];
 
-	$css = '';
-
 	foreach ($severities as $class => $color) {
-		$css .= '.'.$class.', .'.$class.' input[type="radio"]:checked + label { background-color: #'.$color.' }'."\n";
+		$css .= '.'.$class.', .'.$class.' input[type="radio"]:checked + label, .'.$class.':before, .flh-'.$class.
+			', .status-'.$class.' { background-color: #'.$color.' }'."\n";
+	}
+
+	return $css;
+}
+
+/**
+ * Returns css for trigger status colors, if those are customized.
+ *
+ * @param array $config
+ * @param array $config[custom_color]
+ * @param array $config[problem_unack_color]
+ * @param array $config[problem_ack_color]
+ * @param array $config[ok_unack_color]
+ * @param array $config[ok_ack_color]
+ *
+ * @return string
+ */
+function getTriggerStatusCss($config) {
+	$css = '';
+
+	if ($config['custom_color'] == EVENT_CUSTOM_COLOR_ENABLED) {
+		$event_statuses = [
+			ZBX_STYLE_PROBLEM_UNACK_FG => $config['problem_unack_color'],
+			ZBX_STYLE_PROBLEM_ACK_FG => $config['problem_ack_color'],
+			ZBX_STYLE_OK_UNACK_FG => $config['ok_unack_color'],
+			ZBX_STYLE_OK_ACK_FG => $config['ok_ack_color']
+		];
+
+		foreach ($event_statuses as $class => $color) {
+			$css .= '.' . $class . ' {color: #' . $color . ';}' . "\n";
+		}
 	}
 
 	return $css;
