@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -82,6 +82,7 @@ function user_auth_type2str($authType) {
 	$authUserType = [
 		GROUP_GUI_ACCESS_SYSTEM => _('System default'),
 		GROUP_GUI_ACCESS_INTERNAL => _x('Internal', 'user type'),
+		GROUP_GUI_ACCESS_LDAP => _('LDAP'),
 		GROUP_GUI_ACCESS_DISABLED => _('Disabled')
 	];
 
@@ -141,74 +142,6 @@ function granted2update_group($userGroupIds) {
 	return !isset($users[CWebUser::$data['userid']]);
 }
 
-/**
- * Change group status.
- *
- * @param array $userGroupIds
- * @param int   $usersStatus
- *
- * @return bool
- */
-function change_group_status($userGroupIds, $usersStatus) {
-	zbx_value2array($userGroupIds);
-
-	$grant = ($usersStatus == GROUP_STATUS_DISABLED) ? granted2update_group($userGroupIds) : true;
-
-	if ($grant) {
-		return DBexecute(
-			'UPDATE usrgrp'.
-			' SET users_status='.zbx_dbstr($usersStatus).
-			' WHERE '.dbConditionInt('usrgrpid', $userGroupIds)
-		);
-	}
-	else {
-		error(_('User cannot change status of himself.'));
-	}
-
-	return false;
-}
-
-/**
- * Change gui access for group.
- *
- * @param array $userGroupIds
- * @param int   $guiAccess
- *
- * @return bool
- */
-function change_group_gui_access($userGroupIds, $guiAccess) {
-	zbx_value2array($userGroupIds);
-
-	$grant = ($guiAccess == GROUP_GUI_ACCESS_DISABLED) ? granted2update_group($userGroupIds) : true;
-
-	if ($grant) {
-		return DBexecute(
-			'UPDATE usrgrp SET gui_access='.zbx_dbstr($guiAccess).' WHERE '.dbConditionInt('usrgrpid', $userGroupIds)
-		);
-	}
-	else {
-		error(_('User cannot change GUI access for himself.'));
-	}
-
-	return false;
-}
-
-/**
- * Change debug mode for group.
- *
- * @param array $userGroupIds
- * @param int   $debugMode
- *
- * @return bool
- */
-function change_group_debug_mode($userGroupIds, $debugMode) {
-	zbx_value2array($userGroupIds);
-
-	return DBexecute(
-		'UPDATE usrgrp SET debug_mode='.zbx_dbstr($debugMode).' WHERE '.dbConditionInt('usrgrpid', $userGroupIds)
-	);
-}
-
 /**
  * Gets user full name in format "alias (name surname)". If both name and surname exist, returns translated string.
  *
@@ -222,7 +155,7 @@ function change_group_debug_mode($userGroupIds, $debugMode) {
 function getUserFullname($userData) {
 	if (!zbx_empty($userData['surname'])) {
 		if (!zbx_empty($userData['name'])) {
-			return $userData['alias'].' '._x('(%1$s %2$s)', 'user fullname', $userData['name'], $userData['surname']);
+			return $userData['alias'].' '._xs('(%1$s %2$s)', 'user fullname', $userData['name'], $userData['surname']);
 		}
 
 		$fullname = $userData['surname'];
@@ -237,7 +170,7 @@ function getUserFullname($userData) {
 /**
  * Returns the list of permissions to the host groups for selected user groups.
  *
- * @param string $usrgrpid
+ * @param array $usrgrpids		An array of user group IDs.
  *
  * @return array
  */
@@ -334,6 +267,74 @@ function collapseHostGroupRights(array $groups_rights) {
 	return $groups_rights;
 }
 
+/**
+ * Returns the sorted list of the unique tag filters.
+ *
+ * @param array  $tag_filters
+ *
+ * @return array
+ */
+function uniqTagFilters(array $tag_filters) {
+	CArrayHelper::sort($tag_filters, ['groupid', 'tag', 'value']);
+
+	$prev_tag_filter = null;
+
+	foreach ($tag_filters as $key => $tag_filter) {
+		if ($prev_tag_filter !== null && $prev_tag_filter['groupid'] == $tag_filter['groupid']
+				&& ($prev_tag_filter['tag'] === '' || $prev_tag_filter['tag'] === $tag_filter['tag'])
+				&& ($prev_tag_filter['value'] === '' || $prev_tag_filter['value'] === $tag_filter['value'])) {
+			unset($tag_filters[$key]);
+		}
+		else {
+			$prev_tag_filter = $tag_filter;
+		}
+	}
+
+	return $tag_filters;
+}
+
+/**
+ * Returns the sorted list of the unique tag filters and group names.
+ * The list will be enriched by group names. Tag filters with filled tags or values will be overwritten empty.
+ *
+ * @param array  $tag_filters
+ * @param string $tag_filters[]['groupid']
+ * @param string $tag_filters[]['tag']
+ * @param string $tag_filters[]['value']
+ *
+ * @return array
+ */
+function collapseTagFilters(array $tag_filters) {
+	$tag_filters = uniqTagFilters($tag_filters);
+
+	$groupids = [];
+
+	foreach ($tag_filters as $tag_filter) {
+		$groupids[$tag_filter['groupid']] = true;
+	}
+
+	if ($groupids) {
+		$groups = API::HostGroup()->get([
+			'output' => ['name'],
+			'groupids' => array_keys($groupids),
+			'preservekeys' => true
+		]);
+
+		foreach ($tag_filters as $key => $tag_filter) {
+			if (array_key_exists($tag_filter['groupid'], $groups)) {
+				$tag_filters[$key]['name'] = $groups[$tag_filter['groupid']]['name'];
+			}
+			else {
+				unset($tag_filters[$key]);
+			}
+		}
+
+		CArrayHelper::sort($tag_filters, ['name', 'tag', 'value']);
+	}
+
+	return $tag_filters;
+}
+
 /**
  * Applies new permissions to the host groups.
  *
