@@ -22,23 +22,27 @@
 $widget = new CWidget();
 
 if ($data['parent_discoveryid'] === null) {
-	$widget->setTitle(_('Graphs'))->addItem(get_header_host_table('graphs', $data['hostid']));
+	$widget
+		->setTitle(_('Graphs'))
+		->addItem(get_header_host_table('graphs', $data['hostid']));
 }
 else {
-	$widget->setTitle(_('Graph prototypes'))
+	$widget
+		->setTitle(_('Graph prototypes'))
 		->addItem(get_header_host_table('graphs', $data['hostid'], $data['parent_discoveryid']));
 }
 
 // Create form.
 $graphForm = (new CForm())
 	->setName('graphForm')
+	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('form', $this->data['form'])
 	->addVar('hostid', $this->data['hostid'])
 	->addVar('ymin_itemid', $this->data['ymin_itemid'])
 	->addVar('ymax_itemid', $this->data['ymax_itemid']);
 
 if ($data['parent_discoveryid'] !== null) {
-	$graphForm->addVar('parent_discoveryid', $data['parent_discoveryid']);
+	$graphForm->addItem((new CVar('parent_discoveryid', $data['parent_discoveryid']))->removeId());
 }
 
 if ($data['graphid'] != 0) {
@@ -70,20 +74,24 @@ if ($discovered_graph) {
 }
 
 $graphFormList
-	->addRow(_('Name'),
+	->addRow(
+		(new CLabel(_('Name'), 'name'))->setAsteriskMark(),
 		(new CTextBox('name', $this->data['name'], $readonly))
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired()
 			->setAttribute('autofocus', 'autofocus')
 	)
-	->addRow(_('Width'),
+	->addRow((new CLabel(_('Width'), 'width'))->setAsteriskMark(),
 		(new CNumericBox('width', $this->data['width'], 5, $readonly))
 			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+			->setAriaRequired()
 	)
-	->addRow(_('Height'),
+	->addRow((new CLabel(_('Height'), 'height'))->setAsteriskMark(),
 		(new CNumericBox('height', $this->data['height'], 5, $readonly))
 			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+			->setAriaRequired()
 	)
-	->addRow(_('Graph type'),
+	->addRow((new CLabel(_('Graph type'), 'graphtype')),
 		(new CComboBox('graphtype', $this->data['graphtype'], 'submit()', graphType()))->setEnabled(!$readonly)
 	)
 	->addRow(_('Show legend'),
@@ -168,21 +176,28 @@ if ($this->data['graphtype'] == GRAPH_TYPE_NORMAL || $this->data['graphtype'] ==
 		}
 
 		$yaxisMinData[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
-		$yaxisMinData[] = (new CTextBox('ymin_name', $ymin_name, $readonly))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
+		$yaxisMinData[] = (new CTextBox('ymin_name', $ymin_name, $readonly))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired();
 		$yaxisMinData[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 
 		// Select item button.
 		$yaxisMinData[] = (new CButton('yaxis_min', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: '.
-				'return PopUp("popup.php?dstfrm='.$graphForm->getName().
-					'&dstfld1=ymin_itemid'.
-					'&dstfld2=ymin_name'.
-					'&srctbl=items'.
-					'&srcfld1=itemid'.
-					'&srcfld2=name'.
-					'&numeric=1'.
-					'&writeonly=1" + getOnlyHostParam(), 0, 0, "zbx_popup_item");')
+			->onClick('return PopUp("popup.generic",jQuery.extend('.
+				CJs::encodeJson([
+					'srctbl' => 'items',
+					'srcfld1' => 'itemid',
+					'srcfld2' => 'name',
+					'dstfrm' => $graphForm->getName(),
+					'dstfld1' => 'ymin_itemid',
+					'dstfld2' => 'ymin_name',
+					'with_webitems' => '1',
+					'numeric' => '1',
+					'writeonly' => '1'
+				]).
+					',getOnlyHostParam()), null, this);'
+			)
 			->setEnabled(!$readonly);
 
 		// Select item prototype button.
@@ -190,22 +205,32 @@ if ($this->data['graphtype'] == GRAPH_TYPE_NORMAL || $this->data['graphtype'] ==
 			$yaxisMinData[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 			$yaxisMinData[] = (new CButton('yaxis_min_prototype', _('Select prototype')))
 				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick('javascript: '.
-					'return PopUp("popup.php?dstfrm='.$graphForm->getName().
-						'&parent_discoveryid='.$data['parent_discoveryid'].
-						'&dstfld1=ymin_itemid'.
-						'&dstfld2=ymin_name'.
-						'&srctbl=item_prototypes'.
-						'&srcfld1=itemid'.
-						'&srcfld2=name'.
-						'&numeric=1", 0, 0, "zbx_popup_item");');
+				->onClick('return PopUp("popup.generic",'.
+					CJs::encodeJson([
+						'srctbl' => 'item_prototypes',
+						'srcfld1' => 'itemid',
+						'srcfld2' => 'name',
+						'dstfrm' => $graphForm->getName(),
+						'dstfld1' => 'ymin_itemid',
+						'dstfld2' => 'ymin_name',
+						'parent_discoveryid' => $data['parent_discoveryid'],
+						'numeric' => '1'
+					]).', null, this);'
+				);
 		}
 	}
 	else {
 		$graphForm->addVar('yaxismin', $this->data['yaxismin']);
 	}
 
-	$graphFormList->addRow(_('Y axis MIN value'), $yaxisMinData);
+	$yaxismin_label = new CLabel(_('Y axis MIN value'));
+	if ($this->data['ymin_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE) {
+		$yaxismin_label
+			->setAsteriskMark()
+			->setAttribute('for', 'ymin_name');
+	}
+
+	$graphFormList->addRow($yaxismin_label, $yaxisMinData);
 
 	$yaxisMaxData = [(new CComboBox('ymax_type', $this->data['ymax_type'], null, [
 		GRAPH_YAXIS_TYPE_CALCULATED => _('Calculated'),
@@ -232,21 +257,27 @@ if ($this->data['graphtype'] == GRAPH_TYPE_NORMAL || $this->data['graphtype'] ==
 		}
 
 		$yaxisMaxData[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
-		$yaxisMaxData[] = (new CTextBox('ymax_name', $ymax_name, $readonly))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
+		$yaxisMaxData[] = (new CTextBox('ymax_name', $ymax_name, $readonly))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired();
 		$yaxisMaxData[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 
 		// Select item button.
 		$yaxisMaxData[] = (new CButton('yaxis_max', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: '.
-				'return PopUp("popup.php?dstfrm='.$graphForm->getName().
-					'&dstfld1=ymax_itemid'.
-					'&dstfld2=ymax_name'.
-					'&srctbl=items'.
-					'&srcfld1=itemid'.
-					'&srcfld2=name'.
-					'&numeric=1'.
-					'&writeonly=1" + getOnlyHostParam(), 0, 0, "zbx_popup_item");'
+			->onClick('return PopUp("popup.generic",jQuery.extend('.
+				CJs::encodeJson([
+					'srctbl' => 'items',
+					'srcfld1' => 'itemid',
+					'srcfld2' => 'name',
+					'dstfrm' => $graphForm->getName(),
+					'dstfld1' => 'ymax_itemid',
+					'dstfld2' => 'ymax_name',
+					'with_webitems' => '1',
+					'numeric' => '1',
+					'writeonly' => '1'
+				]).
+					',getOnlyHostParam()), null, this);'
 			)
 			->setEnabled(!$readonly);
 
@@ -255,15 +286,17 @@ if ($this->data['graphtype'] == GRAPH_TYPE_NORMAL || $this->data['graphtype'] ==
 			$yaxisMaxData[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 			$yaxisMaxData[] = (new CButton('yaxis_max_prototype', _('Select prototype')))
 				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick('javascript: '.
-					'return PopUp("popup.php?dstfrm='.$graphForm->getName().
-						'&parent_discoveryid='.$data['parent_discoveryid'].
-						'&dstfld1=ymax_itemid'.
-						'&dstfld2=ymax_name'.
-						'&srctbl=item_prototypes'.
-						'&srcfld1=itemid'.
-						'&srcfld2=name'.
-						'&numeric=1", 0, 0, "zbx_popup_item");'
+				->onClick('return PopUp("popup.generic",'.
+					CJs::encodeJson([
+						'srctbl' => 'item_prototypes',
+						'srcfld1' => 'itemid',
+						'srcfld2' => 'name',
+						'dstfrm' => $graphForm->getName(),
+						'dstfld1' => 'ymax_itemid',
+						'dstfld2' => 'ymax_name',
+						'parent_discoveryid' => $data['parent_discoveryid'],
+						'numeric' => '1'
+					]).', null, this);'
 				);
 		}
 	}
@@ -271,7 +304,14 @@ if ($this->data['graphtype'] == GRAPH_TYPE_NORMAL || $this->data['graphtype'] ==
 		$graphForm->addVar('yaxismax', $this->data['yaxismax']);
 	}
 
-	$graphFormList->addRow(_('Y axis MAX value'), $yaxisMaxData);
+	$yaxismax_label = new CLabel(_('Y axis MAX value'));
+	if ($this->data['ymax_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE) {
+		$yaxismax_label
+			->setAsteriskMark()
+			->setAttribute('for', 'ymax_name');
+	}
+
+	$graphFormList->addRow($yaxismax_label, $yaxisMaxData);
 }
 else {
 	$graphFormList->addRow(_('3D view'),
@@ -282,7 +322,7 @@ else {
 }
 
 // Append items to form list.
-$itemsTable = (new CTable())
+$items_table = (new CTable())
 	->setId('itemsTable')
 	->setHeader([
 		(new CColHeader())->setWidth(15),
@@ -306,28 +346,56 @@ $itemsTable = (new CTable())
 		$readonly ? null : (new CColHeader(_('Action')))->setWidth(50)
 	]);
 
-$itemsTable->addRow(
+$popup_options_add = [
+	'srctbl' => 'items',
+	'srcfld1' => 'itemid',
+	'srcfld2' => 'name',
+	'dstfrm' => $graphForm->getName(),
+	'numeric' => '1',
+	'writeonly' => '1',
+	'multiselect' => '1',
+	'with_webitems' => '1'
+];
+if ($data['normal_only']) {
+	$popup_options_add['normal_only'] = '1';
+}
+if ($data['groupid'] && $data['hostid']) {
+	$popup_options_add['groupid'] = $data['groupid'];
+	$popup_options_add['hostid'] = $data['hostid'];
+}
+
+$popup_options_add_prototype = [
+	'srctbl' => 'item_prototypes',
+	'srcfld1' => 'itemid',
+	'srcfld2' => 'name',
+	'dstfrm' => $graphForm->getName(),
+	'numeric' => '1',
+	'writeonly' => '1',
+	'multiselect' => '1',
+	'graphtype' => $data['graphtype']
+];
+if ($data['normal_only']) {
+	$popup_options_add_prototype['normal_only'] = '1';
+}
+if ($data['parent_discoveryid']) {
+	$popup_options_add_prototype['parent_discoveryid'] = $data['parent_discoveryid'];
+}
+
+$items_table->addRow(
 	(new CRow(
 		$readonly
 			? null
 			: (new CCol(
 				new CHorList([
 					(new CButton('add_item', _('Add')))
-						->onClick('return PopUp("popup.php?writeonly=1&multiselect=1&dstfrm='.$graphForm->getName().
-							($this->data['normal_only'] ? '&normal_only=1' : '').
-							(($data['groupid'] && $data['hostid'])
-								? '&groupid='.$data['groupid'].'&hostid='.$data['hostid']
-								: ''
-							).
-							'&srctbl=items&srcfld1=itemid&srcfld2=name&numeric=1" + getOnlyHostParam());'
+						->onClick('return PopUp("popup.generic",jQuery.extend('.
+							CJs::encodeJson($popup_options_add).',getOnlyHostParam()), null, this);'
 						)
 						->addClass(ZBX_STYLE_BTN_LINK),
-					$this->data['parent_discoveryid']
+					$data['parent_discoveryid']
 						? (new CButton('add_protoitem', _('Add prototype')))
-							->onClick('return PopUp("popup.php?writeonly=1&multiselect=1&dstfrm='.$graphForm->getName().
-								url_param($this->data['graphtype'], false, 'graphtype').
-								url_param('parent_discoveryid').($this->data['normal_only'] ? '&normal_only=1' : '').
-								'&srctbl=item_prototypes&srcfld1=itemid&srcfld2=name&numeric=1");'
+							->onClick('return PopUp("popup.generic",'.
+								CJs::encodeJson($popup_options_add_prototype).', null, this);'
 							)
 							->addClass(ZBX_STYLE_BTN_LINK)
 						: null
@@ -354,16 +422,15 @@ foreach ($this->data['items'] as $n => $item) {
 	);
 }
 
-$graphFormList->addRow(_('Items'), (new CDiv($itemsTable))->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR));
+$graphFormList->addRow(
+	(new CLabel(_('Items'), $items_table->getId()))->setAsteriskMark(),
+	(new CDiv($items_table))->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+);
 
 // Append tabs to form.
-$graphTab = new CTabView();
-if (!$data['form_refresh']) {
-	$graphTab->setSelected(0);
-}
-$graphTab->addTab('graphTab', ($data['parent_discoveryid'] === null) ? _('Graph') : _('Graph prototype'),
-	$graphFormList
-);
+$graphTab = (new CTabView())
+	->setSelected(0)
+	->addTab('graphTab', ($data['parent_discoveryid'] === null) ? _('Graph') : _('Graph prototype'), $graphFormList);
 
 /*
  * Preview tab
@@ -409,7 +476,6 @@ else {
 }
 
 // Insert js (depended from some variables inside the file).
-insert_show_color_picker_javascript();
 require_once dirname(__FILE__).'/js/configuration.graph.edit.js.php';
 
 $graphForm->addItem($graphTab);
