@@ -21,16 +21,17 @@
 
 $itemWidget = (new CWidget())->setTitle(_('Items'));
 
-if (!empty($this->data['hostid'])) {
-	$itemWidget->addItem(get_header_host_table('items', $this->data['hostid']));
+if ($data['hostid'] != 0) {
+	$itemWidget->addItem(get_header_host_table('items', $data['hostid']));
 }
 
 // create form
 $itemForm = (new CForm())
 	->setName('itemForm')
-	->addVar('group_itemid', $this->data['itemids'])
-	->addVar('hostid', $this->data['hostid'])
-	->addVar('action', $this->data['action']);
+	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
+	->addVar('group_itemid', $data['itemids'])
+	->addVar('hostid', $data['hostid'])
+	->addVar('action', $data['action']);
 
 // create form list
 $itemFormList = new CFormList('itemFormList');
@@ -39,17 +40,18 @@ $itemFormList = new CFormList('itemFormList');
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[type]', 'type', _('Original')))
 		->setLabel(_('Type'))
-		->setChecked(isset($this->data['visible']['type'])),
-	new CComboBox('type', $this->data['type'], null, $this->data['itemTypes'])
+		->setChecked(isset($data['visible']['type']))
+		->setAttribute('autofocus', 'autofocus'),
+	new CComboBox('type', $data['type'], null, $data['itemTypes'])
 );
 
 // append hosts to form list
-if ($this->data['displayInterfaces']) {
-	$interfacesComboBox = new CComboBox('interfaceid', $this->data['interfaceid']);
+if ($data['displayInterfaces']) {
+	$interfacesComboBox = new CComboBox('interfaceid', $data['interfaceid']);
 	$interfacesComboBox->addItem(new CComboItem(0, '', false, false));
 
 	// Set up interface groups sorted by priority.
-	$interface_types = zbx_objectValues($this->data['hosts']['interfaces'], 'type');
+	$interface_types = zbx_objectValues($data['hosts']['interfaces'], 'type');
 	$interface_groups = [];
 	foreach ([INTERFACE_TYPE_AGENT, INTERFACE_TYPE_SNMP, INTERFACE_TYPE_JMX, INTERFACE_TYPE_IPMI] as $interface_type) {
 		if (in_array($interface_type, $interface_types)) {
@@ -58,13 +60,13 @@ if ($this->data['displayInterfaces']) {
 	}
 
 	// add interfaces to groups
-	foreach ($this->data['hosts']['interfaces'] as $interface) {
+	foreach ($data['hosts']['interfaces'] as $interface) {
 		$option = new CComboItem(
 			$interface['interfaceid'],
 			$interface['useip']
 				? $interface['ip'].' : '.$interface['port']
 				: $interface['dns'].' : '.$interface['port'],
-			($interface['interfaceid'] == $this->data['interfaceid'])
+			($interface['interfaceid'] == $data['interfaceid'])
 		);
 		$option->setAttribute('data-interfacetype', $interface['type']);
 		$interface_groups[$interface['type']]->addItem($option);
@@ -79,14 +81,14 @@ if ($this->data['displayInterfaces']) {
 		->setAttribute('style', 'display: none;');
 
 	$itemFormList->addRow(
-		(new CVisibilityBox('visible[interface]', 'interfaceDiv', _('Original')))
+		(new CVisibilityBox('visible[interfaceid]', 'interfaceDiv', _('Original')))
 			->setLabel(_('Host interface'))
-			->setChecked(isset($this->data['visible']['interface']))
-			->setAttribute('data-multiple-interface-types', $this->data['multiple_interface_types']),
+			->setChecked(isset($data['visible']['interfaceid']))
+			->setAttribute('data-multiple-interface-types', $data['multiple_interface_types']),
 		(new CDiv([$interfacesComboBox, $span]))->setId('interfaceDiv'),
 		'interface_row'
 	);
-	$itemForm->addVar('selectedInterfaceId', $this->data['interfaceid']);
+	$itemForm->addVar('selectedInterfaceId', $data['interfaceid']);
 }
 
 // append jmx endpoint to form list
@@ -97,36 +99,116 @@ $itemFormList->addRow(
 	(new CTextBox('jmx_endpoint', $data['jmx_endpoint']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 );
 
+// ITEM_TYPE_HTTPAGENT URL field.
+$itemFormList->addRow(
+	(new CVisibilityBox('visible[url]', 'url', _('Original')))
+		->setLabel(_('URL'))
+		->setChecked(array_key_exists('url', $data['visible'])),
+	(new CTextBox('url', $data['url'], false, DB::getFieldLength('items', 'url')))
+		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+);
+
+// ITEM_TYPE_HTTPAGENT Request body type.
+$itemFormList->addRow(
+	(new CVisibilityBox('visible[post_type]', 'post_type_container', _('Original')))
+		->setLabel(_('Request body type'))
+		->setChecked(array_key_exists('post_type', $data['visible'])),
+	(new CDiv(
+		(new CRadioButtonList('post_type', (int) $data['post_type']))
+			->addValue(_('Raw data'), ZBX_POSTTYPE_RAW)
+			->addValue(_('JSON data'), ZBX_POSTTYPE_JSON)
+			->addValue(_('XML data'), ZBX_POSTTYPE_XML)
+			->setModern(true)
+	))->setId('post_type_container')
+);
+
+// ITEM_TYPE_HTTPAGENT Request body.
+$itemFormList->addRow(
+	(new CVisibilityBox('visible[posts]', 'posts', _('Original')))
+		->setLabel(_('Request body'))
+		->setChecked(array_key_exists('posts', $data['visible'])),
+	(new CTextArea('posts', $data['posts']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+);
+
+// ITEM_TYPE_HTTPAGENT Headers fields.
+$headers_data = [];
+
+if (is_array($data['headers']) && $data['headers']) {
+	foreach ($data['headers'] as $pair) {
+		$headers_data[] = ['name' => key($pair), 'value' => reset($pair)];
+	}
+}
+else {
+	$headers_data[] = ['name' => '', 'value' => ''];
+}
+$headers = (new CTag('script', true))->setAttribute('type', 'text/json');
+$headers->items = [CJs::encodeJson($headers_data)];
+
+$itemFormList->addRow(
+	(new CVisibilityBox('visible[headers]', 'headers_pairs', _('Original')))
+		->setLabel(_('Headers'))
+		->setChecked(array_key_exists('headers', $data['visible'])),
+	(new CDiv([
+		(new CTable())
+			->setAttribute('style', 'width: 100%;')
+			->setHeader(['', _('Name'), '', _('Value'), ''])
+			->addRow((new CRow)->setAttribute('data-insert-point', 'append'))
+			->setFooter(new CRow(
+				(new CCol(
+					(new CButton(null, _('Add')))
+						->addClass(ZBX_STYLE_BTN_LINK)
+						->setAttribute('data-row-action', 'add_row')
+				))->setColSpan(5)
+			)),
+		(new CTag('script', true))
+			->setAttribute('type', 'text/x-jquery-tmpl')
+			->addItem(new CRow([
+				(new CCol((new CDiv)->addClass(ZBX_STYLE_DRAG_ICON)))->addClass(ZBX_STYLE_TD_DRAG_ICON),
+				(new CTextBox('headers[name][#{index}]', '#{name}'))->setWidth(ZBX_TEXTAREA_TAG_WIDTH),
+				'&rArr;',
+				(new CTextBox('headers[value][#{index}]', '#{value}'))->setWidth(ZBX_TEXTAREA_TAG_WIDTH),
+				(new CButton(null, _('Remove')))
+					->addClass(ZBX_STYLE_BTN_LINK)
+					->setAttribute('data-row-action', 'remove_row')
+			])),
+		$headers
+	]))
+		->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+		->setId('headers_pairs')
+		->setAttribute('data-sortable-pairs-table', '1')
+		->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
+);
+
 // append snmp community to form list
 $itemFormList->addRow(
-	(new CVisibilityBox('visible[community]', 'snmp_community', _('Original')))
+	(new CVisibilityBox('visible[snmp_community]', 'snmp_community', _('Original')))
 		->setLabel(_('SNMP community'))
-		->setChecked(isset($this->data['visible']['community'])),
-	(new CTextBox('snmp_community', $this->data['snmp_community']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setChecked(isset($data['visible']['snmp_community'])),
+	(new CTextBox('snmp_community', $data['snmp_community']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 );
 
 // append snmpv3 contextname to form list
 $itemFormList->addRow(
-	(new CVisibilityBox('visible[contextname]', 'snmpv3_contextname', _('Original')))
+	(new CVisibilityBox('visible[snmpv3_contextname]', 'snmpv3_contextname', _('Original')))
 		->setLabel(_('Context name'))
-		->setChecked(isset($this->data['visible']['contextname'])),
-	(new CTextBox('snmpv3_contextname', $this->data['snmpv3_contextname']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setChecked(isset($data['visible']['snmpv3_contextname'])),
+	(new CTextBox('snmpv3_contextname', $data['snmpv3_contextname']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 );
 
 // append snmpv3 securityname to form list
 $itemFormList->addRow(
-	(new CVisibilityBox('visible[securityname]', 'snmpv3_securityname', _('Original')))
+	(new CVisibilityBox('visible[snmpv3_securityname]', 'snmpv3_securityname', _('Original')))
 		->setLabel(_('Security name'))
-		->setChecked(isset($this->data['visible']['securityname'])),
-	(new CTextBox('snmpv3_securityname', $this->data['snmpv3_securityname']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setChecked(isset($data['visible']['snmpv3_securityname'])),
+	(new CTextBox('snmpv3_securityname', $data['snmpv3_securityname']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 );
 
 // append snmpv3 securitylevel to form list
 $itemFormList->addRow(
-	(new CVisibilityBox('visible[securitylevel]', 'snmpv3_securitylevel', _('Original')))
+	(new CVisibilityBox('visible[snmpv3_securitylevel]', 'snmpv3_securitylevel', _('Original')))
 		->setLabel(_('Security level'))
-		->setChecked(isset($this->data['visible']['securitylevel'])),
-	new CComboBox('snmpv3_securitylevel', $this->data['snmpv3_securitylevel'], null, [
+		->setChecked(isset($data['visible']['snmpv3_securitylevel'])),
+	new CComboBox('snmpv3_securitylevel', $data['snmpv3_securitylevel'], null, [
 		ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV => 'noAuthNoPriv',
 		ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV => 'authNoPriv',
 		ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV => 'authPriv'
@@ -135,11 +217,11 @@ $itemFormList->addRow(
 
 // append snmpv3 authprotocol to form list
 $itemFormList->addRow(
-	(new CVisibilityBox('visible[authprotocol]', 'authprotocol_div', _('Original')))
+	(new CVisibilityBox('visible[snmpv3_authprotocol]', 'authprotocol_div', _('Original')))
 		->setLabel(_('Authentication protocol'))
-		->setChecked(isset($this->data['visible']['authprotocol'])),
+		->setChecked(isset($data['visible']['snmpv3_authprotocol'])),
 	(new CDiv(
-		(new CRadioButtonList('snmpv3_authprotocol', (int) $this->data['snmpv3_authprotocol']))
+		(new CRadioButtonList('snmpv3_authprotocol', (int) $data['snmpv3_authprotocol']))
 			->addValue(_('MD5'), ITEM_AUTHPROTOCOL_MD5)
 			->addValue(_('SHA'), ITEM_AUTHPROTOCOL_SHA)
 			->setModern(true)
@@ -148,20 +230,20 @@ $itemFormList->addRow(
 
 // append snmpv3 authpassphrase to form list
 $itemFormList->addRow(
-	(new CVisibilityBox('visible[authpassphrase]', 'snmpv3_authpassphrase', _('Original')))
+	(new CVisibilityBox('visible[snmpv3_authpassphrase]', 'snmpv3_authpassphrase', _('Original')))
 		->setLabel(_('Authentication passphrase'))
-		->setChecked(isset($this->data['visible']['authpassphrase'])),
-	(new CTextBox('snmpv3_authpassphrase', $this->data['snmpv3_authpassphrase']))
+		->setChecked(isset($data['visible']['snmpv3_authpassphrase'])),
+	(new CTextBox('snmpv3_authpassphrase', $data['snmpv3_authpassphrase']))
 		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 );
 
 // append snmpv3 privprotocol to form list
 $itemFormList->addRow(
-	(new CVisibilityBox('visible[privprotocol]', 'privprotocol_div', _('Original')))
+	(new CVisibilityBox('visible[snmpv3_privprotocol]', 'privprotocol_div', _('Original')))
 		->setLabel(_('Privacy protocol'))
-		->setChecked(isset($this->data['visible']['privprotocol'])),
+		->setChecked(isset($data['visible']['snmpv3_privprotocol'])),
 	(new CDiv(
-		(new CRadioButtonList('snmpv3_privprotocol', (int) $this->data['snmpv3_privprotocol']))
+		(new CRadioButtonList('snmpv3_privprotocol', (int) $data['snmpv3_privprotocol']))
 			->addValue(_('DES'), ITEM_PRIVPROTOCOL_DES)
 			->addValue(_('AES'), ITEM_PRIVPROTOCOL_AES)
 			->setModern(true)
@@ -171,10 +253,10 @@ $itemFormList->addRow(
 
 // append snmpv3 privpassphrase to form list
 $itemFormList->addRow(
-	(new CVisibilityBox('visible[privpassphrase]', 'snmpv3_privpassphrase', _('Original')))
+	(new CVisibilityBox('visible[snmpv3_privpassphrase]', 'snmpv3_privpassphrase', _('Original')))
 		->setLabel(_('Privacy passphrase'))
-		->setChecked(isset($this->data['visible']['privpassphrase'])),
-	(new CTextBox('snmpv3_privpassphrase', $this->data['snmpv3_privpassphrase']))
+		->setChecked(isset($data['visible']['snmpv3_privpassphrase'])),
+	(new CTextBox('snmpv3_privpassphrase', $data['snmpv3_privpassphrase']))
 		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 );
 
@@ -182,16 +264,16 @@ $itemFormList->addRow(
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[port]', 'port', _('Original')))
 		->setLabel(_('Port'))
-		->setChecked(isset($this->data['visible']['port'])),
-	(new CTextBox('port', $this->data['port']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+		->setChecked(isset($data['visible']['port'])),
+	(new CTextBox('port', $data['port']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 );
 
 // append value type to form list
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[value_type]', 'value_type', _('Original')))
 		->setLabel(_('Type of information'))
-		->setChecked(isset($this->data['visible']['value_type'])),
-	new CComboBox('value_type', $this->data['value_type'], null, [
+		->setChecked(isset($data['visible']['value_type'])),
+	new CComboBox('value_type', $data['value_type'], null, [
 		ITEM_VALUE_TYPE_UINT64 => _('Numeric (unsigned)'),
 		ITEM_VALUE_TYPE_FLOAT => _('Numeric (float)'),
 		ITEM_VALUE_TYPE_STR => _('Character'),
@@ -204,16 +286,16 @@ $itemFormList->addRow(
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[units]', 'units', _('Original')))
 		->setLabel(_('Units'))
-		->setChecked(isset($this->data['visible']['units'])),
-	(new CTextBox('units', $this->data['units']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setChecked(isset($data['visible']['units'])),
+	(new CTextBox('units', $data['units']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 );
 
 // append authtype to form list
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[authtype]', 'authtype', _('Original')))
 		->setLabel(_('Authentication method'))
-		->setChecked(isset($this->data['visible']['authtype'])),
-	new CComboBox('authtype', $this->data['authtype'], null, [
+		->setChecked(isset($data['visible']['authtype'])),
+	new CComboBox('authtype', $data['authtype'], null, [
 		ITEM_AUTHTYPE_PASSWORD => _('Password'),
 		ITEM_AUTHTYPE_PUBLICKEY => _('Public key')
 	])
@@ -223,32 +305,32 @@ $itemFormList->addRow(
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[username]', 'username', _('Original')))
 		->setLabel(_('User name'))
-		->setChecked(isset($this->data['visible']['username'])),
-	(new CTextBox('username', $this->data['username']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+		->setChecked(isset($data['visible']['username'])),
+	(new CTextBox('username', $data['username']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 );
 
 // append publickey to form list
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[publickey]', 'publickey', _('Original')))
 		->setLabel(_('Public key file'))
-		->setChecked(isset($this->data['visible']['publickey'])),
-	(new CTextBox('publickey', $this->data['publickey']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+		->setChecked(isset($data['visible']['publickey'])),
+	(new CTextBox('publickey', $data['publickey']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 );
 
 // append privatekey to form list
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[privatekey]', 'privatekey', _('Original')))
 		->setLabel(_('Private key file'))
-		->setChecked(isset($this->data['visible']['privatekey'])),
-	(new CTextBox('privatekey', $this->data['privatekey']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+		->setChecked(isset($data['visible']['privatekey'])),
+	(new CTextBox('privatekey', $data['privatekey']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 );
 
 // append password
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[password]', 'password', _('Original')))
 		->setLabel(_('Password'))
-		->setChecked(isset($this->data['visible']['password'])),
-	(new CTextBox('password', $this->data['password']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+		->setChecked(isset($data['visible']['password'])),
+	(new CTextBox('password', $data['password']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 );
 
 // append item pre-processing
@@ -354,7 +436,7 @@ $preprocessing->addRow(
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[preprocessing]', 'preprocessing_div', _('Original')))
 		->setLabel(_('Preprocessing steps'))
-		->setChecked(isset($this->data['visible']['preprocessing'])),
+		->setChecked(isset($data['visible']['preprocessing'])),
 	(new CDiv($preprocessing))
 		->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 		->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;')
@@ -424,9 +506,9 @@ $update_interval->addRow(
 
 // append delay to form list
 $itemFormList->addRow(
-	(new CVisibilityBox('visible[update_interval]', 'update_interval_div', _('Original')))
+	(new CVisibilityBox('visible[delay]', 'update_interval_div', _('Original')))
 		->setLabel(_('Update interval'))
-		->setChecked(isset($data['visible']['update_interval'])),
+		->setChecked(isset($data['visible']['delay'])),
 	(new CDiv($update_interval))->setId('update_interval_div')
 );
 
@@ -445,14 +527,14 @@ $itemFormList
 	);
 
 // append status to form list
-$statusComboBox = new CComboBox('status', $this->data['status']);
+$statusComboBox = new CComboBox('status', $data['status']);
 foreach ([ITEM_STATUS_ACTIVE, ITEM_STATUS_DISABLED] as $status) {
 	$statusComboBox->addItem($status, item_status2str($status));
 }
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[status]', 'status', _('Original')))
 		->setLabel(_('Status'))
-		->setChecked(isset($this->data['visible']['status'])),
+		->setChecked(isset($data['visible']['status'])),
 	$statusComboBox
 );
 
@@ -460,61 +542,65 @@ $itemFormList->addRow(
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[logtimefmt]', 'logtimefmt', _('Original')))
 		->setLabel(_('Log time format'))
-		->setChecked(isset($this->data['visible']['logtimefmt'])),
-	(new CTextBox('logtimefmt', $this->data['logtimefmt']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setChecked(isset($data['visible']['logtimefmt'])),
+	(new CTextBox('logtimefmt', $data['logtimefmt']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 );
 
 // append valuemap to form list
-$valueMapsComboBox = new CComboBox('valuemapid', $this->data['valuemapid']);
+$valueMapsComboBox = new CComboBox('valuemapid', $data['valuemapid']);
 $valueMapsComboBox->addItem(0, _('As is'));
-foreach ($this->data['valuemaps'] as $valuemap) {
+foreach ($data['valuemaps'] as $valuemap) {
 	$valueMapsComboBox->addItem($valuemap['valuemapid'], $valuemap['name']);
 }
 $valueMapLink = (new CLink(_('show value mappings'), 'adm.valuemapping.php'))
 	->setAttribute('target', '_blank');
 
-$itemFormList->addRow(
-	(new CVisibilityBox('visible[valuemapid]', 'valuemap', _('Original')))
-		->setLabel(_('Show value'))
-		->setChecked(isset($this->data['visible']['valuemapid'])),
-	(new CDiv([$valueMapsComboBox, SPACE, $valueMapLink]))
-		->setId('valuemap')
-);
-
-// append trapper hosts to form list
-$itemFormList->addRow(
-	(new CVisibilityBox('visible[trapper_hosts]', 'trapper_hosts', _('Original')))
-		->setLabel(_('Allowed hosts'))
-		->setChecked(isset($this->data['visible']['trapper_hosts'])),
-	(new CTextBox('trapper_hosts', $this->data['trapper_hosts']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-);
+$itemFormList
+	->addRow(
+		(new CVisibilityBox('visible[valuemapid]', 'valuemap', _('Original')))
+			->setLabel(_('Show value'))
+			->setChecked(isset($data['visible']['valuemapid'])),
+		(new CDiv([$valueMapsComboBox, SPACE, $valueMapLink]))
+			->setId('valuemap')
+	)
+	->addRow(
+		(new CVisibilityBox('visible[allow_traps]', 'allow_traps', _('Original')))
+			->setLabel(_('Enable trapping'))
+			->setChecked(array_key_exists('allow_traps', $data['visible'])),
+		(new CCheckBox('allow_traps', HTTPCHECK_ALLOW_TRAPS_ON))
+			->setChecked($data['allow_traps'] == HTTPCHECK_ALLOW_TRAPS_ON)
+	)
+	->addRow(
+		(new CVisibilityBox('visible[trapper_hosts]', 'trapper_hosts', _('Original')))
+			->setLabel(_('Allowed hosts'))
+			->setChecked(array_key_exists('trapper_hosts', $data['visible'])),
+		(new CTextBox('trapper_hosts', $data['trapper_hosts']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+	);
 
 // append applications to form list
-if ($this->data['displayApplications']) {
+if ($data['displayApplications']) {
 	// replace applications
-	$appToReplace = null;
-	if (hasRequest('applications')) {
-		$getApps = API::Application()->get([
-			'applicationids' => getRequest('applications'),
-			'output' => ['applicationid', 'name']
-		]);
-		foreach ($getApps as $getApp) {
-			$appToReplace[] = [
-				'id' => $getApp['applicationid'],
-				'name' => $getApp['name']
-			];
-		}
-	}
+	$app_to_replace = hasRequest('applications')
+		? CArrayHelper::renameObjectsKeys(API::Application()->get([
+			'output' => ['applicationid', 'name'],
+			'applicationids' => getRequest('applications')
+		]), ['applicationid' => 'id'])
+		: [];
 
-	$replaceApp = (new CDiv(
+	$replace_app = (new CDiv(
 		(new CMultiSelect([
 			'name' => 'applications[]',
-			'objectName' => 'applications',
-			'objectOptions' => ['hostid' => $this->data['hostid']],
-			'data' => $appToReplace,
+			'object_name' => 'applications',
+			'data' => $app_to_replace,
 			'popup' => [
-				'parameters' => 'srctbl=applications&dstfrm='.$itemForm->getName().'&dstfld1=applications_'.
-					'&srcfld1=applicationid&multiselect=1&noempty=1&hostid='.$this->data['hostid']
+				'parameters' => [
+					'srctbl' => 'applications',
+					'srcfld1' => 'applicationid',
+					'dstfrm' => $itemForm->getName(),
+					'dstfld1' => 'applications_',
+					'hostid' => $data['hostid'],
+					'noempty' => true
+				]
 			]
 		]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 	))->setId('replaceApp');
@@ -522,50 +608,51 @@ if ($this->data['displayApplications']) {
 	$itemFormList->addRow(
 		(new CVisibilityBox('visible[applications]', 'replaceApp', _('Original')))
 			->setLabel(_('Replace applications'))
-			->setChecked(isset($this->data['visible']['applications'])),
-		$replaceApp
+			->setChecked(isset($data['visible']['applications'])),
+		$replace_app
 	);
 
 	// add new or existing applications
-	$appToAdd = null;
+	$applications_to_add = [];
 	if (hasRequest('new_applications')) {
+		$applicationids = [];
+
 		foreach (getRequest('new_applications') as $newApplication) {
 			if (is_array($newApplication) && isset($newApplication['new'])) {
-				$appToAdd[] = [
+				$applications_to_add[] = [
 					'id' => $newApplication['new'],
 					'name' => $newApplication['new'].' ('._x('new', 'new element in multiselect').')',
 					'isNew' => true
 				];
 			}
 			else {
-				$appToAddId[] = $newApplication;
+				$applicationids[] = $newApplication;
 			}
 		}
 
-		if (isset($appToAddId)) {
-			$getApps = API::Application()->get([
-				'applicationids' => $appToAddId,
-				'output' => ['applicationid', 'name']
-			]);
-			foreach ($getApps as $getApp) {
-				$appToAdd[] = [
-					'id' => $getApp['applicationid'],
-					'name' => $getApp['name']
-				];
-			}
-		}
+		$applications_to_add = array_merge($applications_to_add, $applicationids
+			? CArrayHelper::renameObjectsKeys(API::Application()->get([
+				'output' => ['applicationid', 'name'],
+				'applicationids' => $applicationids
+			]), ['applicationid' => 'id'])
+			: []);
 	}
 
 	$newApp = (new CDiv(
 		(new CMultiSelect([
 			'name' => 'new_applications[]',
-			'objectName' => 'applications',
-			'objectOptions' => ['hostid' => $this->data['hostid']],
-			'data' => $appToAdd,
-			'addNew' => true,
+			'object_name' => 'applications',
+			'add_new' => true,
+			'data' => $applications_to_add,
 			'popup' => [
-				'parameters' => 'srctbl=applications&dstfrm='.$itemForm->getName().'&dstfld1=new_applications_'.
-					'&srcfld1=applicationid&multiselect=1&noempty=1&hostid='.$this->data['hostid']
+				'parameters' => [
+					'srctbl' => 'applications',
+					'srcfld1' => 'applicationid',
+					'dstfrm' => $itemForm->getName(),
+					'dstfld1' => 'new_applications_',
+					'hostid' => $data['hostid'],
+					'noempty' => true
+				]
 			]
 		]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 	))->setId('newApp');
@@ -573,37 +660,57 @@ if ($this->data['displayApplications']) {
 	$itemFormList->addRow(
 		(new CVisibilityBox('visible[new_applications]', 'newApp', _('Original')))
 			->setLabel(_('Add new or existing applications'))
-			->setChecked(isset($this->data['visible']['new_applications'])),
+			->setChecked(isset($data['visible']['new_applications'])),
 		$newApp
 	);
 }
 
 // Append master item select.
-$master_item = (new CDiv([
-	(new CTextBox('master_itemname', $data['master_itemname'], true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-	(new CVar('master_itemid', $data['master_itemid'], 'master_itemid')),
-	(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-	(new CButton('button', _('Select')))
-		->addClass(ZBX_STYLE_BTN_GREY)
-		->onClick('return PopUp("popup.php?srctbl=items&srcfld1=itemid&dstfld1=master_itemid&dstfrm='.
-			$itemForm->getName().'&srcfld2=master_itemname&dstfld2=master_itemname&only_hostid='.$data['hostid'].
-			'&with_webitems=0&'.http_build_query(['excludeids' => $data['itemids']]).'");'
-		)
-]))->setId('master_item');
+if ($data['displayMasteritems']) {
+	$master_item = (new CDiv([
+		(new CMultiSelect([
+			'name' => 'master_itemid',
+			'object_name' => 'items',
+			'multiple' => false,
+			'data' => ($data['master_itemid'] != 0)
+				? [
+					[
+						'id' => $data['master_itemid'],
+						'prefix' => $data['master_hostname'].NAME_DELIMITER,
+						'name' => $data['master_itemname']
+					]
+				]
+				: [],
+			'popup' => [
+				'parameters' => [
+					'srctbl' => 'items',
+					'srcfld1' => 'itemid',
+					'dstfrm' => $itemForm->getName(),
+					'dstfld1' => 'master_itemid',
+					'hostid' => $data['hostid'],
+					'excludeids' => $data['itemids'],
+					'webitems' => true
+				]
+			]
+		]))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired(true)
+	]))->setId('master_item');
 
-$itemFormList->addRow(
-	(new CVisibilityBox('visible[master_itemid]', 'master_item', _('Original')))
-		->setLabel(_('Master item'))
-		->setChecked(isset($data['visible']['master_itemid'])),
-	$master_item
-);
+	$itemFormList->addRow(
+		(new CVisibilityBox('visible[master_itemid]', 'master_item', _('Original')))
+			->setLabel(_('Master item'))
+			->setChecked(array_key_exists('master_itemid', $data['visible'])),
+		$master_item
+	);
+}
 
 // append description to form list
 $itemFormList->addRow(
 	(new CVisibilityBox('visible[description]', 'description', _('Original')))
 		->setLabel(_('Description'))
-		->setChecked(isset($this->data['visible']['description'])),
-	(new CTextArea('description', $this->data['description']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setChecked(isset($data['visible']['description'])),
+	(new CTextArea('description', $data['description']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 );
 // append tabs to form
 $itemTab = new CTabView();
