@@ -21,28 +21,40 @@
 
 $widget = (new CWidget())
 	->setTitle(_('Templates'))
-	->setControls((new CForm('get'))
-		->cleanItems()
-		->addItem(
-			(new CList())
+	->setControls(new CList([
+		(new CForm('get'))
+			->cleanItems()
+			->setAttribute('aria-label', _('Main filter'))
+			->addItem((new CList())
 				->addItem([
 					new CLabel(_('Group'), 'groupid'),
 					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 					$data['pageFilter']->getGroupsCB()
 				])
-				->addItem(new CSubmit('form', _('Create template')))
+			),
+		(new CTag('nav', true,
+			(new CList())
+				->addItem(new CRedirectButton(_('Create template'),
+				(new CUrl('templates.php'))
+					->setArgument('groupid', $data['pageFilter']->groupid)
+					->setArgument('form', 'create')
+					->getUrl()
+				))
 				->addItem(
-					(new CButton('form', _('Import')))
-						->onClick('redirect("conf.import.php?rules_preset=template")')
+					(new CButton('form', _('Import')))->onClick('redirect("conf.import.php?rules_preset=template")')
 				)
-		)
-	)
-	->addItem((new CFilter('web.templates.filter.state'))
-		->addColumn((new CFormList())->addRow(_('Name'),
-			(new CTextBox('filter_name', $data['filter']['name']))
-				->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH)
-				->setAttribute('autofocus', 'autofocus')
-		))
+		))->setAttribute('aria-label', _('Content controls'))
+	]))
+	->addItem((new CFilter())
+		->setProfile($data['profileIdx'])
+		->setActiveTab($data['active_tab'])
+		->addFilterTab(_('Filter'), [
+			(new CFormList())->addRow(_('Name'),
+				(new CTextBox('filter_name', $data['filter']['name']))
+					->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH)
+					->setAttribute('autofocus', 'autofocus')
+			)
+		])
 	);
 
 $form = (new CForm())->setName('templates');
