@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2013 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,26 +18,18 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
+$pageHeader = (new CPageHeader(_('Warning').' ['._s('refreshed every %1$s sec.', 30).']'))
+	->addCssFile('styles/'.CHtml::encode($data['theme']).'.css')
+	->display();
 
-$pageHeader = new CPageHeader('Warning [refreshed every 30 sec]');
-$pageHeader->addCssInit();
-$pageHeader->display();
-?>
-<body>
-<?php
-// check if a CWarning object is passed
-if(!$warning = $this->get('warning')) {
-	// if not - render a standard warning with a message
-	$warning = new CWarning('Zabbix '.ZABBIX_VERSION, $this->get('message'));
-	$warning->setButtons(array(
-		new CButton('login', _('Retry'), 'document.location.reload();', 'formlist'),
-	));
-}
-$warning->show();
-?>
-<script>
-	setTimeout("document.location.reload();", 30000);
-</script>
+$buttons = array_key_exists('buttons', $data)
+	? $data['buttons']
+	: [(new CButton(null, _('Retry')))->onClick('document.location.reload();')];
 
-</body>
-</html>
+echo '<body lang="'.CWebUser::getLang().'">';
+
+(new CTag('main', true, new CWarning($data['header'], $data['messages'], $buttons)))->show();
+
+echo get_js("setTimeout('document.location.reload();', 30000);");
+echo '</body>';
+echo '</html>';
