@@ -260,7 +260,7 @@ if (typeof (zbx_widget_navtree_trigger) !== typeof (Function)) {
 				levels_moved = Math.floor(Math.abs(parent_item.offset().left - this.positionAbs.left) / o.indent_size);
 			}
 
-			$('.highliglted-parent').removeClass('highliglted-parent');
+			$('.highlighted-parent').removeClass('highlighted-parent');
 
 			if (direction_moved === 'right' && levels_moved) {
 				var drop_to = prev_item,
@@ -270,7 +270,7 @@ if (typeof (zbx_widget_navtree_trigger) !== typeof (Function)) {
 
 				this.changing_parent = setTimeout(function() {
 					$(drop_to)
-						.addClass('highliglted-parent opened')
+						.addClass('highlighted-parent opened')
 						.removeClass('closed');
 
 					if (prev_offset_top && (prev_offset_top <= prev_item.offset().top)) {
@@ -297,7 +297,7 @@ if (typeof (zbx_widget_navtree_trigger) !== typeof (Function)) {
 					levels_moved--;
 				}
 
-				$(drop_to).addClass('highliglted-parent');
+				$(drop_to).addClass('highlighted-parent');
 
 				this.changing_parent = setTimeout(function() {
 					if (one_before && one_before.length) {
@@ -316,7 +316,7 @@ if (typeof (zbx_widget_navtree_trigger) !== typeof (Function)) {
 				this._isAllowed(prev_item, level, level + child_levels);
 			}
 			else {
-				$(this.placeholder.parent().closest('.tree-item')).addClass('highliglted-parent');
+				$(this.placeholder.parent().closest('.tree-item')).addClass('highlighted-parent');
 				this._isAllowed(prev_item, level, level + child_levels);
 			}
 
@@ -341,7 +341,7 @@ if (typeof (zbx_widget_navtree_trigger) !== typeof (Function)) {
 				return;
 			}
 
-			$('.highliglted-parent').removeClass('highliglted-parent');
+			$('.highlighted-parent').removeClass('highlighted-parent');
 			this.placeholder.removeClass('sortable-error');
 
 			if (this.changing_parent) {
@@ -595,8 +595,9 @@ jQuery(function($) {
 			 * @param {numeric} id - widget field ID or 0 when creating new item.
 			 * @param {numeric} parent - ID of parent item under which new item is created.
 			 * @param {numeric} depth - a depth of parent item under which new item is created.
+			 * @param {object}  trigger_elmnt - UI element clicked to open dialog.
 			 */
-			var itemEditDialog = function($obj, id, parent, depth) {
+			var itemEditDialog = function($obj, id, parent, depth, trigger_elmnt) {
 				var url = new Curl('zabbix.php'),
 					item_edit = !!id,
 					ajax_data = {
@@ -614,7 +615,7 @@ jQuery(function($) {
 					ajax_data['map_id'] = getNextId($obj);
 				}
 
-				url.setArgument('action', 'widget.navigationtree.edititemdialog');
+				url.setArgument('action', 'widget.navtree.item.edit');
 
 				jQuery.ajax({
 					url: url.getUrl(),
@@ -642,7 +643,7 @@ jQuery(function($) {
 												mapid: id
 											};
 
-										url.setArgument('action', 'widget.navigationtree.edititem');
+										url.setArgument('action', 'widget.navtree.item.update');
 
 										jQuery.ajax({
 											url: url.getUrl(),
@@ -725,7 +726,7 @@ jQuery(function($) {
 															.removeClass('closed');
 													}
 
-													overlayDialogueDestroy();
+													overlayDialogueDestroy('navtreeitem');
 													setTreeHandlers($obj);
 												}
 											}
@@ -739,8 +740,9 @@ jQuery(function($) {
 									'class': 'btn-alt',
 									'action': function() {}
 								}
-							]
-						});
+							],
+							'dialogueid': 'navtreeitem'
+						}, trigger_elmnt);
 					}
 				});
 			};
@@ -895,7 +897,7 @@ jQuery(function($) {
 						}
 
 						if (widget_data.max_depth > +depth) {
-							itemEditDialog($obj, 0, parentId, +depth + 1);
+							itemEditDialog($obj, 0, parentId, +depth + 1, event.target);
 						}
 					});
 					tools.appendChild(btn1);
@@ -906,14 +908,8 @@ jQuery(function($) {
 					btn2.setAttribute('data-id', item.id);
 					btn2.setAttribute('class', 'import-items-btn');
 					btn2.setAttribute('title', t('Add multiple maps'));
-					btn2.addEventListener('click', function() {
-						var url = new Curl('popup.php'),
-							id = $(this).data('id');
-
-						url.setArgument('srctbl', 'sysmaps');
-						url.setArgument('srcfld1', 'sysmapid');
-						url.setArgument('srcfld2', 'name');
-						url.setArgument('multiselect', '1');
+					btn2.addEventListener('click', function(event) {
+						var id = $(this).data('id');
 
 						if (typeof addPopupValues === 'function') {
 							old_addPopupValues = addPopupValues;
@@ -947,8 +943,13 @@ jQuery(function($) {
 							}
 						};
 
-						return PopUp(url.getUrl());
-					});
+						return PopUp('popup.generic', {
+							srctbl: 'sysmaps',
+							srcfld1: 'sysmapid',
+							srcfld2: 'name',
+							multiselect: '1'
+						}, null, event.target);
+						});
 					tools.appendChild(btn2);
 
 					if (editable) {
@@ -958,12 +959,12 @@ jQuery(function($) {
 						btn3.setAttribute('data-id', item.id);
 						btn3.setAttribute('class', 'edit-item-btn');
 						btn3.setAttribute('title', t('Edit'));
-						btn3.addEventListener('click', function() {
+						btn3.addEventListener('click', function(event) {
 							var id = $(this).data('id'),
 								parent = +$('input[name="map.parent.' + id + '"]', $obj).val(),
 								depth = +$(this).closest('[data-depth]').attr('data-depth');
 
-							itemEditDialog($obj, id, parent, depth);
+							itemEditDialog($obj, id, parent, depth, event.target);
 						});
 						tools.appendChild(btn3);
 
