@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
+
 // Array indexOf method for javascript<1.6 compatibility
 if (!Array.prototype.indexOf) {
 	Array.prototype.indexOf = function (searchElement) {
@@ -72,7 +73,11 @@ var PageRefresh = {
 
 		this.delayLeft -= 1000;
 		if (this.delayLeft < 0) {
-			location.replace(location.href);
+			if (IE || ED) {
+				sessionStorage.scrollTop = jQuery(window).scrollTop();
+			}
+
+			location.reload();
 		}
 		else {
 			this.timeout = setTimeout('PageRefresh.check()', 1000);
@@ -101,18 +106,92 @@ var PageRefresh = {
  * Main menu
  */
 var MMenu = {
-	menus:			{'empty': 0, 'view': 0, 'cm': 0, 'reports': 0, 'config': 0, 'admin': 0},
+	menus:			{'view': 0, 'cm': 0, 'reports': 0, 'config': 0, 'admin': 0},
 	def_label:		null,
 	sub_active: 	false,
 	timeout_reset:	null,
 	timeout_change:	null,
 
+	init: function() {
+		// Detects when none of the selected elements are focused.
+		var elems = jQuery('.top-nav a, .top-subnav a').on('keydown', function(event) {
+			clearTimeout(this.timeout_reset);
+
+			if (event.which == 9) {
+				setTimeout(function() {
+					if (elems.toArray().indexOf(document.querySelector(':focus')) == -1) {
+						clearTimeout(this.timeout_reset);
+						this.timeout_reset = setTimeout(function() {
+							if (elems.toArray().indexOf(document.querySelector(':focus')) == -1){
+								MMenu.showSubMenu(MMenu.def_label)
+							}
+						}, 2500);
+					}
+				});
+			}
+		});
+
+		if (SF) {
+			var nav_elems = jQuery('.top-subnav a, .search, .btn-search, .top-nav-zbbshare, .top-nav-help, .top-nav-profile, .top-nav-signout')
+					.on('keydown', function(event) {
+				if (event.which == 9) {
+					var visible_subnav_elems = nav_elems.filter(function() {
+						return jQuery(this).is(':visible');
+					});
+					var current = visible_subnav_elems.toArray().indexOf(this);
+
+					if (event.shiftKey && current > 0) {
+						visible_subnav_elems.get(current - 1).focus();
+					}
+					else if (!event.shiftKey && visible_subnav_elems.length > current + 1) {
+						visible_subnav_elems.get(current + 1).focus();
+					}
+					else if (event.shiftKey && current == 0) {
+						// Find the previous :focusable element to focus.
+						var active_element_index = jQuery('*', 'body').toArray().indexOf(document.activeElement),
+							prev_element = null;
+
+						jQuery('*', 'body').each(function(i) {
+							if (active_element_index > i && jQuery(this).is(':focusable')) {
+								prev_element = this;
+							};
+						});
+
+						if (prev_element) {
+							prev_element.focus();
+						}
+					}
+					else if (current + 1 == visible_subnav_elems.length) {
+						// If this is the last item in the sub-menu list, focus next :focusable element.
+						var active_element_index = jQuery('*', 'body').toArray().indexOf(document.activeElement);
+
+						jQuery('*', 'body').filter(function(i) {
+							return (i > active_element_index && jQuery(this).is(':focusable'));
+						}).get(0).focus();
+					}
+
+					event.preventDefault();
+
+					return false;
+				}
+			});
+		}
+	},
+
 	mouseOver: function(show_label) {
 		clearTimeout(this.timeout_reset);
-		this.timeout_change = setTimeout('MMenu.showSubMenu("' + show_label + '")', 200);
+		this.timeout_change = setTimeout('MMenu.showSubMenu("' + show_label + '", true)', 10);
 		PageRefresh.restart();
 	},
 
+	keyUp: function(show_label, event) {
+		if (event.which == 13) {
+			clearTimeout(this.timeout_reset);
+			this.timeout_change = setTimeout('MMenu.showSubMenu("' + show_label + '", true)', 10);
+			PageRefresh.restart();
+		}
+	},
+
 	submenu_mouseOver: function() {
 		clearTimeout(this.timeout_reset);
 		clearTimeout(this.timeout_change);
@@ -124,24 +203,31 @@ var MMenu = {
 		this.timeout_reset = setTimeout('MMenu.showSubMenu("' + this.def_label + '")', 2500);
 	},
 
-	showSubMenu: function(show_label) {
-		var menu_div = $('sub_' + show_label);
-		if (!is_null(menu_div)) {
-			$(show_label).className = 'active';
-			menu_div.show();
+	showSubMenu: function(show_label, focus_subitem) {
+		var sub_menu = $('sub_' + show_label),
+			focus_subitem = focus_subitem || false;
+
+		if (sub_menu !== null) {
+			$(show_label).className = 'selected';
+			sub_menu.show();
+
+			if (focus_subitem) {
+				jQuery('li:first > a', sub_menu).focus();
+			}
+
 			for (var key in this.menus) {
 				if (key == show_label) {
 					continue;
 				}
 
 				var menu_cell = $(key);
-				if (!is_null(menu_cell)) {
-					if (menu_cell.tagName.toLowerCase() != 'select') {
-						menu_cell.className = '';
-					}
+				if (menu_cell !== null) {
+					menu_cell.className = '';
+					jQuery('a', menu_cell).blur();
 				}
+
 				var sub_menu_cell = $('sub_' + key);
-				if (!is_null(sub_menu_cell)) {
+				if (sub_menu_cell !== null) {
 					sub_menu_cell.hide();
 				}
 			}
@@ -300,62 +386,41 @@ var AudioControl = {
  * @author Konstantin Buravcov
  */
 var jqBlink = {
-	shown: false, // are objects currently shown or hidden?
-	blinkInterval: 1000, // how fast will they blink (ms)
-	secondsSinceInit: 0,
+	shown: true, // are objects currently shown or hidden?
+	interval: 1000, // how fast will they blink (ms)
 
 	/**
 	 * Shows/hides the elements and repeats it self after 'this.blinkInterval' ms
 	 */
 	blink: function() {
-		var objects = jQuery('.blink');
-
-		// maybe some of the objects should not blink any more?
-		objects = this.filterOutNonBlinking(objects);
-
-		// changing visibility state
-		fun = this.shown ? 'removeClass' : 'addClass';
-		jQuery.each(objects, function() {
-			if (typeof jQuery(this).data('toggleClass') !== 'undefined') {
-				jQuery(this)[fun](jQuery(this).data('toggleClass'));
-			}
-			else {
-				jQuery(this).css('visibility', jqBlink.shown ? 'hidden' : 'visible');
-			}
-		})
-
-		// reversing the value of indicator attribute
-		this.shown = !this.shown;
-
-		// I close my eyes only for a moment, and a moment's gone
-		this.secondsSinceInit += this.blinkInterval / 1000;
-
-		// repeating this function with delay
-		setTimeout(jQuery.proxy(this.blink, this), this.blinkInterval);
-	},
-
-	/**
-	 * Check all currently found objects and exclude ones that should stop blinking by now
-	 */
-	filterOutNonBlinking: function(objects) {
 		var that = this;
 
-		return objects.filter(function() {
-			var obj = jQuery(this);
-			if (typeof obj.data('timeToBlink') !== 'undefined') {
-				var shouldBlink = parseInt(obj.data('timeToBlink'), 10) > that.secondsSinceInit;
+		setInterval(function() {
+			var $collection = jQuery('.blink');
 
-				// if object stops blinking, it should be left visible
-				if (!shouldBlink && !that.shown) {
-					obj.css('visibility', 'visible');
+			$collection.each(function() {
+				var $el = jQuery(this),
+					blink = true;
+
+				if (typeof $el.data('timeToBlink') !== 'undefined') {
+					blink = (($el.data()['timeToBlink']--) > 0);
 				}
-				return shouldBlink;
-			}
-			else {
-				// no time-to-blink attribute, should blink forever
-				return true;
-			}
-		});
+
+				if (blink) {
+					if (typeof $el.data('toggleClass') !== 'undefined') {
+						$el[that.shown ? 'removeClass' : 'addClass']($el.data('toggleClass'));
+					}
+					else {
+						$el.css('visibility', that.shown ? 'visible' : 'hidden');
+					}
+				}
+				else if (that.shown) {
+					$el.removeClass('blink').removeClass($el.data('toggleClass')).css('visibility', '');
+				}
+			});
+
+			that.shown = !that.shown;
+		}, this.interval);
 	}
 };
 
@@ -364,8 +429,80 @@ var jqBlink = {
  */
 var hintBox = {
 
-	createBox: function(e, target, hintText, className, isStatic) {
-		var box = jQuery('<div></div>').addClass('hintbox');
+	/**
+	 * Initialize hint box event handlers.
+	 *
+	 * Event 'remove' is triggered on:
+	 * - content update using flickerfreeScreen.refreshHtml()
+	 * - widget update by updateWidgetContent()
+	 * - widget remove by deleteWidget().
+	 *
+	 * Triggered events:
+	 * - onDeleteHint.hintBox 	- when removing a hintbox.
+	 */
+	bindEvents: function () {
+		jQuery(document).on('keydown click mouseenter mouseleave remove', '[data-hintbox=1]', function (e) {
+			var target = jQuery(this);
+
+			switch (e.type) {
+				case 'mouseenter':
+					hintBox.showHint(e, this, target.next('.hint-box').html(), target.data('hintbox-class'), false,
+						target.data('hintbox-style')
+					);
+					break;
+
+				case 'mouseleave':
+					hintBox.hideHint(e, this);
+					break;
+
+				case 'remove':
+					hintBox.deleteHint(this);
+					break;
+
+				case 'keydown':
+					if (e.which == 13 && target.data('hintbox-static') == 1) {
+						var offset = target.offset(),
+							w = jQuery(window);
+						// Emulate click on left middle point of link.
+						e.clientX = offset.left - w.scrollLeft();
+						e.clientY = offset.top - w.scrollTop() + (target.height() / 2);
+						e.preventDefault();
+
+						hintBox.showStaticHint(e, this, target.data('hintbox-class'), false,
+							target.data('hintbox-style')
+						);
+					}
+					break;
+
+				case 'click':
+					if (target.data('hintbox-static') == 1) {
+						hintBox.showStaticHint(e, this, target.data('hintbox-class'), false,
+							target.data('hintbox-style')
+						);
+					}
+					break;
+			}
+		});
+	},
+
+	createBox: function(e, target, hintText, className, isStatic, styles, appendTo) {
+		var hintboxid = hintBox.getUniqueId(),
+			box = jQuery('<div></div>', {'data-hintboxid': hintboxid}).addClass('overlay-dialogue'),
+			appendTo = appendTo || 'body';
+
+		if (styles) {
+			// property1: value1; property2: value2; property(n): value(n)
+
+			var style_list = styles.split(';');
+
+			for (var i = 0; i < style_list.length; i++) {
+				var style_props = style_list[i].split(':');
+
+				if (style_props[1]) {
+					box.css(style_props[0].trim(), style_props[1].trim());
+				}
+			}
+		}
 
 		if (typeof hintText === 'string') {
 			hintText = hintText.replace(/\n/g, '<br />');
@@ -379,66 +516,57 @@ var hintBox = {
 		}
 
 		if (isStatic) {
-			var close_link = jQuery('<div>' + locale['S_CLOSE'] + '</div>')
-				.addClass('link')
-				.css({
-					'text-align': 'right',
-					'border-bottom': '1px #333 solid'
-				}).click(function() {
+			target.hintboxid = hintboxid;
+			addToOverlaysStack(hintboxid, target, 'hintbox');
+
+			var close_link = jQuery('<button>', {
+					'class': 'overlay-close-btn',
+					'title': t('Close')
+				}
+			)
+				.click(function() {
 					hintBox.hideHint(e, target, true);
 				});
 			box.prepend(close_link);
 		}
 
-		jQuery('body').append(box);
+		jQuery(appendTo).append(box);
 
 		return box;
 	},
 
-	HintWraper: function(e, target, hintText, className) {
-		target.isStatic = false;
-
-		jQuery(target).on('mouseenter', function(e, d) {
-			if (d) {
-				e = d;
-			}
-			hintBox.showHint(e, target, hintText, className, false);
-
-		}).on('mouseleave', function(e) {
-			hintBox.hideHint(e, target);
-
-		}).on('remove', function(e) {
-			hintBox.deleteHint(target);
-		});
-
-		jQuery(target).removeAttr('onmouseover');
-		jQuery(target).trigger('mouseenter', e);
-	},
-
-	showStaticHint: function(e, target, hint, className, resizeAfterLoad) {
+	showStaticHint: function(e, target, className, resizeAfterLoad, styles, hintText) {
 		var isStatic = target.isStatic;
 		hintBox.hideHint(e, target, true);
 
 		if (!isStatic) {
+			if (typeof hintText === 'undefined') {
+				hintText = jQuery(target).next('.hint-box').html();
+			}
+
 			target.isStatic = true;
-			hintBox.showHint(e, target, hint, className, true);
+			hintBox.showHint(e, target, hintText, className, true, styles);
 
 			if (resizeAfterLoad) {
-				hint.one('load', function(e) {
+				hintText.one('load', function(e) {
 					hintBox.positionHint(e, target);
 				});
 			}
 		}
 	},
 
-	showHint: function(e, target, hintText, className, isStatic) {
+	showHint: function(e, target, hintText, className, isStatic, styles) {
 		if (target.hintBoxItem) {
 			return;
 		}
 
-		target.hintBoxItem = hintBox.createBox(e, target, hintText, className, isStatic);
+		target.hintBoxItem = hintBox.createBox(e, target, hintText, className, isStatic, styles);
 		hintBox.positionHint(e, target);
 		target.hintBoxItem.show();
+
+		if (target.isStatic) {
+			overlayDialogueOnLoad(true, target.hintBoxItem);
+		}
 	},
 
 	positionHint: function(e, target) {
@@ -446,6 +574,8 @@ var hintBox = {
 			wHeight = jQuery(window).height(),
 			scrollTop = jQuery(window).scrollTop(),
 			scrollLeft = jQuery(window).scrollLeft(),
+			hint_width = jQuery(target.hintBoxItem).outerWidth(),
+			hint_height = jQuery(target.hintBoxItem).outerHeight(),
 			top, left;
 
 		// uses stored clientX on afterload positioning when there is no event
@@ -455,25 +585,25 @@ var hintBox = {
 		}
 
 		// doesn't fit in the screen horizontally
-		if (target.hintBoxItem.width() + 10 > wWidth) {
+		if (hint_width + 10 > wWidth) {
 			left = scrollLeft + 2;
 		}
 		// 10px to right if fit
-		else if (wWidth - target.clientX - 10 > target.hintBoxItem.width()) {
+		else if (wWidth - target.clientX - 10 > hint_width) {
 			left = scrollLeft + target.clientX + 10;
 		}
 		// 10px from screen right side
 		else {
-			left = scrollLeft + wWidth - 10 - target.hintBoxItem.width();
+			left = scrollLeft + wWidth - 10 - hint_width;
 		}
 
 		// 10px below if fit
-		if (wHeight - target.clientY - target.hintBoxItem.height() - 10 > 0) {
+		if (wHeight - target.clientY - hint_height - 10 > 0) {
 			top = scrollTop + target.clientY + 10;
 		}
 		// 10px above if fit
-		else if (target.clientY - target.hintBoxItem.height() - 10 > 0) {
-			top = scrollTop + target.clientY - target.hintBoxItem.height() - 10;
+		else if (target.clientY - hint_height - 10 > 0) {
+			top = scrollTop + target.clientY - hint_height - 10;
 		}
 		// 10px below as fallback
 		else {
@@ -481,21 +611,21 @@ var hintBox = {
 		}
 
 		// fallback if doesn't fit verticaly but could fit if aligned to right or left
-		if ((top - scrollTop + target.hintBoxItem.height() > wHeight)
-				&& (target.clientX - 10 > target.hintBoxItem.width() || wWidth - target.clientX - 10 > target.hintBoxItem.width())) {
+		if ((top - scrollTop + hint_height > wHeight)
+				&& (target.clientX - 10 > hint_width || wWidth - target.clientX - 10 > hint_width)) {
 
 			// align to left if fit
-			if (wWidth - target.clientX - 10 > target.hintBoxItem.width()) {
+			if (wWidth - target.clientX - 10 > hint_width) {
 				left = scrollLeft + target.clientX + 10;
 			}
 			// align to right
 			else {
-				left = scrollLeft + target.clientX - target.hintBoxItem.width() - 10;
+				left = scrollLeft + target.clientX - hint_width - 10;
 			}
 
 			// 10px from bottom if fit
-			if (wHeight - 10 > target.hintBoxItem.height()) {
-				top = scrollTop + wHeight - target.hintBoxItem.height() - 10;
+			if (wHeight - 10 > hint_height) {
+				top = scrollTop + wHeight - hint_height - 10;
 			}
 			// 10px from top
 			else {
@@ -519,7 +649,12 @@ var hintBox = {
 	},
 
 	deleteHint: function(target) {
+		if (typeof target.hintboxid !== 'undefined') {
+			removeFromOverlaysStack(target.hintboxid);
+		}
+
 		if (target.hintBoxItem) {
+			target.hintBoxItem.trigger('onDeleteHint.hintBox');
 			target.hintBoxItem.remove();
 			delete target.hintBoxItem;
 
@@ -527,138 +662,81 @@ var hintBox = {
 				delete target.isStatic;
 			}
 		}
+	},
+
+	getUniqueId: function() {
+		var hintboxid = Math.random().toString(36).substring(7);
+		while (jQuery('[data-hintboxid="' + hintboxid + '"]').length) {
+			hintboxid = Math.random().toString(36).substring(7);
+		}
+
+		return hintboxid;
 	}
 };
 
-/*
- * Color picker
+/**
+ * Add object to the list of favourites.
  */
-function hide_color_picker() {
-	if (!color_picker) {
-		return;
-	}
-
-	color_picker.style.zIndex = 1000;
-	color_picker.style.visibility = 'hidden';
-	color_picker.style.left = '-' + ((color_picker.style.width) ? color_picker.style.width : 100) + 'px';
-	curr_lbl = null;
-	curr_txt = null;
-}
-
-function show_color_picker(id) {
-	if (!color_picker) {
-		return;
-	}
-
-	curr_lbl = document.getElementById('lbl_' + id);
-	curr_txt = document.getElementById(id);
-	var pos = getPosition(curr_lbl);
-	color_picker.x = pos.left;
-	color_picker.y = pos.top;
-	color_picker.style.left = (color_picker.x + 20) + 'px';
-	color_picker.style.top = color_picker.y + 'px';
-	color_picker.style.visibility = 'visible';
-}
-
-function create_color_picker() {
-	if (color_picker) {
-		return;
-	}
-
-	color_picker = document.createElement('div');
-	color_picker.setAttribute('id', 'color_picker');
-	color_picker.innerHTML = color_table;
-	document.body.appendChild(color_picker);
-	hide_color_picker();
-}
-
-function set_color(color) {
-	if (curr_lbl) {
-		curr_lbl.style.background = curr_lbl.style.color = '#' + color;
-		curr_lbl.title = '#' + color;
-	}
-	if (curr_txt) {
-		curr_txt.value = color.toString().toUpperCase();
-	}
-	hide_color_picker();
-}
-
-function set_color_by_name(id, color) {
-	curr_lbl = document.getElementById('lbl_' + id);
-	curr_txt = document.getElementById(id);
-	set_color(color);
-}
-
-function add2favorites(favobj, favid) {
-	sendAjaxData({
+function add2favorites(object, objectid) {
+	sendAjaxData('zabbix.php?action=favourite.create', {
 		data: {
-			favobj: favobj,
-			favid: favid,
-			favaction: 'add'
+			object: object,
+			objectid: objectid
 		}
 	});
 }
 
-function rm4favorites(favobj, favid) {
-	sendAjaxData({
+/**
+ * Remove object from the list of favourites. Remove all favourites if objectid==0.
+ */
+function rm4favorites(object, objectid) {
+	sendAjaxData('zabbix.php?action=favourite.delete', {
 		data: {
-			favobj: favobj,
-			favid: favid,
-			favaction: 'remove'
+			object: object,
+			objectid: objectid
 		}
 	});
 }
 
-
 /**
  * Toggles filter state and updates title and icons accordingly.
  *
- * @param {int} 	id					Id of filter in DOM
- * @param {string} 	titleWhenVisible	Title to set when filter is visible
- * @param {string} 	titleWhenHidden		Title to set when filter is collapsed
+ * @param {string} 	idx					User profile index
+ * @param {string} 	value_int			Integer value
+ * @param {object} 	idx2				An array of IDs
  */
-function changeFlickerState(id, titleWhenVisible, titleWhenHidden) {
-	var state = showHide(id);
-
-	switchElementClass('flicker_icon_l', 'dbl_arrow_up', 'dbl_arrow_down');
-	switchElementClass('flicker_icon_r', 'dbl_arrow_up', 'dbl_arrow_down');
-
-	var title = state ? titleWhenVisible : titleWhenHidden;
-
-	jQuery('#flicker_title').html(title);
-
-	sendAjaxData({
+function updateUserProfile(idx, value_int, idx2) {
+	return sendAjaxData('zabbix.php?action=profile.update', {
 		data: {
-			filterState: state
+			idx: idx,
+			value_int: value_int,
+			idx2: idx2
 		}
 	});
-
-	// resize multiselects in the flicker
-	if (jQuery('.multiselect').length > 0 && state == 1) {
-		jQuery('.multiselect', jQuery('#' + id)).multiSelect('resize');
-	}
 }
 
-function changeWidgetState(obj, widgetId) {
+function changeWidgetState(obj, widgetId, url) {
 	var widgetObj = jQuery('#' + widgetId + '_widget'),
-		css = switchElementClass(obj, 'arrowup', 'arrowdown'),
+		css = switchElementClass(obj, 'btn-widget-collapse', 'btn-widget-expand'),
 		state = 0;
 
-	if (css === 'arrowdown') {
+	if (css === 'btn-widget-expand') {
 		jQuery('.body', widgetObj).slideUp(50);
-		jQuery('.footer', widgetObj).slideUp(50);
+		jQuery('.dashbrd-widget-foot', widgetObj).slideUp(50);
 	}
 	else {
 		jQuery('.body', widgetObj).slideDown(50);
-		jQuery('.footer', widgetObj).slideDown(50);
+		jQuery('.dashbrd-widget-foot', widgetObj).slideDown(50);
 
 		state = 1;
 	}
 
-	sendAjaxData({
+	obj.title = (state == 1) ? locale['S_COLLAPSE'] : locale['S_EXPAND'];
+
+	sendAjaxData(url, {
 		data: {
-			widgetName: widgetId,
-			widgetState: state
+			widget: widgetId,
+			state: state
 		}
 	});
 }
@@ -666,69 +744,17 @@ function changeWidgetState(obj, widgetId) {
 /**
  * Send ajax data.
  *
+ * @param string url
  * @param object options
  */
-function sendAjaxData(options) {
-	var url = new Curl(location.href);
-	url.setQuery('?output=ajax');
+function sendAjaxData(url, options) {
+	var url = new Curl(url);
+	url.setArgument('output', 'ajax');
 
-	var defaults = {
-		type: 'post',
-		url: url.getUrl()
-	};
+	options.type = 'post';
+	options.url = url.getUrl();
 
-	jQuery.ajax(jQuery.extend({}, defaults, options));
-}
-
-/**
- * Finds all elements with a 'placeholder' attribute and emulates the placeholder in IE.
- */
-function createPlaceholders() {
-	if (IE) {
-		jQuery('[placeholder]').each(function() {
-			var placeholder = jQuery(this);
-
-			if (!placeholder.data('has-placeholder-handlers')) {
-				placeholder
-					.data('has-placeholder-handlers', true)
-					.focus(function() {
-						var obj = jQuery(this);
-
-						if (!obj.attr('placeholder')) {
-							return;
-						}
-
-						if (obj.val() == obj.attr('placeholder')) {
-							obj.val('');
-							obj.removeClass('placeholder');
-						}
-					})
-					.blur(function() {
-						var obj = jQuery(this);
-
-						if (!obj.attr('placeholder')) {
-							return;
-						}
-
-						if (obj.val() == '' ||  obj.val() == obj.attr('placeholder')) {
-							obj.val(obj.attr('placeholder'));
-							obj.addClass('placeholder');
-						}
-					})
-					.blur();
-			}
-
-			jQuery('form').submit(function() {
-				jQuery('.placeholder').each(function() {
-					var obj = jQuery(this);
-
-					if (obj.val() == obj.attr('placeholder')) {
-						obj.val('');
-					}
-				});
-			});
-		});
-	}
+	return jQuery.ajax(options);
 }
 
 /**
@@ -793,14 +819,14 @@ function getConditionFormula(conditions, evalType) {
 		case 1:
 			conditionOperator = 'and';
 			groupOperator = conditionOperator;
-
 			break;
+
 		// or
 		case 2:
 			conditionOperator = 'or';
 			groupOperator = conditionOperator;
-
 			break;
+
 		// and/or
 		default:
 			conditionOperator = 'or';
@@ -808,15 +834,17 @@ function getConditionFormula(conditions, evalType) {
 	}
 
 	var groupedFormulas = [];
+
 	for (var i = 0; i < conditions.length; i++) {
 		if (typeof conditions[i] === 'undefined') {
 			continue;
 		}
 
 		var groupedConditions = [];
+
 		groupedConditions.push(conditions[i].id);
 
-		// search for other conditions of the same type
+		// Search for other conditions of the same type.
 		for (var n = i + 1; n < conditions.length; n++) {
 			if (typeof conditions[n] !== 'undefined' && conditions[i].type == conditions[n].type) {
 				groupedConditions.push(conditions[n].id);
@@ -824,7 +852,7 @@ function getConditionFormula(conditions, evalType) {
 			}
 		}
 
-		// join conditions of the same type
+		// Join conditions of the same type.
 		if (groupedConditions.length > 1) {
 			groupedFormulas.push('(' + groupedConditions.join(' ' + conditionOperator + ' ') + ')');
 		}
@@ -835,7 +863,7 @@ function getConditionFormula(conditions, evalType) {
 
 	var formula = groupedFormulas.join(' ' + groupOperator + ' ');
 
-	// strip parentheses if there's only one condition group
+	// Strip parentheses if there's only one condition group.
 	if (groupedFormulas.length == 1) {
 		formula = formula.substr(1, formula.length - 2);
 	}
@@ -856,8 +884,10 @@ function getConditionFormula(conditions, evalType) {
 	 * - dataCallback	- function to generate the data passed to the template
 	 *
 	 * Triggered events:
-	 * - rowremove.dynamicRows 	- after removing a row (triggered before tableupdate.dynamicRows)
-	 * - tableupdate.dynamicRows 	- after adding or removing a row
+	 * - tableupdate.dynamicRows 	- after adding or removing a row.
+	 * - beforeadd.dynamicRows 	    - only before adding a new row.
+	 * - afteradd.dynamicRows 	    - only after adding a new row.
+	 * - afterremove.dynamicRows 	- only after removing a row.
 	 *
 	 * @param options
 	 */
@@ -868,6 +898,7 @@ function getConditionFormula(conditions, evalType) {
 			add: '.element-table-add',
 			remove: '.element-table-remove',
 			counter: null,
+			beforeRow: null,
 			dataCallback: function(data) {
 				return {};
 			}
@@ -882,8 +913,15 @@ function getConditionFormula(conditions, evalType) {
 
 			// add buttons
 			table.on('click', options.add, function() {
+				table.trigger('beforeadd.dynamicRows', options);
+
 				// add the new row before the row with the "Add" button
-				addRow(table, $(this).closest('tr'), options);
+				var beforeRow = (options['beforeRow'] !== null)
+					? $(options['beforeRow'], table)
+					:  $(this).closest('tr');
+				addRow(table, beforeRow, options);
+
+				table.trigger('afteradd.dynamicRows', options);
 			});
 
 			// remove buttons
@@ -911,8 +949,6 @@ function getConditionFormula(conditions, evalType) {
 		beforeRow.before(template.evaluate(data));
 		table.data('dynamicRows').counter++;
 
-		createPlaceholders();
-
 		table.trigger('tableupdate.dynamicRows', options);
 	}
 
@@ -926,47 +962,14 @@ function getConditionFormula(conditions, evalType) {
 	function removeRow(table, row, options) {
 		row.remove();
 
-		table.trigger('rowremove.dynamicRows', options);
 		table.trigger('tableupdate.dynamicRows', options);
+		table.trigger('afterremove.dynamicRows', options);
 	}
 }(jQuery));
 
 jQuery(function ($) {
 	var verticalHeaderTables = {};
 
-	var tablesWidthChangeChecker = function() {
-		for (var tableId in verticalHeaderTables) {
-			if (verticalHeaderTables.hasOwnProperty(tableId)) {
-				var table = verticalHeaderTables[tableId];
-
-				if (table && table.width() != table.data('last-width')) {
-					centerVerticalCellContents(table);
-				}
-			}
-		}
-		setTimeout(tablesWidthChangeChecker, 100);
-	};
-
-	var centerVerticalCellContents = function(table) {
-		var verticalCells = $('.vertical_rotation', table);
-
-		verticalCells.each(function() {
-			var cell = $(this),
-				cellWidth = cell.width();
-
-			if (cellWidth > 30) {
-				cell.children().css({
-					position: 'relative',
-					left: (cellWidth / 2 - 12) + 'px'
-				});
-			}
-		});
-
-		table.data('last-width', table.width());
-	};
-
-	tablesWidthChangeChecker();
-
 	$.fn.makeVerticalRotation = function() {
 		this.each(function(i) {
 			var table = $(this);
@@ -981,15 +984,10 @@ jQuery(function ($) {
 
 			// insert spans
 			cellsToRotate.each(function() {
-				var cell = $(this);
-
-				var text = $('<span>', {
-					text: cell.html()
-				});
-
-				if (IE) {
-					text.css({'font-family': 'monospace'});
-				}
+				var cell = $(this),
+					text = $('<span>', {
+						text: cell.html()
+					}).css({'white-space': 'nowrap'});
 
 				cell.text('').append(text);
 			});
@@ -1002,21 +1000,17 @@ jQuery(function ($) {
 					width = span.width(),
 					transform = (width / 2) + 'px ' + (width / 2) + 'px';
 
-				var css = {
-					"transform-origin": transform,
-					"-webkit-transform-origin": transform,
-					"-moz-transform-origin": transform,
-					"-o-transform-origin": transform
-				};
-
-				if (IE) {
-					css['font-family'] = 'monospace';
-					css['-ms-transform-origin'] = '50% 50%';
-				}
+				var css = {};
 
 				if (IE9) {
 					css['-ms-transform-origin'] = transform;
 				}
+				else {
+					css['transform-origin'] = transform;
+					css['-webkit-transform-origin'] = transform;
+					css['-moz-transform-origin'] = transform;
+					css['-o-transform-origin'] = transform;
+				}
 
 				var divInner = $('<div>', {
 					'class': 'vertical_rotation_inner'
@@ -1037,8 +1031,6 @@ jQuery(function ($) {
 				$(this).html(betterCells[i]);
 			});
 
-			centerVerticalCellContents(table);
-
 			table.on('remove', function() {
 				delete verticalHeaderTables[table.attr('id')];
 			});
@@ -1046,4 +1038,87 @@ jQuery(function ($) {
 			verticalHeaderTables[table.attr('id')] = table;
 		});
 	};
+
+	if ((IE || ED) && typeof sessionStorage.scrollTop !== 'undefined') {
+		$(window).scrollTop(sessionStorage.scrollTop);
+	}
+});
+
+
+jQuery(function ($) {
+	"use strict";
+
+	function calcRows($obj, options) {
+		var max_height = (options && 'maxHeight' in options) ? options.maxHeight : null,
+			clone = $obj
+				.clone(false)
+				.css({'position': 'absolute', 'z-index': '-1'})
+				.removeClass('patternselect')
+				.insertAfter($obj),
+			padding = parseInt(clone.css('padding-top'), 10) + parseInt(clone.css('padding-bottom'), 10),
+			line_height = parseInt(clone.css('font-size'), 10) * 1.14,
+			rows = 0;
+
+		do {
+			rows++;
+			clone.height(rows * line_height);
+		}
+		while (clone[0].scrollHeight - padding > clone.innerHeight()
+			&& (max_height === null || rows * line_height + padding <= max_height));
+		clone.remove();
+
+		return rows;
+	}
+
+	var methods = {
+		init: function(options) {
+			options = $.extend({}, options);
+
+			this.each(function() {
+				if (typeof $(this).data('autogrow') === 'undefined') {
+					$(this)
+						.css({
+							'resize': 'none',
+							'overflow-x': 'hidden',
+							'white-space': 'pre-line'
+						})
+						.on('paste change keyup', function() {
+							var rows = calcRows($(this), options);
+
+							if (options && 'pair' in options) {
+								var pair_rows = calcRows($(options.pair), options);
+								if (pair_rows > rows) {
+									rows = pair_rows;
+								}
+								$(options.pair).attr('rows', rows);
+							}
+
+							$(this).attr('rows', rows);
+						})
+						.data('autogrow', options)
+						.trigger('keyup');
+				}
+
+				if ($(this).prop('maxlength') !== 'undefined' && !CR && !GK) {
+					$(this).bind('paste contextmenu change keydown keypress keyup', function() {
+						if ($(this).val().length > $(this).attr('maxlength')) {
+							$(this).val($(this).val().substr(0, $(this).attr('maxlength')));
+						}
+					});
+				}
+
+				if (options && 'pair' in options) {
+					$(options.pair).css({'resize': 'none'});
+				}
+			});
+		}
+	};
+
+	$.fn.autoGrowTextarea = function(method, options) {
+		if (methods[method]) {
+			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
+		}
+
+		return methods.init.apply(this, arguments);
+	};
 });
