@@ -18,8 +18,11 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-
-require_once dirname(__FILE__).'/include/config.inc.php';
+require_once dirname(__FILE__).'/include/func.inc.php';
+require_once dirname(__FILE__).'/include/defines.inc.php';
+require_once dirname(__FILE__).'/include/classes/json/CJson.php';
+require_once dirname(__FILE__).'/include/classes/user/CWebUser.php';
+require_once dirname(__FILE__).'/include/classes/core/CHttpRequest.php';
 
 $requestType = getRequest('type', PAGE_TYPE_JSON);
 if ($requestType == PAGE_TYPE_JSON) {
@@ -31,6 +34,13 @@ else {
 	$data = $_REQUEST;
 }
 
+if (is_array($data) && array_key_exists('method', $data)
+		&& in_array($data['method'], ['message.settings', 'message.get', 'zabbix.status'])) {
+	CWebUser::disableSessionExtension();
+}
+
+require_once dirname(__FILE__).'/include/config.inc.php';
+
 $page['title'] = 'RPC';
 $page['file'] = 'jsrpc.php';
 $page['type'] = detect_page_type($requestType);
@@ -97,6 +107,9 @@ switch ($data['method']) {
 		if (!$msgsettings['triggers.recovery']) {
 			$options['value'] = [TRIGGER_VALUE_TRUE];
 		}
+		if (!$msgsettings['show_suppressed']) {
+			$options['suppressed'] = $msgsettings['show_suppressed'];
+		}
 		$events = getLastEvents($options);
 
 		$sortClock = [];
@@ -120,7 +133,11 @@ switch ($data['method']) {
 						$sound = $msgsettings['sounds.'.$trigger['priority']];
 					}
 
-					$url_tr_status = 'tr_status.php?hostid='.$host['hostid'];
+					$url_problems = (new CUrl('zabbix.php'))
+						->setArgument('action', 'problem.view')
+						->setArgument('filter_hostids[]', $host['hostid'])
+						->setArgument('filter_set', '1')
+						->getUrl();
 					$url_events = (new CUrl('zabbix.php'))
 						->setArgument('action', 'problem.view')
 						->setArgument('filter_triggerids[]', $event['objectid'])
@@ -136,7 +153,7 @@ switch ($data['method']) {
 						'priority' => $priority,
 						'sound' => $sound,
 						'severity_style' => getSeverityStyle($trigger['priority'], $event['value'] == TRIGGER_VALUE_TRUE),
-						'title' => $title.' [url='.$url_tr_status.']'.CHtml::encode($host['name']).'[/url]',
+						'title' => $title.' [url='.$url_problems.']'.CHtml::encode($host['name']).'[/url]',
 						'body' => [
 							'[url='.$url_events.']'.CHtml::encode($trigger['description']).'[/url]',
 							'[url='.$url_tr_events.']'.
@@ -215,7 +232,7 @@ switch ($data['method']) {
 			]);
 
 			foreach ($triggers as $trigger) {
-				$trigger['color'] = getSeverityColor($trigger['priority']);
+				$trigger['class_name'] = getSeverityStyle($trigger['priority']);
 				$result[] = $trigger;
 			}
 		}
@@ -237,11 +254,11 @@ switch ($data['method']) {
 		switch ($data['objectName']) {
 			case 'hostGroup':
 				$hostGroups = API::HostGroup()->get([
-					'editable' => isset($data['editable']) ? $data['editable'] : false,
+					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
 					'output' => ['groupid', 'name'],
-					'search' => isset($data['search']) ? ['name' => $data['search']] : null,
-					'filter' => isset($data['filter']) ? $data['filter'] : null,
-					'limit' => isset($data['limit']) ? $data['limit'] : null
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
+					'limit' => array_key_exists('limit', $data) ? $data['limit'] : null
 				]);
 
 				if ($hostGroups) {
@@ -253,24 +270,20 @@ switch ($data['method']) {
 						$hostGroups = array_slice($hostGroups, 0, $data['limit']);
 					}
 
-					foreach ($hostGroups as $hostGroup) {
-						$result[] = [
-							'id' => $hostGroup['groupid'],
-							'name' => $hostGroup['name']
-						];
-					}
+					$result = CArrayHelper::renameObjectsKeys($hostGroups, ['groupid' => 'id']);
 				}
 				break;
 
 			case 'hosts':
 				$hosts = API::Host()->get([
-					'editable' => isset($data['editable']) ? $data['editable'] : false,
+					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
 					'output' => ['hostid', 'name'],
-					'templated_hosts' => isset($data['templated_hosts']) ? $data['templated_hosts'] : null,
-					'search' => isset($data['search']) ? ['name' => $data['search']] : null,
+					'templated_hosts' => array_key_exists('templated_hosts', $data) ? $data['templated_hosts'] : null,
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
 					'limit' => $config['search_limit']
 				]);
 
+
 				if ($hosts) {
 					CArrayHelper::sort($hosts, [
 						['field' => 'name', 'order' => ZBX_SORT_UP]
@@ -280,10 +293,37 @@ switch ($data['method']) {
 						$hosts = array_slice($hosts, 0, $data['limit']);
 					}
 
-					foreach ($hosts as $host) {
+					$result = CArrayHelper::renameObjectsKeys($hosts, ['hostid' => 'id']);
+				}
+				break;
+
+			case 'items':
+				$items = API::Item()->get([
+					'output' => ['itemid', 'hostid', 'name', 'key_'],
+					'selectHosts' => ['name'],
+					'hostids' => array_key_exists('hostid', $data) ? $data['hostid'] : null,
+					'templated' => array_key_exists('real_hosts', $data) ? false : null,
+					'webitems' => array_key_exists('webitems', $data) ? $data['webitems'] : null,
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
+					'limit' => $config['search_limit']
+				]);
+
+				if ($items) {
+					$items = CMacrosResolverHelper::resolveItemNames($items);
+					CArrayHelper::sort($items, [
+						['field' => 'name_expanded', 'order' => ZBX_SORT_UP]
+					]);
+
+					if (array_key_exists('limit', $data)) {
+						$items = array_slice($items, 0, $data['limit']);
+					}
+
+					foreach ($items as $item) {
 						$result[] = [
-							'id' => $host['hostid'],
-							'name' => $host['name']
+							'id' => $item['itemid'],
+							'name' => $item['name_expanded'],
+							'prefix' => $item['hosts'][0]['name'].NAME_DELIMITER
 						];
 					}
 				}
@@ -306,12 +346,25 @@ switch ($data['method']) {
 						$templates = array_slice($templates, 0, $data['limit']);
 					}
 
-					foreach ($templates as $template) {
-						$result[] = [
-							'id' => $template['templateid'],
-							'name' => $template['name']
-						];
+					$result = CArrayHelper::renameObjectsKeys($templates, ['templateid' => 'id']);
+				}
+				break;
+
+			case 'proxies':
+				$proxies = API::Proxy()->get([
+					'output' => ['proxyid', 'host'],
+					'search' => array_key_exists('search', $data) ? ['host' => $data['search']] : null,
+					'limit' => $config['search_limit']
+				]);
+
+				if ($proxies) {
+					CArrayHelper::sort($proxies, ['host']);
+
+					if (isset($data['limit'])) {
+						$proxies = array_slice($proxies, 0, $data['limit']);
 					}
+
+					$result = CArrayHelper::renameObjectsKeys($proxies, ['proxyid' => 'id', 'host' => 'name']);
 				}
 				break;
 
@@ -332,12 +385,7 @@ switch ($data['method']) {
 						$applications = array_slice($applications, 0, $data['limit']);
 					}
 
-					foreach ($applications as $application) {
-						$result[] = [
-							'id' => $application['applicationid'],
-							'name' => $application['name']
-						];
-					}
+					$result = CArrayHelper::renameObjectsKeys($applications, ['applicationid' => 'id']);
 				}
 				break;
 
@@ -397,7 +445,6 @@ switch ($data['method']) {
 
 			case 'users':
 				$users = API::User()->get([
-					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
 					'output' => ['userid', 'alias', 'name', 'surname'],
 					'search' => array_key_exists('search', $data)
 						? [
@@ -444,9 +491,7 @@ switch ($data['method']) {
 						$groups = array_slice($groups, 0, $data['limit']);
 					}
 
-					foreach ($groups as $group) {
-						$result[] = CArrayHelper::renameKeys($group, ['usrgrpid' => 'id']);
-					}
+					$result = CArrayHelper::renameObjectsKeys($groups, ['usrgrpid' => 'id']);
 				}
 				break;
 
