@@ -25,26 +25,27 @@ require_once dirname(__FILE__).'/include/hosts.inc.php';
 
 $page['title'] = _('Availability report');
 $page['file'] = 'report2.php';
-$page['scripts'] = ['class.calendar.js'];
+$page['scripts'] = ['class.calendar.js', 'gtlc.js'];
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'mode' =>				[T_ZBX_INT,	O_OPT,	P_SYS,			IN('0,1'),	null],
-	'hostgroupid' =>		[T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null],
-	'tpl_triggerid' =>		[T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null],
-	'triggerid' =>			[T_ZBX_INT,	O_OPT,	P_SYS|P_NZERO,	DB_ID,		null],
+	'mode' =>			[T_ZBX_INT,			O_OPT,	P_SYS,			IN('0,1'),	null],
+	'hostgroupid' =>	[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
+	'tpl_triggerid' =>	[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
+	'triggerid' =>		[T_ZBX_INT,			O_OPT,	P_SYS|P_NZERO,	DB_ID,		null],
 	// filter
-	'filter_groupid'=>		[T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null],
-	'filter_hostid' =>		[T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null],
-	'filter_rst'=>			[T_ZBX_STR,	O_OPT,	P_SYS,			null,		null],
-	'filter_set' =>			[T_ZBX_STR,	O_OPT,	P_SYS,			null,		null],
-	'filter_timesince' =>	[T_ZBX_STR,	O_OPT,	P_UNSET_EMPTY,	null,		null],
-	'filter_timetill' =>	[T_ZBX_STR,	O_OPT,	P_UNSET_EMPTY,	null,		null]
+	'filter_groupid' =>	[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
+	'filter_hostid' =>	[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
+	'filter_rst'=>		[T_ZBX_STR,			O_OPT,	P_SYS,			null,		null],
+	'filter_set' =>		[T_ZBX_STR,			O_OPT,	P_SYS,			null,		null],
+	'from' =>			[T_ZBX_RANGE_TIME,	O_OPT,	P_SYS,			null,		null],
+	'to' =>				[T_ZBX_RANGE_TIME,	O_OPT,	P_SYS,			null,		null],
 ];
 check_fields($fields);
+validateTimeSelectorPeriod(getRequest('from'), getRequest('to'));
 
 $availabilityReportMode = getRequest('mode', CProfile::get('web.avail_report.mode', AVAILABILITY_REPORT_BY_HOST));
 CProfile::update('web.avail_report.mode', $availabilityReportMode, PROFILE_TYPE_INT);
@@ -91,8 +92,6 @@ if (getRequest('triggerid') && !isReadableTriggers([getRequest('triggerid')])) {
 if (hasRequest('filter_rst')) {
 	$_REQUEST['filter_groupid'] = 0;
 	$_REQUEST['filter_hostid'] = 0;
-	$_REQUEST['filter_timesince'] = 0;
-	$_REQUEST['filter_timetill'] = 0;
 
 	if ($availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) {
 		$_REQUEST['tpl_triggerid'] = 0;
@@ -107,37 +106,44 @@ if (!hasRequest('filter_rst')) {
 	$_REQUEST['filter_hostid'] = getRequest('filter_hostid',
 		CProfile::get('web.avail_report.'.$availabilityReportMode.'.hostid', 0)
 	);
-	$_REQUEST['filter_timesince'] = getRequest('filter_timesince',
-		CProfile::get('web.avail_report.'.$availabilityReportMode.'.timesince', 0)
-	);
-	$_REQUEST['filter_timetill'] = getRequest('filter_timetill',
-		CProfile::get('web.avail_report.'.$availabilityReportMode.'.timetill', 0)
-	);
+
+	if ($availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) {
+		$_REQUEST['tpl_triggerid'] = getRequest('tpl_triggerid',
+			CProfile::get('web.avail_report.'.$availabilityReportMode.'.tpl_triggerid', 0)
+		);
+
+		$_REQUEST['hostgroupid'] = getRequest('hostgroupid',
+			CProfile::get('web.avail_report.'.$availabilityReportMode.'.hostgroupid', 0)
+		);
+	}
 }
 
 CProfile::update('web.avail_report.'.$availabilityReportMode.'.groupid', getRequest('filter_groupid', 0),
 	PROFILE_TYPE_ID
 );
-CProfile::update('web.avail_report.'.$availabilityReportMode.'.timesince', getRequest('filter_timesince', 0),
-	PROFILE_TYPE_STR
-);
-CProfile::update('web.avail_report.'.$availabilityReportMode.'.timetill', getRequest('filter_timetill', 0),
-	PROFILE_TYPE_STR
-);
 CProfile::update('web.avail_report.'.$availabilityReportMode.'.hostid', getRequest('filter_hostid', 0),
 	PROFILE_TYPE_ID
 );
 
-$config = select_config();
+if ($availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) {
+	CProfile::update('web.avail_report.'.$availabilityReportMode.'.tpl_triggerid', getRequest('tpl_triggerid', 0),
+		PROFILE_TYPE_ID
+	);
 
-if ($_REQUEST['filter_timetill'] > 0 && $_REQUEST['filter_timesince'] > $_REQUEST['filter_timetill']) {
-	zbx_swap($_REQUEST['filter_timesince'], $_REQUEST['filter_timetill']);
+	CProfile::update('web.avail_report.'.$availabilityReportMode.'.hostgroupid', getRequest('hostgroupid', 0),
+		PROFILE_TYPE_ID
+	);
 }
 
-$_REQUEST['filter_timesince'] = zbxDateToTime($_REQUEST['filter_timesince']
-	? $_REQUEST['filter_timesince'] : date(TIMESTAMP_FORMAT_ZERO_TIME, time() - SEC_PER_DAY));
-$_REQUEST['filter_timetill'] = zbxDateToTime($_REQUEST['filter_timetill']
-	? $_REQUEST['filter_timetill'] : date(TIMESTAMP_FORMAT_ZERO_TIME, time()));
+$timeselector_options = [
+	'profileIdx' => 'web.avail_report.filter',
+	'profileIdx2' => 0,
+	'from' => getRequest('from'),
+	'to' => getRequest('to')
+];
+updateTimeSelectorPeriod($timeselector_options);
+
+$config = select_config();
 
 /*
  * Header
@@ -160,10 +166,12 @@ if ($triggerData) {
 	$triggerData['hostid'] = $host['hostid'];
 	$triggerData['hostname'] = $host['name'];
 
-	$reportWidget->setControls(
+	$reportWidget->setControls((new CTag('nav', true,
 		(new CList())
 			->addItem(new CLink($triggerData['hostname'], '?filter_groupid='.$_REQUEST['filter_groupid']))
 			->addItem($triggerData['description'])
+		))
+			->setAttribute('aria-label', _('Content controls'))
 	);
 
 	$table = (new CTableInfo())
@@ -173,18 +181,19 @@ if ($triggerData) {
 		->addItem($table)
 		->show();
 }
-elseif (isset($_REQUEST['filter_hostid'])) {
-	$headerForm = (new CForm('get'))->addItem((new CList())
-		->addItem([
-			new CLabel(_('Mode'), 'mode'),
-			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-			new CComboBox('mode', $availabilityReportMode, 'submit()', [
-				AVAILABILITY_REPORT_BY_HOST => _('By host'),
-				AVAILABILITY_REPORT_BY_TEMPLATE => _('By trigger template')
+elseif (hasRequest('filter_hostid')) {
+	$reportWidget->setControls((new CForm('get'))
+		->setAttribute('aria-label', _('Main filter'))
+		->addItem((new CList())
+			->addItem([
+				new CLabel(_('Mode'), 'mode'),
+				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
+				new CComboBox('mode', $availabilityReportMode, 'submit()', [
+					AVAILABILITY_REPORT_BY_HOST => _('By host'),
+					AVAILABILITY_REPORT_BY_TEMPLATE => _('By trigger template')
+				])
 			])
-		])
-	);
-	$reportWidget->setControls($headerForm);
+	));
 
 	$triggerOptions = [
 		'output' => ['triggerid', 'description', 'expression', 'value'],
@@ -199,13 +208,14 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 	/*
 	 * Filter
 	 */
-	$filterForm = (new CFilter('web.avail_report.filter.state'))
-		->addVar('config', $availabilityReportMode)
-		->addVar('filter_timesince', date(TIMESTAMP_FORMAT, $_REQUEST['filter_timesince']))
-		->addVar('filter_timetill', date(TIMESTAMP_FORMAT, $_REQUEST['filter_timetill']));
+	$data = [
+		'filter' => [
+			'timeline' => getTimeSelectorPeriod($timeselector_options),
+			'active_tab' => CProfile::get('web.avail_report.filter.active', 1)
+		]
+	];
 
-	$filterColumn1 = new CFormList();
-	$filterColumn2 = new CFormList();
+	$filter_column = new CFormList();
 
 	// report by template
 	if ($availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) {
@@ -240,7 +250,7 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 		foreach ($groups as $group) {
 			$groupsComboBox->addItem($group['groupid'], $group['name']);
 		}
-		$filterColumn1->addRow(_('Template group'), $groupsComboBox);
+		$filter_column->addRow(_('Template group'), $groupsComboBox);
 
 		// filter template
 		$templateComboBox = new CComboBox('filter_hostid', $_REQUEST['filter_hostid'], 'javascript: submit();');
@@ -272,7 +282,7 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 
 			$templateComboBox->addItem($template['templateid'], $template['name']);
 		}
-		$filterColumn1->addRow(_('Template'), $templateComboBox);
+		$filter_column->addRow(_('Template'), $templateComboBox);
 
 		// filter trigger
 		$triggerComboBox = new CComboBox('tpl_triggerid', getRequest('tpl_triggerid', 0), 'javascript: submit()');
@@ -305,7 +315,7 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 			unset($triggerOptions['filter']['templateid']);
 		}
 
-		$filterColumn1->addRow(_('Template trigger'), $triggerComboBox);
+		$filter_column->addRow(_('Template trigger'), $triggerComboBox);
 
 		// filter host group
 		$hostGroupsComboBox = new CComboBox('hostgroupid', getRequest('hostgroupid', 0), 'javascript: submit()');
@@ -344,14 +354,13 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 			unset($triggerOptions['groupids']);
 		}
 
-		$filterColumn1->addRow(_('Host group'), $hostGroupsComboBox);
+		$filter_column->addRow(_('Host group'), $hostGroupsComboBox);
 	}
-
 	// report by host
 	elseif ($availabilityReportMode == AVAILABILITY_REPORT_BY_HOST) {
 		// filter host group
 		$groupsComboBox = (new CComboBox('filter_groupid', $_REQUEST['filter_groupid'], 'javascript: submit();'))
-				->setAttribute('autofocus', 'autofocus');
+			->setAttribute('autofocus', 'autofocus');
 		$groupsComboBox->addItem(0, _('all'));
 
 		$groups = API::HostGroup()->get([
@@ -367,7 +376,7 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 		foreach ($groups as $group) {
 			$groupsComboBox->addItem($group['groupid'], $group['name']);
 		}
-		$filterColumn1->addRow(_('Host group'), $groupsComboBox);
+		$filter_column->addRow(_('Host group'), $groupsComboBox);
 
 		// filter host
 		$hostsComboBox = new CComboBox('filter_hostid', $_REQUEST['filter_hostid'], 'javascript: submit();');
@@ -397,7 +406,7 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 		foreach ($hosts as $host) {
 			$hostsComboBox->addItem($host['hostid'], $host['name']);
 		}
-		$filterColumn1->addRow(_('Host'), $hostsComboBox);
+		$filter_column->addRow(_('Host'), $hostsComboBox);
 
 		// trigger options
 		$triggerOptions['groupids'] = $filter_groupids;
@@ -406,65 +415,71 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 		}
 	}
 
-	// filter period
-	$filterColumn2->addRow(_('From'), createDateSelector('filter_timesince', $_REQUEST['filter_timesince'], 'filter_timetill'));
-	$filterColumn2->addRow(_('To'), createDateSelector('filter_timetill', $_REQUEST['filter_timetill'], 'filter_timesince'));
-
-	$filterForm->addColumn($filterColumn1);
-	$filterForm->addColumn($filterColumn2);
-
-	$reportWidget->addItem($filterForm);
+	$reportWidget->addItem(
+		(new CFilter())
+			->setProfile($data['filter']['timeline']['profileIdx'])
+			->setActiveTab($data['filter']['active_tab'])
+			->addFormItem((new CVar('config', $availabilityReportMode))->removeId())
+			->addTimeSelector($data['filter']['timeline']['from'], $data['filter']['timeline']['to'], ZBX_DATE_TIME)
+			->addFilterTab(_('Filter'), [$filter_column])
+	);
 
 	/*
 	 * Triggers
 	 */
-	$triggerTable = (new CTableInfo())
-		->setHeader([
-			($_REQUEST['filter_hostid'] == 0 || $availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) ? _('Host') : null,
-			_('Name'),
-			_('Problems'),
-			_('Ok'),
-			_('Graph')
-		]);
+	$triggerTable = (new CTableInfo())->setHeader([_('Host'), _('Name'), _('Problems'), _('Ok'), _('Graph')]);
 
 	$triggers = API::Trigger()->get($triggerOptions);
 
-	if (getRequest('filter_hostid') == 0 || $availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) {
-		foreach ($triggers as &$trigger) {
-			$trigger['host_name'] = $trigger['hosts'][0]['name'];
-		}
-		unset($trigger);
+	foreach ($triggers as &$trigger) {
+		$trigger['host_name'] = $trigger['hosts'][0]['name'];
+	}
+	unset($trigger);
 
-		CArrayHelper::sort($triggers, ['host_name', 'description']);
-	}
-	else {
-		CArrayHelper::sort($triggers, ['description']);
-	}
+	CArrayHelper::sort($triggers, ['host_name', 'description']);
 
 	$paging = getPagingLine($triggers, ZBX_SORT_UP, new CUrl('report2.php'));
 
 	foreach ($triggers as $trigger) {
-		$availability = calculateAvailability($trigger['triggerid'], getRequest('filter_timesince'),
-			getRequest('filter_timetill')
+		$availability = calculateAvailability($trigger['triggerid'], $data['filter']['timeline']['from_ts'],
+			$data['filter']['timeline']['to_ts']
 		);
 
 		$triggerTable->addRow([
-			($_REQUEST['filter_hostid'] == 0 || $availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE)
-				? $trigger['host_name'] : null,
+			$trigger['host_name'],
 			new CLink($trigger['description'],
 				(new CUrl('zabbix.php'))
 					->setArgument('action', 'problem.view')
 					->setArgument('filter_triggerids[]', $trigger['triggerid'])
 					->setArgument('filter_set', '1')
 			),
-			$availability['true'] == 0 ? '' : (new CSpan(sprintf('%.4f%%', $availability['true'])))->addClass(ZBX_STYLE_RED),
-			$availability['false'] == 0 ? '' : (new CSpan(sprintf('%.4f%%', $availability['false'])))->addClass(ZBX_STYLE_GREEN),
+			($availability['true'] == 0)
+				? ''
+				: (new CSpan(sprintf('%.4f%%', $availability['true'])))->addClass(ZBX_STYLE_RED),
+			($availability['false'] == 0)
+				? ''
+				: (new CSpan(sprintf('%.4f%%', $availability['false'])))->addClass(ZBX_STYLE_GREEN),
 			new CLink(_('Show'), 'report2.php?filter_groupid='.$_REQUEST['filter_groupid'].
-				'&filter_hostid='.$_REQUEST['filter_hostid'].'&triggerid='.$trigger['triggerid'])
+				'&filter_hostid='.$_REQUEST['filter_hostid'].'&triggerid='.$trigger['triggerid']
+			)
 		]);
 	}
 
-	$reportWidget->addItem([$triggerTable, $paging])
+	$obj_data = [
+		'id' => 'timeline_1',
+		'domid' => 'avail_report',
+		'loadSBox' => 0,
+		'loadImage' => 0,
+		'dynamic' => 0,
+		'mainObject' => 1
+	];
+	zbx_add_post_js(
+		'timeControl.addObject("avail_report", '.zbx_jsvalue($data['filter']).', '.zbx_jsvalue($obj_data).');'
+	);
+	zbx_add_post_js('timeControl.processObjects();');
+
+	$reportWidget
+		->addItem([$triggerTable, $paging])
 		->show();
 }
 
